SUBROUTINE avectx( mot, longm, long, ideb, iend, kode, nomv, lnom, &
                   incr, minx, maxx, imin, imax, icol, jcol, lerr, &
                   dx, rx, ix, lx, cx )
!***********************************************************************
!     - avectx -
!     affecte une liste de valeurs lues au vecteur dx/rx/ix/lx/cx
!     et retourne les indices extremes trouves correspondants.
!     le vecteur .x est l'analogue a une dimension d'un vecteur
!     dimensionne (1:incr,minx:*).
!     l'indice a partir duquel les valeurs sont stockees peut etre
!     precise, soit par un nombre, soit par un nom. dans ce cas
!     l'indice exact dans le vecteur .x est alors donne par:
!     indice = minx + (k-minx)*incr
!     ou k est l'indice indique (ou celui, dans nomv, du nom indique).
!***********************************************************************
!
!     MOT   - ENTREE. Le mot cle                              - CAR
!     LONGM - ENTREE. Sa longueur complete                    - ENTIER
!     LONG  - ENTREE. Sa longueur en abreviation maximum      - ENTIER
!     IDEB  - ENTREE. Col. de debut de la zone analysee       - ENTIER
!     IEND  - ENTREE. Col. de fin        "                    - ENTIER
!     KODE  - ENTREE. Code du type de vecteur :               - ENTIER
!                        1 = REEL DOUBLE PRECISION
!                        2 = REEL SIMPLE PRECISION
!                        3 = ENTIER
!                        4 = LOGIQUE
!                        5 = CARACTERE
!     NOMV  - ENTREE. Les noms des indices                    - CAR
!                     (inutilise si lnom est .false.)
!     LNOM  - ENTREE. .true. si NOMV est utilisable,          - LOGIQUE
!                     .false. sinon
!     INCR  - ENTREE. Increment pour le vecteur .X            - ENTIER
!     MINX  - ENTREE. Indice minimum pour le dimensionnement  - ENTIER
!                     des vecteurs NOMV et .X
!     MAXX  - ENTREE. Indice maximum pour le dimensionnement  - ENTIER
!                     des vecteurs .X
!     IMIN  - SORTIE. Plus petit indice du vecteur trouve     - ENTIER
!     IMAX  - SORTIE. Plus grand indice du vecteur trouve     - ENTIER
!     ICOL  - SORTIE. Col. de debut de la derniere occurrence - ENTIER
!                     du mot cle
!     JCOL  - SORTIE. Col. de fin de phrase de la derniere    - ENTIER
!                     occurrence du mot cle
!     LERR  - SORTIE. .true. si une erreur est detectee       - LOGIQUE
!     DX    - SORTIE. Vecteur retourne (optionnel)            - REAL*8
!     RX    - SORTIE. Vecteur retourne (optionnel)            - REEL
!     IX    - SORTIE. Vecteur retourne (optionnel)            - ENTIER
!     LX    - SORTIE. Vecteur retourne (optionnel)            - LOGIQUE
!     CX    - SORTIE. Vecteur retourne (optionnel)            - CAR
!
!     APPELLE DNUM, INDVEY, INUM, KARIN, POSITM
!-----------------------------------------------------------------------

   USE carl
   USE flag_msg

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN)  :: longm, long, ideb, iend, kode, incr, minx, maxx
   LOGICAL, INTENT(IN)  :: lnom
   CHARACTER(LEN=*), INTENT(IN) :: mot
   CHARACTER(LEN=*), DIMENSION(minx:*), INTENT(IN) :: nomv
   INTEGER, INTENT(OUT) :: imin, imax, icol, jcol
   LOGICAL, INTENT(OUT) :: lerr
   REAL(KIND=8)    , DIMENSION(minx:maxx), INTENT(OUT), OPTIONAL :: dx
   REAL            , DIMENSION(minx:maxx), INTENT(OUT), OPTIONAL :: rx
   INTEGER         , DIMENSION(minx:maxx), INTENT(OUT), OPTIONAL :: ix
   LOGICAL         , DIMENSION(minx:maxx), INTENT(OUT), OPTIONAL :: lx
   CHARACTER(LEN=*), DIMENSION(minx:maxx), INTENT(OUT), OPTIONAL :: cx

   !  Declarations locales
   !  --------------------

   REAL(KIND=8) :: a, b, c, dval
   INTEGER :: i, ii, ijk, irep, ival, i1
   INTEGER :: ja, jb, jc, kol, kola, kolz, kol1, kol2, kol3, kol4, k3
   INTEGER :: larg, ll, maxv, minv, nrep
   LOGICAL :: lgl, lexit
   CHARACTER(LEN=1) :: car3
   CHARACTER(LEN=4) :: numero
   CHARACTER(LEN=255) :: ca, nomb
   INTEGER, DIMENSION(1) :: iv

   INCLUDE 'outrngp.'
   INCLUDE 'avectxp.'

   !--------------------------------------------------------------------

   imin = maxx + 1
   imax = minx - 1
   icol = 0
   jcol = 0
   lerr = .false.

   minv = minx
   maxv = (maxx + (incr-1)*(minx-1) ) / incr
   i1   = ideb

   ! Boucle sur le mot cle
   ! ---------------------
   do
      call positm( mot, longm, long, i1, iend, kol, kol1, kol2, lerr )
      if ( lerr ) return
      if ( kol <= 0 ) return
      icol = kol
      jcol = kol2

      ! Determination de l'indice de debut du vecteur (minx a priori)
      ! -------------------------------------------------------------
      ii = minx
      if ( car(kol1:kol1) /= '=' ) then
         ! Donc c'est par1: l'indice est donne explicitement
         if ( lnom ) then
            kol = kol1
            call indvey( mot, nomv, minv, maxv, kol, jcol, i, kol1, iv, 1, lerr )
            if ( lerr ) return
            if ( i > 0 ) ii = iv(1)
         else
            call inum( kol1+1, jcol, 1, par2, 1, par2, 1, .true., kol, lgl, ii, lerr )
            if ( lerr ) return
            ! Quelques verifications
            kol1 = kol + 1
            if ( ii < minv .or. ii > maxv ) then
               write(l6,*) mrker1
               write(l6,11) typer, car(icol:min(icol+64,jcol))
               write(l6,*)  outrng, mot
               write(l6,*) mrker2
               lerr = .true.
               return
            end if
            if ( car(kol1:kol1) /= '=' ) then
               kola = max( kol-25, 1 )
               kolz = min( kol+25, ncar )
               write(l6,*) mrker1
               write(l6,11) typer, car(kola:kolz), flag(:kol-kola)
               write(l6,33) car(kol1:kol1), par2
               write(l6,*) mrker2
               lerr = .true.
               return
            end if
         end if
         ii = minx + (ii-minx)*incr
      end if
      if ( ii < imin ) imin = ii
      ii = ii - 1

      ! kol1 = col. du signe "=" precedant la liste
      ! jcol = col. de la fin de liste (sep2)
      ! ii   = indice de debut pour le chargement du vecteur
      ! kol3 = col. du 1er separateur d'elements dans la liste
      ! car3 = caractere attendu avant l'element analyse

      car3 = '='

      if ( kode < 5 ) then

         ! Analyse d'un vecteur numerique ou logique
         ! -----------------------------------------

         v_num: do      ! Boucle sur les elements de la liste

            kol3 = index( car(kol1+1:jcol), sep1 ) + kol1
            if ( kol3 > kol1 ) then      ! Plusieurs valeurs
               lexit = .false.
            else                         ! Une seule valeur (ou boucle)
               kol3 = jcol
               lexit = .true.
            end if
            nrep = 1
            ! Boucle ?
            kol = index( car(kol1+1:kol3), to(:ncto) ) + kol1
            if ( kol > kol1 ) then      ! Boucle (a>b:c)
               c  = 1.0_8
               k3 = kol3 - 1
               if ( car(k3:k3) == par2 ) k3 = k3 - 1
               call dnum( k3, kol, -1, by, ncby, to, ncto, .true., ll, lgl, b, lerr )
               if ( lerr ) return
               if ( lgl ) then
                  c = b
                  if ( c == 0.0_8 ) c = 1.0_8
                  call dnum( ll-1, kol, -1, to, ncto, to, ncto, .true., ll, lgl, b, lerr )
                  if ( lerr ) return
               end if
               call dnum( ll-1, kol1, -1, par1, 1, car3, 1, .true., ll, lgl, a, lerr )
               if ( lerr ) return
               ! Correction eventuelle du pas
               if ( a > b .and. c > 0.0_8 ) c = -c
               ! Repetition n* ?
               if ( lgl .and. car(ll-1:ll-1) == repet ) then
                  call inum( ll-2, kol1, -1, car3, 1, car3, 1, .true., ll, lgl, nrep, lerr )
                  if ( lerr ) return
               end if
            else
               ! Structure simple a ou n*a
               call dnum( kol3-1, kol1, -1, repet, 1, car3, 1, .true., ll, lgl, a, lerr )
               if ( lerr ) return
               if ( lgl ) then
                  call inum( ll-1, kol1, -1, car3, 1, car3, 1, .true., ll, lgl, nrep, lerr )
                  if ( lerr ) return
               end if
               b = a
               c = 1.0_8
            end if
            ! Mise a jour du vecteur
            select case ( kode )
            case ( 1 )      ! Vecteur double precision
               do irep = 1, nrep
                  do dval = a, b, c
                     ii     = ii + 1
                     dx(ii) = dval
                     if ( ii >= maxx ) exit v_num
                  end do
               end do
            case ( 2 )      ! Vecteur simple precision
               do irep = 1, nrep
                  do dval = a, b, c
                     ii     = ii + 1
                     rx(ii) = dval
                     if ( ii >= maxx ) exit v_num
                  end do
               end do
            case ( 3 )      ! Vecteur entier
               do irep = 1, nrep
                  do ival = nint(a), nint(b), nint(c)
                     ii     = ii + 1
                     ix(ii) = ival
                     if ( ii >= maxx ) exit v_num
                  end do
               end do
            case ( 4 )      ! Vecteur logique
               do irep = 1, nrep
                  do ival = nint(a), nint(b), nint(c)
                     ii     = ii + 1
                     lx(ii) = ival /= 0
                     if ( ii >= maxx ) exit v_num
                  end do
               end do
            end select

            ! Mise a jour pour analyse de l'element suivant
            kol1 = kol3
            car3 = sep1
            if ( lexit ) exit

         end do v_num

      else

         ! Analyse d'un vecteur caractere
         ! ------------------------------

         v_char: do     ! Boucle sur les elements de la liste

            kol4 = kol1
            do
               kol3 = index( car(kol4+1:jcol), sep1 ) + kol4
               if ( kol3 > kol4 ) then
                  if ( com(kol3) ) then
                     kol4 = kol3
                     cycle
                  end if
                  ! Plusieurs valeurs
                  lexit = .false.
               else
                  ! Une seule valeur
                  kol3 = jcol
                  lexit = .true.
               end if
               exit
            end do
            if ( kol3 <= (kol4+1) ) then
               kola = max( kol4-25, icol )
               kolz = min( kol4+25, ncar )
               write(l6,*) mrker1
               write(l6,11)  typer, car(kola:kolz), flag(:kol4-kola+1)
               write(l6,555) car(kol4:kol4)
               write(l6,*) mrker2
               lerr = .true.
               return
            end if
            kola = max( kol3-65, icol )
            kol4 = kol1
         10 continue
            kol  = index( car(kol4+1:kol3-1), to(:ncto) ) + kol4
            if ( kol > kol4 ) then
               if ( com(kol) ) then
                  kol4 = kol
                  goto 10
               end if
               kol = kol + ncto
               ! C'est une boucle sur les noms
               ! Analyse du dernier terme: nomb suivi du numero jb
               jc = 1
               k3 = kol3 - 1
               if ( car(k3:k3) == par2 ) k3 = k3 - 1
               call inum( k3, kol, -1, ' ', 1, ' ', 1, .false., ll, lgl, jb, lerr )
               if ( lerr ) return
               if ( .not. com(ll) .and. car(ll-ncby+1:ll) == by(:ncby) ) then
                  jc = jb
                  if ( jc == 0 ) jc = 1
                  ll = ll - ncby + 1
                  call inum( ll-1, kol, -1, ' ', 1, ' ', 1, .false., ll, lgl, jb, lerr )
                  if ( lerr ) return
               end if
               if ( ll+1 <= kol ) then
                  kola = max( ll-25, icol )
                  kolz = min( ll+25, ncar )
                  write(l6,*) mrker1
                  write(l6,11) typer, car(kola:kolz), flag(:ll-kola)
                  write(l6,111)
                  write(l6,*)  car(kol1:kol3)
                  write(l6,*) mrker2
                  lerr = .true.
                  return
               end if
               nomb = car(kol:ll)
               larg = ll - kol + 1
               ! Analyse du 1er terme: nomb suivi de ja
               call inum( kol-ncto-1, kol1-1, -1, ' ', 1, ' ',1, .false., ll, lgl, ja, lerr )
               if ( lerr ) return
               if ( car(ll-larg+1:ll) /= nomb ) then
                  kolz = min( kol3, kola+64 )
                  write(l6,*) mrker1
                  write(l6,11) typer, car(kola:kolz)
                  write(l6,222)
                  write(l6,*)  car(kol1:kol3)
                  write(l6,*) mrker2
                  lerr = .true.
                  return
               end if
               ! Verif et repetition ?
               ll   = ll - larg - 1
               nrep = 1
               if ( ll+1 /= kol1 ) then
                  if ( car(ll+1:ll+1) /= par1 ) then
                     kola = max( ll-25, icol )
                     kolz = min( ll+25, ncar )
                     write(l6,*) mrker1
                     write(l6,11) typer, car(kola:kolz), flag(:ll+1-kola)
                     write(l6,333)
                     write(l6,*)  car(kol1:kol3)
                     write(l6,*) mrker2
                     lerr = .true.
                     return
                  else
                     if ( car(ll:ll) == repet ) then
                        call inum( ll-1, kol1, -1, car3, 1, car3, 1, .false., ll, lgl, nrep, lerr )
                        if ( lerr ) return
                     end if
                     if ( ll /= kol1 ) then
                        kola = max( ll-25, icol )
                        kolz = min( ll+25, ncar )
                        write(l6,*) mrker1
                        write(l6,11) typer, car(kola:kolz), flag(:ll-kola)
                        write(l6,444)
                        write(l6,*)  car(kol1:kol3)
                        write(l6,*) mrker2
                        lerr = .true.
                        return
                     end if
                  end if
               end if
               ! Correction eventuelle du pas
               if ( ja > jb .and. jc > 0 ) jc = -jc
               ! Construction des noms: nomb//ja a jb
               do irep = 1, nrep
                  do i = ja, jb, jc
                     ii = ii + 1
                     call karin( i, numero, ijk )
                     cx(ii) = nomb(:larg) // numero
                     if ( ii >= maxx ) exit v_char
                  end do
               end do

            else
               ! Pas de boucle
               ! Repetition ?
               kol = index( car(kol1+1:kol3-1), repet ) + kol1
               if ( kol > kol1 .and. .not. com(kol) ) then
                  call inum( kol-1, kol1+1, -1, ' ', 1, ' ',1, .false., ll, lgl, nrep, lerr )
                  if ( lerr ) return
                  ca = car(kol+1:kol3-1)
               else
                  nrep = 1
                  ca   = car(kol1+1:kol3-1)
               end if
               do irep = 1, nrep
                  ii     = ii + 1
                  cx(ii) = ca
                  if ( ii >= maxx ) exit v_char
               end do

            end if

            ! Mise a jour pour analyse de l'element suivant
            kol1 = kol3
            car3 = sep1
            if ( lexit ) exit

         end do v_char

      end if

      if ( ii > imax ) imax = ii

      ! Effacement de la zone analysee
      if ( car(icol:icol) == sep2 ) then
         car(icol+1:jcol-1) = ' '
      else
         car(icol:jcol-1) = ' '
      end if

      ! D'autres occurrences du mot cle ? i1=col de debut d'exploration
      i1 = jcol
   end do

   900 continue   ! Retour en erreur
   lerr = .true.

   INCLUDE 'anaf11.'
   INCLUDE 'anaf33.'
   INCLUDE 'avectxyf.'

END SUBROUTINE avectx
