SUBROUTINE avectz( mot, longm, long, ideb, iend, nomv, minv, maxv, &
                   ix, minx, maxx, nx, icol, jcol, lerr )
!***********************************************************************
!     - avectz -
!     retourne dans vecteur ix(minx:maxx) des valeurs entieres
!     donnees directement (minv et maxv sont les bornes des valeurs
!     admissibles) ou par l'intermediaire de noms appartenant a
!     la liste de noms nomv(minv:maxv) fournie en entree; dans ce
!     cas le nom est remplace par son indice dans nomv(minv:maxv)
!     nx retourne le nombre de valeurs trouvees (retourne 0 si le
!     mot-cle n'est pas trouve)
!
!     Remarque: s'il y a plusieurs occurrence du mot-cle, c'est la
!               derniere qui est prise en compte
!***********************************************************************
!
!     MOT   - ENTREE. LE MOT CLE                              - CAR
!     LONGM - ENTREE. SA LONGUEUR COMPLETE                    - ENTIER
!     LONG  - ENTREE. SA LONGUEUR EN ABREVIATION MAXIMUM      - ENTIER
!     IDEB  - ENTREE. COL. DE DEBUT DE LA ZONE ANALYSEE       - ENTIER
!     IEND  - ENTREE. COL. DE FIN        "                    - ENTIER
!     NOMV  - ENTREE. LES NOMS POSSIBLES                      - CAR
!     MINV  - ENTREE. INDICE MINIMUM POUR LE DIMENSIONNEMENT  - ENTIER
!                     DE NOMV
!     MAXV  - ENTREE. INDICE MAXIMUM ...                      - ENTIER
!     IX    - SORTIE. VECTEUR RETOURNE                        - ENTIER
!     MINX  - ENTREE. INDICE MINIMUM POUR LE DIMENSIONNEMENT  - ENTIER
!                     DU VECTEUR IX
!     MAXX  - ENTREE. INDICE MAXIMUM ...                      - ENTIER
!     NX    - SORTIE. NOMBRE DE VALEURS STOCKEES DANS IX      - ENTIER
!     ICOL  - SORTIE. COL. DE DEBUT DE LA DERNIERE OCCURRENCE - ENTIER
!                     DU MOT CLE
!     JCOL  - SORTIE. COL. DE FIN DE PHRASE DE LA DERNIERE    - ENTIER
!                     OCCURRENCE DU MOT CLE
!     LERR  - SORTIE. .TRUE. SI UNE ERREUR EST DETECTEE       - LOGIQUE
!
!     APPELLE INUM, ITEMIX
!-----------------------------------------------------------------------

   USE carl
   USE flag_msg

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN)  :: longm, long, ideb, iend, minv, maxv, minx, maxx
   CHARACTER(LEN=*), INTENT(IN) :: mot
   CHARACTER(LEN=*), DIMENSION(minv:maxv), INTENT(IN) :: nomv
   INTEGER, INTENT(OUT) :: nx, icol, jcol
   LOGICAL, INTENT(OUT) :: lerr
   INTEGER, DIMENSION(minx:maxx), INTENT(OUT) :: ix

   !  Declarations locales
   !  --------------------

   INTEGER :: ia, ib, ic, irep, ival, iz, i0x, i1, j1
   INTEGER :: kol, kola, kolr, kolx, kolz, kol3, kol4
   INTEGER :: ll, lonv, maxnx, nrep
   LOGICAL :: lgl, lexit, scan
   CHARACTER(LEN=1) :: car3, car4

   INCLUDE 'fkwdp.'
   INCLUDE 'notabbp.'
   INCLUDE 'notfolp.'
   INCLUDE 'avectzp.'

   !--------------------------------------------------------------------

   nx   = 0
   icol = 0
   jcol = 0
   lerr = .false.

   lonv  = len( nomv(1) )      ! Longueur de nomv
   i0x   = minx - 1
   maxnx = maxx - minx + 1
   i1    = ideb
   scan = .true.

   ! Boucle sur le mot cle
   ! ---------------------
   do
      ! Positionnement sur le mot cle
      if ( iend < i1 ) return     ! Fin
      kol = index( car(i1:iend), mot(:long) ) + i1 - 1
      if ( kol < i1 ) return      ! Fin

      i1 = kol + long
      if ( com(kol) .or. com(kol+1) ) cycle
      icol = kol
      kolz = min( icol+64, ncar )
      ! Etude de l'abreviation eventuelle
      do j1 = long+1, longm
         i1 = icol - 1 + j1
         if ( (car(i1:i1) == '=') .and. .not. com(i1) ) then
            scan = .false.
            exit
         end if
         if ( car(i1:i1) /= mot(j1:j1) ) then
            write(l6,*) mrker1
            write(l6,11) typer, car(icol:kolz)
            write(l6,*)  car(icol:i1), notabb, mot(:longm)
            write(l6,*) mrker2
            lerr = .true.
            return
         end if
      end do
      if ( scan ) then
         i1 = icol + longm
         if ( com(i1) .or. (car(i1:i1) /= '=') ) then
            kola = max( i1-25, icol )
            kolz = min( i1+25, ncar )
            write(l6,*) mrker1
            write(l6,11) typer, car(kola:kolz), flag(:i1-kola)
            write(l6,*)  car(icol:i1-1), notfol, '='
            write(l6,*) mrker2
            lerr = .true.
            return
         end if
      end if

      ! Recherche de la fin de phrase
      iz = i1
      do
         if ( iz+1 > iend ) then
            write(l6,*) mrker1
            write(l6,11) typer, car(icol:kolz)
            write(l6,22) sep2
            write(l6,*)  fkwd, mot
            write(l6,*) mrker2
            lerr = .true.
            return
         end if
         jcol = index( car(iz+1:iend), sep2 ) + iz
         if ( jcol <= iz ) then
            write(l6,*) mrker1
            write(l6,11) typer, car(icol:kolz)
            write(l6,22) sep2
            write(l6,*)  fkwd, mot
            write(l6,*) mrker2
            lerr = .true.
            return
         end if
         if ( .not. com(jcol) ) exit
         iz = jcol + 1   ! sep2 etant en "commentaire" on recommence
      end do

      ! Analyse de la liste
      ! -------------------
      nx   = 0
      i1   = i1 + 1
      kolx = i1
      do             ! Boucle sur les elements de la liste
         kol3 = index( car(kolx:jcol), sep1 ) + kolx - 1
         if ( kol3 >= kolx ) then
            ! Si le separateur fait partie du nom on recommence
            if ( com(kol3) ) then
               kolx = kol3 + 1
               cycle
            end if
            ! Plusieurs valeurs ou boucles
            car3 = sep1
            lexit = .false.
         else
            ! Une seule valeur ou boucle
            kol3 = jcol
            car3 = sep2
            lexit = .true.
         end if

         kol4 = kol3
         car4 = car3
         nrep = 1
         ! Repetition n* ?
         kolr = index( car(i1:kol3), repet ) + i1 - 1
         if ( .not. com(i1) .and. .not. com(kolr) .and. kolr >= i1 ) then
            call inum( i1, kolr, 1, repet, 1, repet, 1, .true., ll, lgl, nrep, lerr )
            if ( lerr ) return
            i1 = ll + 1
            if ( car(i1:i1) == par1 ) then
               i1   = i1 + 1
               kol4 = kol3 - 1
               car4 = par2
            end if
         end if
         call itemix( mot, i1, kol4, car4, nomv, minv, maxv, lonv, ia, ib, ic, lerr )
         if ( lerr ) return
         ! Stockage des indices
         do irep = 1, nrep
            do ival = ia, ib, ic
               nx = nx + 1
               if ( nx > maxnx ) then
                  write(l6,*) mrker1
                  write(l6,11) typer, car(icol:min(jcol,kolz))
                  write(l6,111)
                  write(l6,*)  fkwd, mot(:long)
                  write(l6,*) mrker2
                  lerr = .true.
                  return
               end if
               ix(i0x+nx) = ival
            end do
         end do
         if ( lexit ) exit
         ! Suite de la liste
         i1   = kol3 + 1
         kolx = i1
      end do

      ! Effacement de la zone analysee
      if ( car(icol:icol) == sep2 ) then
         car(icol+1:jcol-1) = ' '
      else
         car(icol:jcol-1) = ' '
      end if

      ! D'autres occurrences du mot cle ? i1=col de debut d'exploration
      i1 = jcol
   end do

   lerr = .true.

   INCLUDE 'anaf11.'
   INCLUDE 'anaf22.'
   INCLUDE 'avectzf.'

END SUBROUTINE avectz
