SUBROUTINE dnum( i1, i2, ipas, car1, nc1, car2, nc2, code, ind, lc1, dval, lerr )

!***********************************************************************
!     - dnum -
!     analyse d'un nombre real*8. version "lecture en memoire".
!     la valeur est copiee en memoire dans mem puis relue par le format
!     fmtc -> meme valeur qu'avec lecture fortran classique.
!
!     lecture gauche->droite si pas=1, droite->gauche si ipas=-1.
!     donne le nombre dval (real*8) situe entre col. i1 et i2 de
!     car et, si code = .true., la colonne ind :
!        du dernier caractere du mot suivant   dval, si ipas = 1
!        du premier caractere du mot precedant dval, si ipas =-1
!     quand code=.true. ce mot doit exister et etre car1(:nc1) ou
!     car2(:nc2) et si c'est car1(:nc1) alors lc1=.true. en retour
!     il faut i1<i2 si ipas=1, i1>i2 si ipas=-1 .
!     si aucun nombre trouve retour en erreur par *
!     retour * si erreur
!
!     maxc = nombre maximum de caracteres pour l'ecriture du nombre
!     peut interpreter les double precision avec exposant e ou d
!***********************************************************************
!
!     I1    - ENTREE. 1ERE BORNE DE LA ZONE EXAMINEE          - ENTIER
!     I2    - ENTREE. 2EME BORNE DE LA ZONE EXAMINEE          - ENTIER
!     IPAS  - ENTREE. SENS DE LECTURE :                       - ENTIER
!                        1 = GAUCHE -> DROITE (=> I1<=I2)
!                       -1 = DROITE -> GAUCHE (=> I1>=I2)
!     CAR1  - ENTREE. 1ER  MOT "CIBLE"                        - CAR
!     NC1   - ENTREE. SA LONGUEUR                             - ENTIER
!     CAR2  - ENTREE. 2EME MOT "CIBLE"                        - CAR
!     NC2   - ENTREE. SA LONGUEUR                             - ENTIER
!     CODE  - ENTREE. SI CODE=.TRUE. ALORS LE MOT SUIVANT/    - LOGIQUE
!                     PRECEDANT DVAL DOIT EXISTER ET ETRE
!                     CAR1(:NC1) OU CAR2(:NC2)
!     IND   - SORTIE. LA COLONNE DU DERNIER/PREMIER CARACTERE - ENTIER
!                     SUIVANT/PRECEDANT DVAL (SELON QUE IPAS
!                     VAUT 1 OU -1)
!                     SI CODE=.FALSE., C'EST LA COLONNE
!                     SUIVANT/PRECEDANT DVAL
!     LC1   - SORTIE. .TRUE. QUAND CODE=.TRUE. ET LE MOT      - LOGIQUE
!                     "CIBLE" EST(CAR1:NC1)
!     DVAL  - SORTIE. LA VALEUR TROUVEE (ERREUR SI AUCUNE)    - REAL*8
!     LERR  - SORTIE. .TRUE. SI UNE ERREUR EST DETECTEE       - LOGIQUE
!-----------------------------------------------------------------------

   USE carl
   USE flag_msg
   USE is_digit

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN)  :: i1, i2, ipas, nc1, nc2
   LOGICAL, INTENT(IN)  :: code
   CHARACTER(LEN=*), INTENT(IN) :: car1, car2
   INTEGER, INTENT(OUT) :: ind
   LOGICAL, INTENT(OUT) :: lc1, lerr
   REAL(KIND=8), INTENT(OUT) :: dval

   !  Declarations locales
   !  --------------------

   INTEGER :: i, ii1, ik1, j, jnd, k, kola, kolz, m, max12, min12
   LOGICAL :: lkar, lpoint, lexp, scan
   INTEGER, PARAMETER :: imns=IACHAR('-'), ipls=IACHAR('+'), ipnt=IACHAR('.')
   INTEGER, PARAMETER :: ieex=IACHAR('E'), idex=IACHAR('D')
   INTEGER, PARAMETER :: jeex=IACHAR('e'), jdex=IACHAR('d')

   INTEGER, PARAMETER :: maxc=75, maxc1=maxc+1
   ! maxc = nombre maximum de caracteres pour l'ecriture d'un nombre
   !        (le format fmtc doit y etre adapte)
   CHARACTER(LEN=maxc) :: mem
   CHARACTER(LEN=8) :: fmtc='(F75.0) '
   INTEGER, DIMENSION(-1:1) :: ip=(/ -1, -1, maxc /), maxi=(/ 0, 0, 1 /)
   LOGICAL, DIMENSION(-1:1) :: lg=(/ .false., .false., .true. /)

   INCLUDE 'illglp.'
   INCLUDE 'dnump.'
   INCLUDE 'msgernp.'

   !--------------------------------------------------------------------

   max12 = max( i1, i2 )
   min12 = min( i1, i2 )
   kola  = max( 1, min12-17 )
   if ( (i2-i1)*ipas < 0 ) then
      write(l6,*) mrker1
      write(l6,11) typer, car(kola:min(max12,kola+64)), msgn0
      write(l6,*) mrker2
      lerr = .true.
      return
   end if

   lerr   = .false.
   mem    = ' '
   ii1    = i1
   k      = -ipas
   m      = ip(ipas)
   lexp   = .false.
   lpoint = .false.
   lkar   = .true.
   scan   = .true.

   ! Boucle sur les caracteres de la zone
   ! ------------------------------------
   do i = i1, i2, ipas
      j = iachar( car(i:i) )
      if ( ldigit(j) .and. lkar ) then  ! Le caractere est un chiffre
         m = m + k
      else
         if ( (j == imns .or. j == ipls) .and. lkar .and. i-ii1 < maxi(ipas) ) then
            lkar = lg(ipas)   ! Signe + ou -
         else
            if ( .not. lpoint .and. j == ipnt ) then   ! Point decimal
               lpoint = .true.
            else
               ! Prise en compte de l'exposant s'il existe
               if ( .not. lexp .and. (j == ieex .or. j == idex .or. j == jeex .or. j == jdex) ) then
                  lexp = .true.
                  lkar = .true.
                  ii1  = i + ipas
               else
                  ind = i
                  scan = .false.
                  exit
               end if
            end if
         end if
      end if
   end do

   if ( scan ) then
      if ( code ) then
         kola = max( i2-25, kola )
         kolz = min( i2+25, ncar )
         write(l6,*) mrker1
         write(l6,11) typer, car(kola:kolz), flag(:max(i2-kola,1))
         write(l6,*)  msgn1, beaf(ipas), msgn2
         write(l6,*)  car(min12:max12)
         write(l6,*) mrker2
         lerr = .true.
         return
      else
         ind = i2 + ipas
      end if
   end if

   if ( m == ip(ipas) ) then
      ik1  = max( 1, i1+k )
      kola = max( ik1-25, kola )
      kolz = min( ik1+25, ncar )
      write(l6,*) mrker1
      write(l6,11) typer, car(kola:kolz), flag(:max(ik1-kola,1))
      write(l6,*)  msgn3, beaf(ipas), ' ', car(ik1:ik1)
      write(l6,*) mrker2
      lerr = .true.
      return
   end if
   if ( abs(ind-i1) > maxc ) then
      write(l6,*) mrker1
      write(l6,11) typer, car(kola:min(kola+64,ncar))
      write(l6,*)  msgn4
      write(l6,*) mrker2
      lerr = .true.
      return
   end if

   ! Lecture de la valeur: car(i1:ind-1) ou car(ind+1:i1)
   ! selon que ipas = 1 ou -1
   mem(maxc1+k*(ind-i1):maxc) = car(min(i1,ind+k):)
   read(mem,fmtc) dval

   ! Test sur le mot attendu
   ! -----------------------
   lc1 = .false.
   if ( .not. code ) return
   jnd = ind
   ind = jnd + ipas*nc1 + k
   j   = min( ind, jnd )
   m   = max( ind, jnd )
   if ( j > 0 .and. m <= ncar ) then
      if ( car(j:m) == car1(:nc1) ) then
         lc1 = .true.
         return
      end if
   end if
   ind = jnd + ipas*nc2 + k
   j   = min( ind, jnd )
   m   = max( ind, jnd )
   if ( j > 0 .and. m <= ncar ) then
       if ( car(j:m) == car2(:nc2) ) return
   end if
   kola = max( jnd-25, kola )
   kolz = min( jnd+25, ncar )
   write(l6,*) mrker1
   write(l6,11) typer, car(kola:kolz), flag(:jnd-kola)
   write(l6,*)  illgl1, car(jnd:jnd), illgl2, car1(:nc1), illgl3, car2(:nc2)
   write(l6,*) mrker2
   lerr = .true.

   INCLUDE 'anaf11.'

END SUBROUTINE dnum
