SUBROUTINE indmix( mot, nomv, minv, maxv, kol1,icol2, car1, car2, indx, maxid, nind, lerr )

!***********************************************************************
!     - indmix -
!     retourne dans vecteur indx(maxid) une liste d'entiers qui sont
!     donnes soit explicitement (valeur entiere), soit indirectement
!     par un nom (valeur caractere) auquel cas ce nom est remplace
!     par son indice dans nomv(minv:maxv). Dans les deux cas on peut
!     avoir des boucles.
!***********************************************************************
!
!     MOT   - ENTREE. LE MOT CLE CONCERNE                     - CAR
!                     (AUQUEL EST ASSOCIE LA LISTE)
!     NOMV  - ENTREE. VECTEUR DES NOMS POSSIBLES DES ITEMS    - CAR
!                     DE LA LISTE
!     MINV  - ENTREE. BORNE INFERIEURE DE DIMENSION DE NOMV   - ENTIER
!     MAXV  - ENTREE. BORNE SUPERIEURE DE DIMENSION DE NOMV   - ENTIER
!     KOL1  - ENTREE. COL. DE DEBUT DE LA ZONE EXPLOREE       - ENTIER  E
!                     (PRECEDE LA LISTE)
!     ICOL2 - ENTREE. COL. DE FIN   DE LA ZONE EXPLOREE       - ENTIER
!                     (FERME LA LISTE)
!     CAR1  - ENTREE. CARACTERE DEVANT PRECEDER LA LISTE      - CAR
!     CAR2  - ENTREE. CARCATERE FERMANT LA LISTE              - CAR
!     INDX  - SORTIE. VECTEUR RECEVANT LA LISTE D'ENTIERS     - ENTIER
!     MAXID - ENTREE. SA DIMENSION                            - ENTIER
!     NIND  - SORTIE. LE NOMBRE D'ELEMENTS MIS DANS INDX      - ENTIER
!                     (INFERIEUR OU EGAL A MAXID)
!     LERR  - SORTIE. .TRUE. SI UNE ERREUR EST DETECTEE       - LOGIQUE
!
!     APPELLE ITEMIX
!-----------------------------------------------------------------------

   USE carl
   USE flag_msg

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: minv, maxv, kol1, icol2, maxid
   CHARACTER(LEN=*), INTENT(IN) :: mot, car1, car2
   CHARACTER(LEN=*), DIMENSION(minv:maxv), INTENT(IN) :: nomv
   INTEGER, INTENT(OUT) :: nind
   LOGICAL, INTENT(OUT) :: lerr
   INTEGER, DIMENSION(maxid), INTENT(OUT) :: indx

   !  Declarations locales
   !  --------------------

   INTEGER :: ia, ib, ic, ival, kola, kolx, kolz,  kol3, lonv
   LOGICAL :: lexit
   CHARACTER(LEN=1) :: car3

   INCLUDE 'fkwdp.'
   INCLUDE 'notfolp.'
   INCLUDE 'indmixp.'

   !--------------------------------------------------------------------

   lerr = .false.
   kola = max( kol1-17, 1 )
   kolz = min( kola+64, ncar )
   nind = 0
   if ( car(kol1:kol1) /= car1 ) then
      write(l6,*) mrker1
      write(l6,11) typer, car(kola:kolz), flag(:kol1+1-kola)
      write(l6,*)  mot, notfol, car1
      write(l6,*) mrker2
      lerr = .true.
      return
   end if
   ! lonv = longueur des noms nomv
   lonv = len( nomv(1) )
   kolx = kol1 + 1
   lexit = .false.

   ! Boucle sur les elements separes par sep1
   ! ----------------------------------------
   do
      kol3 = index( car(kolx:icol2), sep1 ) + kolx - 1
      if ( kol3 >= kolx ) then
         ! Si le separateur fait partie d'un nom on recommence
         if ( com(kol3) ) then
            kolx = kol3 + 1
            cycle
         end if

         ! Plusieurs valeurs ou boucles
         car3 = sep1
         lexit = .false.
      else
         ! Une seule valeur ou boucle
         kol3 = icol2
         car3 = car2
         lexit = .true.
      end if
      call itemix( mot, kolx, kol3, car3, nomv, minv, maxv, lonv, ia, ib, ic, lerr )
      if ( lerr ) return
      ! Stockage des indices
      do ival = ia, ib, ic
         nind = nind + 1
         if ( nind > maxid ) then
            write(l6,*) mrker1
            write(l6,11) typer, car(kola:kolz)
            write(l6,111)
            write(l6,*)  fkwd, mot
            write(l6,*) mrker2
            lerr = .true.
            return
         end if
         indx(nind) = ival
      end do
      kolx = kol3 + 1

      if ( lexit ) exit
   end do

   INCLUDE 'anaf11.'
   INCLUDE 'indmixf.'

END SUBROUTINE indmix
