SUBROUTINE indvey( mot, nomv, minv, maxv, icol, icol2, nind, kol1, indx, maxid, lerr )

!***********************************************************************
!     - indvey -
!     analyse la zone suivant immediatement un mot-cle designant un
!     vecteur et en deduit quels sont les indices du vecteur concernes:
!     si le mot cle est suivi de "=", renvoie: 0 indice; s'il est suivi
!     de par1, alors les indices sont donnes explicitement soit par
!     boucle(s) soit par liste(s), soit par melange de boucle et liste.
!     le nombre d'indices ne doit pas depasser maxid.
!     N.B. les indices peuvent etre donnes par l'intermediaire d'un nom
!     (y compris par boucle de noms), dans ce cas les noms sont
!     remplaces par leur indice dans nomv(minv:maxv)
!***********************************************************************
!
!     MOT   - ENTREE. LE MOT CLE                              - CAR
!     NOMV  - ENTREE. VECTEUR DES NOMS POSSIBLES POUR LES     - CAR
!                     INDICES
!     MINV  - ENTREE. INDICE MINIMUM DECLARE POUR LE          - ENTIER
!                     DIMENSIONNENT DE NOMV
!     MAXV  - ENTREE.    "   MAXIMUM
!     ICOL  - ENTREE. COL. DE DEBUT D'EXPLORATION             - ENTIER
!     ICOL2 - ENTREE. COL. DE LA FIN DE PHRASE (SEP2)         - ENTIER
!     NIND  - SORTIE. NOMBRE D'ELEMENTS DE LA LISTE D'INDICES - ENTIER
!     KOL1  - SORTIE. COL. DU CARACTERE "=" SUIVANT LE MOT CLE- ENTIER
!     INDX  - SORTIE. TABLEAU DE LA LISTE DES INDICES         - ENTIER
!     MAXID - ENTREE. DIMENSION DU TABLEAU INDX               - ENTIER
!     LERR  - SORTIE. .TRUE. SI UNE ERREUR EST DETECTEE       - LOGIQUE
!
!     APPELLE INDMIX
!-----------------------------------------------------------------------

   USE carl
   USE flag_msg

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN)  :: minv, maxv, icol, icol2, maxid
   CHARACTER(LEN=*), INTENT(IN) :: mot
   CHARACTER(LEN=*), DIMENSION(minv:maxv), INTENT(IN) :: nomv
   INTEGER, INTENT(OUT) :: nind, kol1
   LOGICAL, INTENT(OUT) :: lerr
   INTEGER, DIMENSION(maxid), INTENT(OUT) :: indx

   !  Declarations locales
   !  --------------------

   INTEGER :: kol, kola, kolp, kolz

   INCLUDE 'fkwdp.'
   INCLUDE 'indveyp.'

   !--------------------------------------------------------------------

   lerr = .false.
   kola = max( 1, icol-17 )
   kolz = min( kola+64, ncar )
   nind = 0

   kol1 = icol
   if ( car(kol1:kol1) == '=' ) return   ! Pas d'indice

   ! Donc c'est par1:
   ! le ou les indices sont donnes explicitement avec liste
   ! mixte (numeros et/ou noms)
   kol = kol1 + 1
   do      ! Recherche de par2
      kolp = index( car(kol:icol2), par2 ) + kol - 1
      if ( kolp < kol ) then
         write(l6,*) mrker1
         write(l6,11)  typer, car(kola:min(icol2,kolz))
         write(l6,111) par2
         write(l6,*)   fkwd, mot
         write(l6,*) mrker2
         kol1 = 0
         lerr = .true.
         return
      else
         if ( com(kolp) ) then
            if ( kol >= icol2 ) then
               write(l6,*) mrker1
               write(l6,11)  typer, car(kola:min(icol2,kolz))
               write(l6,111) par2
               write(l6,*)   fkwd, mot
               write(l6,*) mrker2
               kol1 = 0
               lerr = .true.
               return
            end if
            kol = kol + 1
         else
            exit
         end if
      end if
   end do
   ! Recuperation des indices
   call indmix( mot, nomv, minv, maxv, kol1, kolp, par1, par2, indx, maxid, nind, lerr )
   if ( lerr ) return
   kol1 = kolp + 1
   if ( car(kol1:kol1) /= '=' ) then
      kola = max( kol1-25, kola )
      kolz = min( kol1+25, ncar )
      write(l6,*) mrker1
      write(l6,11) typer, car(kola:kolz), flag(:kol1-kola)
      write(l6,33) car(kol1:kol1), par2
      write(l6,*) mrker2
      kol1 = 0
      lerr = .true.
   end if

   INCLUDE 'anaf11.'
   INCLUDE 'anaf33.'
   INCLUDE 'indveyf.'

END SUBROUTINE indvey
