SUBROUTINE inum( i1, i2, ipas, car1, nc1, car2, nc2, code, ind, lc1, ival, lerr )

!***********************************************************************
!     - inum -
!     lecture gauche->droite si pas=1, droite->gauche si ipas=-1.
!     donne le nombre ival (entier) situe entre col. i1 et i2 de
!     car et, si code = .true., la colonne ind :
!        du dernier caractere du mot suivant   ival, si ipas = 1
!        du premier caractere du mot precedant ival, si ipas =-1
!     quand code=.true. ce caractere doit exister et etre car1 ou
!     car2 et si c'est car1 alors lc1=.true. en retour
!     il faut i1<i2 si ipas=1, i1>i2 si ipas=-1 .
!     si aucun nombre trouve retour en erreur par *
!     retour * si erreur
!
!     maxci = le nombre maximum de chiffres dans un nombre
!     n.b. le plus grand entier lu sans erreur par inum est maxint,
!     maxint designant le plus grand entier possible; par exemple,
!     sur ibm on pourra lire jusqu'a 2**31-1
!***********************************************************************
!
!     I1    - ENTREE. 1ERE BORNE DE LA ZONE EXAMINEE          - ENTIER
!     I2    - ENTREE. 2EME BORNE DE LA ZONE EXAMINEE          - ENTIER
!     IPAS  - ENTREE. SENS DE LECTURE :                       - ENTIER
!                        1 = GAUCHE -> DROITE (=> I1<=I2)
!                       -1 = DROITE -> GAUCHE (=> I1>=I2)
!     CAR1  - ENTREE. 1ER  MOT "CIBLE"                        - CAR
!     NC1   - ENTREE. SA LONGUEUR                             - ENTIER
!     CAR2  - ENTREE. 2EME MOT "CIBLE"                        - CAR
!     NC2   - ENTREE. SA LONGUEUR                             - ENTIER
!     CODE  - ENTREE. SI CODE=.TRUE. ALORS LE MOT SUIVANT/    - LOGIQUE
!                     PRECEDANT IVAL DOIT EXISTER ET ETRE
!                     CAR1(:NC1) OU CAR2(:NC2)
!     IND   - SORTIE. LA COLONNE DU DERNIER/PREMIER CARACTERE - ENTIER
!                     SUIVANT/PRECEDANT IVAL (SELON QUE IPAS
!                     VAUT 1 OU -1)
!                     SI CODE=.FALSE., C'EST LA COLONNE
!                     SUIVANT/PRECEDANT IVAL
!     LC1   - SORTIE. .TRUE. QUAND CODE=.TRUE. ET LE MOT      - LOGIQUE
!                     "CIBLE" EST(CAR1:NC1)
!     IVAL  - SORTIE. LA VALEUR TROUVEE (ERREUR SI AUCUNE)    - ENTIER
!     LERR  - SORTIE. .TRUE. SI UNE ERREUR EST DETECTEE       - LOGIQUE
!-----------------------------------------------------------------------

   USE carl
   USE flag_msg
   USE is_digit

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN)  :: i1, i2, ipas, nc1, nc2
   LOGICAL, INTENT(IN)  :: code
   CHARACTER(LEN=*), INTENT(IN) :: car1, car2
   INTEGER, INTENT(OUT) :: ind
   LOGICAL, INTENT(OUT) :: lc1, lerr
   INTEGER, INTENT(OUT) :: ival

   !  Declarations locales
   !  --------------------

   INTEGER :: i, ik1, j, jnd, jpw, kola, kolz, m, max12, min12
   LOGICAL :: negatif, lkar, scan
   INTEGER, PARAMETER :: imns=IACHAR('-'), ipls=IACHAR('+')
   INTEGER, PARAMETER :: izero=IACHAR('0')

   INTEGER, PARAMETER :: maxci=10   ! Nombre maximum de chiffres dans un nombre

   INCLUDE 'illglp.'
   INCLUDE 'inump.'
   INCLUDE 'msgernp.'

   !--------------------------------------------------------------------

   max12 = max( i1, i2 )
   min12 = min( i1, i2 )
   kola  = max( 1, min12-17 )
   if ( (i2-i1)*ipas < 0 ) then
      write(l6,*) mrker1
      write(l6,11) typer, car(kola:min(max12,kola+64)), msgn0
      write(l6,*) mrker2
      lerr = .true.
      return
   end if

   lerr  = .false.
   ival  = 0
   negatif = .false.
   lkar  = .true.
   m     = 0
   scan  = .true.

   if ( ipas > 0 ) then   ! Lecture de gauche a droite
                          ! --------------------------

      ! Boucle sur les caracteres de la zone
      ! ------------------------------------
      do i = i1, i2, ipas
         j = iachar( car(i:i) )
         if ( ldigit(j) .and. lkar ) then  ! Ce caractere est un chiffre
            m    = m + 1
            ival = ival*10 + (j-izero)
         else
            if ( (j == imns .or. j == ipls) .and. lkar .and. i < i1+1 ) then
               negatif = j == imns
               lkar  = .true.
            else
               ind = i
               scan = .false.
               exit
            end if
         end if
      end do

   else                   ! Lecture de droite a gauche
                          ! --------------------------

      ! Le 1er caractere
      ! ----------------
      j = iachar( car(i1:i1) )
      if ( ldigit(j) ) then
         ! Ce caractere est un chiffre
         m    = 1
         ival = j - izero
         ! Boucle sur les caracteres suivants
         ! ----------------------------------
         jpw = 1
         do i = i1+ipas, i2, ipas
            j = iachar( car(i:i) )
            if ( ldigit(j) .and. lkar ) then  ! Ce caractere est un chiffre
               m    = m + 1
               jpw  = jpw * 10
               ival = ival + (j-izero)*jpw
            else
               if ( (j == imns .or. j == ipls) .and. lkar .and. i < i1 ) then
                  negatif = j == imns
                  lkar  = .false.
               else
                  ind = i
                  scan = .false.
                  exit
               end if
            end if
         end do
      else
         scan = .false.
      end if

   end if

   if ( scan ) then
      ! On arrive ici quand toute la zone est numerique
      ! -----------------------------------------------
      if ( code ) then
         kola = max( i2-25, kola )
         kolz = min( i2+25, ncar )
         write(l6,*) mrker1
         write(l6,11) typer, car(kola:kolz), flag(:max(i2-kola,1))
         write(l6,*)  msgn1, beaf(ipas), msgn2
         write(l6,*)  car(min12:max12)
         write(l6,*) mrker2
         lerr = .true.
         return
      else
         ind = i2 + ipas
      end if
   end if

   if ( m <= 0 .or. m > maxci ) then
      ik1  = max( 1, i1-ipas )
      kola = max( ik1-25, kola )
      kolz = min( ik1+25, ncar )
      write(l6,*) mrker1
      write(l6,11) typer, car(kola:kolz), flag(:max(ik1-kola,1))
      write(l6,*)  msgn3, beaf(ipas), ' ', car(ik1:ik1), msgni, msgn4
      write(l6,*) mrker2
      lerr = .true.
      return
   end if

   ! Retablissement du signe
   ! -----------------------
   if ( negatif ) ival = -ival

   ! Test sur le mot attendu
   ! -----------------------
   lc1 = .false.
   if ( .not. code ) return
   jnd = ind
   ind = jnd + ipas*nc1 - ipas
   j   = min( ind, jnd )
   m   = max( ind, jnd )
   if ( j > 0 .and. m <= ncar ) then
      if ( car(j:m) == car1(:nc1) ) then
         lc1 = .true.
         return
      end if
   end if
   ind = jnd + ipas*nc2 - ipas
   j   = min( ind, jnd )
   m   = max( ind, jnd )
   if ( j > 0 .and. m <= ncar ) then
       if ( car(j:m) == car2(:nc2) ) return
   end if
   kola = max( jnd-25, kola )
   kolz = min( jnd+25, ncar )
   write(l6,*) mrker1
   write(l6,11) typer, car(kola:kolz), flag(:jnd-kola)
   write(l6,*)  illgl1, car(jnd:jnd), illgl2, car1(:nc1), illgl3, car2(:nc2)
   write(l6,*) mrker2
   lerr = .true.

   INCLUDE 'anaf11.'

END SUBROUTINE inum
