SUBROUTINE itemix( mot, col1, col2, car2, nomv, minv, maxv, lonv, ia, ib, ic, lerr )

!***********************************************************************
!     - itemix -
!     analyse d'un element d'une liste mixte (numeros et/ou noms),
!     un element pouvant etre une valeur simple (entiere ou caractere)
!     ou une boucle (entiere ou caractere)
!     N.B. pour les noms, l'analyse ne distingue pas les minuscules et
!     les majuscules
!***********************************************************************
!
!     MOT   - ENTREE. LE MOT-CLE CONCERNE                      - CAR
!     COL1  - ENTREE. DEBUT DE LA ZONE EXPLOREE                - CAR
!     COL2  - ENTREE. FIN   DE LA ZONE EXPLOREE                - CAR
!     CAR2  - ENTREE. CARACTERE ATTENDU COMME FIN DE ZONE      - CAR
!                     (EN COL2)
!     NOMV  - ENTREE. VECTEURS DES NOMS POSSIBLES              - CAR
!     MINV  - ENTREE. INDICE MINIMUM DE NOMV                   - ENTIER
!     MAXV  - ENTREE. INDICE MAXIMUM DE NOMV                   - ENTIER
!     LONV  - ENTREE. LONGUEUR DE CES NOMS                     - ENTIER
!     IA    - SORTIE. NUMERO DU 1ER ELEMENT                    - ENTIER
!     IB    - SORTIE. NUMERO DE LA FIN DE BOUCLE               - ENTIER
!                     (=IA SI UN SEUL ELEMENT)
!     IC    - SORTIE. PAS DE LA BOUCLE                         - ENTIER
!                     (=1 SI UN SEUL ELEMENT)
!     LERR  - SORTIE. .TRUE. SI UNE ERREUR EST DETECTEE       - LOGIQUE
!
!     APPELLE INUM, CNUM, SAMSTR
!-----------------------------------------------------------------------

   USE carl
   USE is_digit

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN)  :: col1, col2, minv, maxv, lonv
   CHARACTER(LEN=*), INTENT(IN) :: mot
   CHARACTER(LEN=1), INTENT(IN) :: car2
   CHARACTER(LEN=*), DIMENSION(minv:maxv), INTENT(IN) :: nomv
   INTEGER, INTENT(OUT) :: ia, ib, ic
   LOGICAL, INTENT(OUT) :: lerr


   !  Declarations locales
   !  --------------------

   INTEGER :: i, kol, kola, kolx, kolz
   LOGICAL :: lgl, lglx, lgly
   CHARACTER(LEN=64) :: mem
   LOGICAL, EXTERNAL :: samstr

   INCLUDE 'outrngp.'
   INCLUDE 'itemixp.'

   !--------------------------------------------------------------------

   ia   = 0
   ib   = -1
   ic   = 1
   lerr = .false.
   kola = max( col1-17, 1 )
   kolz = min( kola+64, ncar )

   ! Numero ou nom ?
   if ( .not. com(col1) .and. ldigit(iachar(car(col1:col1))) ) then
      ! Numero
      call inum( col1, col2, 1, car2, 1, to, ncto, .true., kol, lgl, ia, lerr )
      if ( lerr ) return
   else
      ! Nom
      call cnum( col1, col2, 1, car2, 1, to, ncto, .true., kol, lgl, mem, lerr )
      if ( lerr ) return
      ! Recherche de l'indice
      lerr = .true.
      do i = minv, maxv
         if ( samstr( mem(:lonv), nomv(i)(:lonv) ) ) then
            ia = i
            lerr = .false.
            exit
         end if
      end do
      if ( lerr ) then
         write(l6,*) mrker1
         write(l6,11) typer, car(kola:kolz)
         write(l6,*)  mem(:lonv)
         write(l6,111)
         write(l6,*) mrker2
         return
      end if
   end if

   if ( lgl ) then
      ! Valeur unique
      ib = ia
   else
      ! Boucle
      kol = kol + 1
      if ( .not. com(kol) .and. ldigit(iachar(car(kol:kol))) ) then
         ! Numero
         call inum( kol, col2, 1, by, ncby, car2, 1, .true., kolx, lglx, ib, lerr )
         if ( lerr ) return
      else
         ! Nom
         call cnum( kol, col2, 1, by, ncby, car2, 1, .true., kolx, lglx, mem, lerr )
         if ( lerr ) return
         ! Recherche de l'indice
         lerr = .true.
         do i = minv, maxv
            if ( samstr( mem(:lonv), nomv(i)(:lonv) ) ) then
               ib = i
               lerr = .false.
               exit
            end if
         end do
         if ( lerr ) then
            write(l6,*) mrker1
            write(l6,11) typer, car(kola:kolz)
            write(l6,*)  mem(:lonv)
            write(l6,111)
            write(l6,*) mrker2
            return
         end if
      end if
      if ( lglx ) then
         call inum( kolx+1, col2,  1, car2, 1, car2, 1, .true., kol, lgly, ic, lerr )
         if ( lerr ) return
         if ( ic == 0 ) ic = 1
      end if
      ! Correction eventuelle du pas
      if ( ia > ib .and. ic > 0 ) ic = -ic
   end if

   if ( min(ia,ib) < minv .or. max(ia,ib) > maxv ) then
      write(l6,*) mrker1
      write(l6,11) typer, car(kola:kolz)
      write(l6,*)  outrng, mot
      write(l6,*) mrker2
      lerr = .true.
   end if

   INCLUDE 'anaf11.'
   INCLUDE 'itemixf.'

END SUBROUTINE itemix
