SUBROUTINE comprs( str, a, b, nsup )

!***********************************************************************
!     - comprs -
!     suppresses character a in character string str, and replaces
!     successive characters b by a single one
!***********************************************************************
!
!     str   - INPUT . Character string to be treated          - CHAR
!             OUTPUT. Compressed character string
!                     (padded with blanks)
!     a     - INPUT . Character to be suppressed in str       - CHAR
!     b     - INPUT . Character to be compressed in str:      - CHAR
!                     successive occurrences are replaced by
!                     a single one
!     nsup  - OUTPUT. Number of characters suppressed         - INTEGER
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        LEN_TRIM
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   CHARACTER(LEN=1), INTENT(IN) :: a, b
   CHARACTER(LEN=*), INTENT(INOUT) :: str
   INTEGER, INTENT(OUT) :: nsup

   !  Local declarations
   !  ------------------

   INTEGER :: i, ia, jbeg, jend, n, ncar, ncol, nn
   LOGICAL :: lb, lc, ls

   !-----------------------------------------------------------------

      ! jend is last column of the zone scanned, thus it changes
      ! during the process
      ! ia is the starting column for a suppression step

   nn   = len_trim( str )
   ncol = nn
   ncar = nn
   jend = nn
   jbeg = 1
   ls   = .false.
   lc   = .false.
   n    = 0
   i    = jbeg

   do
      lb = str(i:i) == b
      if ( lb .or. str(i:i) == a ) then
         if ( lb ) lc = .true.
         n = n + 1
         if ( .not. ls ) then
            ia = i
            ls = .true.
         end if
      else
         if ( lc ) then      ! compression
            str(ia:ia) = b
            n  = n - 1
            ia = ia + 1
            lc = .false.
         end if
         if ( n > 0 ) then   ! suppression
            ncol = ncar - n
            str(ia:ncol) = str(i:ncar)
            ncar = ncol
            jend = jend - n
            i    = ia - 1
            n    = 0
         end if
         ls = .false.
      end if
      i = i + 1
      if ( i > jend ) exit
   end do

   if ( ls ) then
      !  last suppression/compression is pending
      if ( lc ) then      ! compression
         str(ia:ia) = b
         n  = n - 1
         ia = ia + 1
      end if
      if ( n > 0 ) then   ! suppression
         ncol = ncar - n
         str(ia:ncol) = str(i:ncar)
      end if
   end if

   nsup = nn - ncol
   if ( nsup > 0 ) str(ncol+1:nn) = ' '   ! padd with blanks

END SUBROUTINE comprs
