SUBROUTINE cvucs( str, com, sepa, sepb )

!***********************************************************************
!     - cvucs -
!     selectively converts lower case letters of a character string to
!     upper case letters:
!     for a letter to be converted, 2 conditions must hold:
!     . com(i) must be .false. (i being the location in str of the
!       letter)
!     . the letter must not belong to a zone beginning with sepa
!       (sepb cancels a previous sepa, i.e. marks the end of a
!       non-convertible zone and so the beginning of a convertible zone)
!     N.B. a sepa or sepb character serves as a delimitor (on location
!     j, for example) only if com(j)=.false., otherwise it is ignored.
!     If the 1st character in the character string is not sepa, the
!     zone is considered convertible
!***********************************************************************
!
!     STR   - INPUT . Character string                        - CHAR
!             OUTPUT. Modified character string
!     COM   - INPUT . Array of same length as str             - LOGICAL
!                     If com(i)=.true. str(i:i) is unchanged
!     SEPA  - INPUT . Character delimiting starting location  - CHAR
!                     of an unconvertible zone
!                     (active only if com(i)=.false.)
!     SEPB  - INPUT . Character delimiting starting location  - CHAR
!                     of a convertible zone
!                     (active only if com(i)=.false.)
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        ACHAR  IACHAR  LEN  LEN_TRIM
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   CHARACTER(LEN=*), INTENT(INOUT) :: str
   LOGICAL, DIMENSION(LEN(str)), INTENT(IN) :: com
   CHARACTER(LEN=1), INTENT(IN) :: sepa, sepb

   !  Local declarations
   !  ------------------

   INTEGER, PARAMETER :: konv=iachar('A')-iachar('a')
   INTEGER, PARAMETER :: ia=iachar('a'), iz=iachar('z')
   INTEGER :: i, kod
   LOGICAL :: lsepb

   !--------------------------------------------------------------------

   lsepb = .true.

   do i = 1, len_trim(str)
      if ( .not. com(i) ) then

         if ( lsepb ) then
            if ( str(i:i) == sepa ) then
               lsepb = .false.
            else
               kod = iachar( str(i:i) )
               if ( kod >= ia .and. kod <= iz ) str(i:i) = achar( kod+konv )
            end if
         else
            lsepb = str(i:i) == sepb
         end if

      end if
   end do

END SUBROUTINE cvucs
