SUBROUTINE DSPDIX(AP,N,KPVT,DET,INERT,WORK,JOB)
      INTEGER :: N,JOB
      real(kind=8) :: ap(n*(n+1)/2),work(n)
      real(kind=8) :: DET(2)
      integer :: kpvt(n),INERT(3)
!
! **********************************************************************
!     THIS IS A SLIGHTLY MODIFIED VERSION OF THE ORIGINAL LINPACK
!     SUBROUTINE DSPDI :
!     INERT IS ALWAYS COMPUTED AS IT SERVES AS AN ERROR-CONDITION:
!     IF THE NUMBER OF POSITIVE EIGENVALUES IS NOT N (THE ORIGINAL
!     MATRIX IS NOT POSITIVE DEFINITE) INVERSE IS NOT COMPUTED
! **********************************************************************
!
!     DSPDI COMPUTES THE DETERMINANT, INERTIA AND INVERSE
!     OF A DOUBLE PRECISION SYMMETRIC MATRIX USING THE FACTORS FROM
!     DSPFA, WHERE THE MATRIX IS STORED IN PACKED FORM.
!
!     ON ENTRY
!
!        AP      DOUBLE PRECISION (N*(N+1)/2)
!                THE OUTPUT FROM DSPFA.
!
!        N       INTEGER
!                THE ORDER OF THE MATRIX A.
!
!        KPVT    INTEGER(N)
!                THE PIVOT VECTOR FROM DSPFA.
!
!        WORK    DOUBLE PRECISION(N)
!                WORK VECTOR.  CONTENTS IGNORED.
!
!        JOB     INTEGER
!                JOB HAS THE DECIMAL EXPANSION  ABC  WHERE
!                   IF  C .NE. 0, THE INVERSE IS COMPUTED,
!                   IF  B .NE. 0, THE DETERMINANT IS COMPUTED,
!   XXXXXXXXXXXXXXX IF  A .NE. 0, THE INERTIA IS COMPUTED. ALWAYS CMPTED
!
!                FOR EXAMPLE, JOB = 111  GIVES ALL THREE.
!
!     ON RETURN
!
!        VARIABLES NOT REQUESTED BY JOB ARE NOT USED.
!
!        AP     CONTAINS THE UPPER TRIANGLE OF THE INVERSE OF
!               THE ORIGINAL MATRIX, STORED IN PACKED FORM.
!               THE COLUMNS OF THE UPPER TRIANGLE ARE STORED
!               SEQUENTIALLY IN A ONE-DIMENSIONAL ARRAY.
!
!        DET    DOUBLE PRECISION(2)
!               DETERMINANT OF ORIGINAL MATRIX.
!               DETERMINANT = DET(1) * 10.0**DET(2)
!               WITH 1.0 .LE. abs(DET(1)) .LT. 10.0
!               OR DET(1) = 0.0.
!
!        INERT  INTEGER(3)
!               THE INERTIA OF THE ORIGINAL MATRIX.
!               INERT(1)  =  NUMBER OF POSITIVE EIGENVALUES.
!               INERT(2)  =  NUMBER OF NEGATIVE EIGENVALUES.
!               INERT(3)  =  NUMBER OF ZERO EIGENVALUES.
!
!     ERROR CONDITION
!
!        A DIVISION BY ZERO WILL OCCUR IF THE INVERSE IS REQUESTED
!        AND  DSPCO  HAS SET RCOND .EQ. 0.0
!        OR  DSPFA  HAS SET  INFO .NE. 0 .
!
!     LINPACK. THIS VERSION DATED 08/14/78 .
!     JAMES BUNCH, UNIV. CALIF. SAN DIEGO, ARGONNE NAT. LAB.
!
!     SUBROUTINES AND FUNCTIONS
!
!     BLAS DAXPY,DCOPY,DDOT,DSWAP
!     FORTRAN abs,MOD
!
!     INTERNAL VARIABLES.
!
      real(kind=8) :: AKKP1,DDOT,TEMP
      real(kind=8) :: TEN,D,T,AK,AKP1
      INTEGER :: IJ,IK,IKP1,IKS,J,JB,JK,JKP1
      INTEGER :: K,KK,KKP1,KM1,KS,KSJ,KSKP1,KSTEP
      LOGICAL :: NOINV,NODET     !!!! ,NOERT
!
      NOINV = MOD(JOB,10) .EQ. 0
      NODET = MOD(JOB,100)/10 .EQ. 0
!!!!  NOERT = MOD(JOB,1000)/100 .EQ. 0
!
!!!!  IF (NODET .AND. NOERT) GO TO 140
!!!!     IF (NOERT) GO TO 10
            INERT(1) = 0
            INERT(2) = 0
            INERT(3) = 0
!!!10    CONTINUE
         IF (NODET) GO TO 20
            DET(1) = 1.0_8
            DET(2) = 0.0_8
            TEN = 10.0_8
   20    CONTINUE
         T = 0.0_8
         IK = 0
         DO 130 K = 1, N
            KK = IK + K
            D = AP(KK)
!
!           CHECK IF 1 BY 1
!
            IF (KPVT(K) .GT. 0) GO TO 50
!
!              2 BY 2 BLOCK
!              USE DET (D  S)  =  (D/T * C - T) * T  ,  T = abs(S)
!                      (S  C)
!              TO AVOID UNDERFLOW/OVERFLOW TROUBLES.
!              TAKE TWO PASSES THROUGH SCALING.  USE  T  FOR FLAG.
!
               IF (T .NE. 0.0_8) GO TO 30
                  IKP1 = IK + K
                  KKP1 = IKP1 + K
                  T = abs(AP(KKP1))
                  D = (D/T)*AP(KKP1+1) - T
               GO TO 40
   30          CONTINUE
                  D = T
                  T = 0.0_8
   40          CONTINUE
   50       CONTINUE
!
!!!!!       IF (NOERT) GO TO 60
               IF (D .GT. 0.0_8) INERT(1) = INERT(1) + 1
               IF (D .LT. 0.0_8) INERT(2) = INERT(2) + 1
               IF (D .EQ. 0.0_8) INERT(3) = INERT(3) + 1
!!!60       CONTINUE
!
            IF (NODET) GO TO 120
               DET(1) = D*DET(1)
               IF (DET(1) .EQ. 0.0_8) GO TO 110
   70             IF (abs(DET(1)) .GE. 1.0_8) GO TO 80
                     DET(1) = TEN*DET(1)
                     DET(2) = DET(2) - 1.0_8
                  GO TO 70
   80             CONTINUE
   90             IF (abs(DET(1)) .LT. TEN) GO TO 100
                     DET(1) = DET(1)/TEN
                     DET(2) = DET(2) + 1.0_8
                  GO TO 90
  100             CONTINUE
  110          CONTINUE
  120       CONTINUE
            IK = IK + K
  130    CONTINUE
  140 CONTINUE
!
!     COMPUTE INVERSE(A)
!
!        CHECK THE MATRIX IS POSITIVE DEFINITE
      IF( INERT(1) .NE. N ) GO TO 270
!
      IF (NOINV) GO TO 270
         K = 1
         IK = 0
  150    IF (K .GT. N) GO TO 260
            KM1 = K - 1
            KK = IK + K
            IKP1 = IK + K
            KKP1 = IKP1 + K
            IF (KPVT(K) .LT. 0) GO TO 180
!
!              1 BY 1
!
               AP(KK) = 1.0_8/AP(KK)
               IF (KM1 .LT. 1) GO TO 170
                  call dcopy(km1,ap(ik+1:),1,work,1)
                  IJ = 0
                  DO 160 J = 1, KM1
                     JK = IK + J
                     ap(jk) = ddot(j,ap(ij+1:),1,work,1)
                     call daxpy(j-1,work(j),ap(ij+1:),1,ap(ik+1:),1)
                     IJ = IJ + J
  160             CONTINUE
                  ap(kk) = ap(kk) + ddot(km1,work,1,ap(ik+1:),1)
  170          CONTINUE
               KSTEP = 1
            GO TO 220
  180       CONTINUE
!
!              2 BY 2
!
               T = abs(AP(KKP1))
               AK = AP(KK)/T
               AKP1 = AP(KKP1+1)/T
               AKKP1 = AP(KKP1)/T
               D = T*(AK*AKP1 - 1.0_8)
               AP(KK) = AKP1/D
               AP(KKP1+1) = AK/D
               AP(KKP1) = -AKKP1/D
               IF (KM1 .LT. 1) GO TO 210
                  call dcopy(km1,ap(ikp1+1:),1,work,1)
                  IJ = 0
                  DO 190 J = 1, KM1
                     JKP1 = IKP1 + J
                     ap(jkp1) = ddot(j,ap(ij+1:),1,work,1)
                     call daxpy(j-1,work(j),ap(ij+1:),1,ap(ikp1+1:),1)
                     IJ = IJ + J
  190             CONTINUE
                  ap(kkp1+1) = ap(kkp1+1) + ddot(km1,work,1,ap(ikp1+1:),1)
                  ap(kkp1) = ap(kkp1) + ddot(km1,ap(ik+1:),1,ap(ikp1+1:),1)
                  call dcopy(km1,ap(ik+1:),1,work,1)
                  IJ = 0
                  DO 200 J = 1, KM1
                     JK = IK + J
                     ap(jk) = ddot(j,ap(ij+1:),1,work,1)
                     call daxpy(j-1,work(j),ap(ij+1:),1,ap(ik+1:),1)
                     IJ = IJ + J
  200             CONTINUE
                  ap(kk) = ap(kk) + ddot(km1,work,1,ap(ik+1:),1)
  210          CONTINUE
               KSTEP = 2
  220       CONTINUE
!
!           SWAP
!
            KS = abs(KPVT(K))
            IF (KS .EQ. K) GO TO 250
               IKS = (KS*(KS - 1))/2
               call dswap(ks,ap(iks+1:),1,ap(ik+1:),1)
               KSJ = IK + KS
               DO 230 JB = KS, K
                  J = K + KS - JB
                  JK = IK + J
                  TEMP = AP(JK)
                  AP(JK) = AP(KSJ)
                  AP(KSJ) = TEMP
                  KSJ = KSJ - (J - 1)
  230          CONTINUE
               IF (KSTEP .EQ. 1) GO TO 240
                  KSKP1 = IKP1 + KS
                  TEMP = AP(KSKP1)
                  AP(KSKP1) = AP(KKP1)
                  AP(KKP1) = TEMP
  240          CONTINUE
  250       CONTINUE
            IK = IK + K
            IF (KSTEP .EQ. 2) IK = IK + K + 1
            K = K + KSTEP
         GO TO 150
  260    CONTINUE
  270 CONTINUE

END SUBROUTINE DSPDIX
