SUBROUTINE DSPSL(AP,N,KPVT,B)
      INTEGER :: N
      integer :: kpvt(n)
      real(kind=8) :: ap(n*(n+1)/2),b(n)
!
!     DSISL SOLVES THE DOUBLE PRECISION SYMMETRIC SYSTEM
!     A * X = B
!     USING THE FACTORS COMPUTED BY DSPFA.
!
!     ON ENTRY
!
!        AP      DOUBLE PRECISION(N*(N+1)/2)
!                THE OUTPUT FROM DSPFA.
!
!        N       INTEGER
!                THE ORDER OF THE MATRIX  A .
!
!        KPVT    INTEGER(N)
!                THE PIVOT VECTOR FROM DSPFA.
!
!        B       DOUBLE PRECISION(N)
!                THE RIGHT HAND SIDE VECTOR.
!
!     ON RETURN
!
!        B       THE SOLUTION VECTOR  X .
!
!     ERROR CONDITION
!
!        A DIVISION BY ZERO MAY OCCUR IF  DSPCO  HAS SET RCOND .EQ. 0.0
!        OR  DSPFA  HAS SET INFO .NE. 0  .
!
!     TO COMPUTE  INVERSE(A) * C  WHERE  C  IS A MATRIX
!     WITH  P  COLUMNS
!           CALL DSPFA(AP,N,KPVT,INFO)
!           IF (INFO .NE. 0) GO TO ...
!           DO 10 J = 1, P
!              CALL DSPSL(AP,N,KPVT,C(1,J))
!        10 CONTINUE
!
!     LINPACK. THIS VERSION DATED 08/14/78 .
!     JAMES BUNCH, UNIV. CALIF. SAN DIEGO, ARGONNE NAT. LAB.
!
!     SUBROUTINES AND FUNCTIONS
!
!     BLAS DAXPY,DDOT
!     FORTRAN abs
!
!     INTERNAL VARIABLES.
!
      real(kind=8) :: AK,AKM1,BK,BKM1,DDOT,DENOM,TEMP
      INTEGER :: IK,IKM1,IKP1,K,KK,KM1K,KM1KM1,KP
!
!     LOOP BACKWARD APPLYING THE TRANSFORMATIONS AND
!     D INVERSE TO B.
!
      K = N
      IK = (N*(N - 1))/2
   10 IF (K .EQ. 0) GO TO 80
         KK = IK + K
         IF (KPVT(K) .LT. 0) GO TO 40
!
!           1 X 1 PIVOT BLOCK.
!
            IF (K .EQ. 1) GO TO 30
               KP = KPVT(K)
               IF (KP .EQ. K) GO TO 20
!
!                 INTERCHANGE.
!
                  TEMP = B(K)
                  B(K) = B(KP)
                  B(KP) = TEMP
   20          CONTINUE
!
!              APPLY THE TRANSFORMATION.
!
               call daxpy(k-1,b(k),ap(ik+1:),1,b(1:),1)
   30       CONTINUE
!
!           APPLY D INVERSE.
!
            B(K) = B(K)/AP(KK)
            K = K - 1
            IK = IK - K
         GO TO 70
   40    CONTINUE
!
!           2 X 2 PIVOT BLOCK.
!
            IKM1 = IK - (K - 1)
            IF (K .EQ. 2) GO TO 60
               KP = abs(KPVT(K))
               IF (KP .EQ. K - 1) GO TO 50
!
!                 INTERCHANGE.
!
                  TEMP = B(K-1)
                  B(K-1) = B(KP)
                  B(KP) = TEMP
   50          CONTINUE
!
!              APPLY THE TRANSFORMATION.
!
               call daxpy(k-2,b(k),ap(ik+1:),1,b(1:),1)
               call daxpy(k-2,b(k-1),ap(ikm1+1:),1,b(1:),1)
   60       CONTINUE
!
!           APPLY D INVERSE.
!
            KM1K = IK + K - 1
            KK = IK + K
            AK = AP(KK)/AP(KM1K)
            KM1KM1 = IKM1 + K - 1
            AKM1 = AP(KM1KM1)/AP(KM1K)
            BK = B(K)/AP(KM1K)
            BKM1 = B(K-1)/AP(KM1K)
            DENOM = AK*AKM1 - 1.0_8
            B(K) = (AKM1*BK - BKM1)/DENOM
            B(K-1) = (AK*BKM1 - BK)/DENOM
            K = K - 2
            IK = IK - (K + 1) - K
   70    CONTINUE
      GO TO 10
   80 CONTINUE
!
!     LOOP FORWARD APPLYING THE TRANSFORMATIONS.
!
      K = 1
      IK = 0
   90 IF (K .GT. N) GO TO 160
         IF (KPVT(K) .LT. 0) GO TO 120
!
!           1 X 1 PIVOT BLOCK.
!
            IF (K .EQ. 1) GO TO 110
!
!              APPLY THE TRANSFORMATION.
!
               b(k) = b(k) + ddot(k-1,ap(ik+1:),1,b(1:),1)
               KP = KPVT(K)
               IF (KP .EQ. K) GO TO 100
!
!                 INTERCHANGE.
!
                  TEMP = B(K)
                  B(K) = B(KP)
                  B(KP) = TEMP
  100          CONTINUE
  110       CONTINUE
            IK = IK + K
            K = K + 1
         GO TO 150
  120    CONTINUE
!
!           2 X 2 PIVOT BLOCK.
!
            IF (K .EQ. 1) GO TO 140
!
!              APPLY THE TRANSFORMATION.
!
               b(k) = b(k) + ddot(k-1,ap(ik+1:),1,b(1:),1)
               IKP1 = IK + K
               b(k+1) = b(k+1) + ddot(k-1,ap(ikp1+1:),1,b(1:),1)
               KP = abs(KPVT(K))
               IF (KP .EQ. K) GO TO 130
!
!                 INTERCHANGE.
!
                  TEMP = B(K)
                  B(K) = B(KP)
                  B(KP) = TEMP
  130          CONTINUE
  140       CONTINUE
            IK = IK + K + K + 1
            K = K + 2
  150    CONTINUE
      GO TO 90
  160 CONTINUE

END SUBROUTINE DSPSL
