SUBROUTINE mikabc( a, b, c, ii, ia, ja, jb )

!***********************************************************************
!     - mikabc -
!     multiplies ("direct" - Kroneker - product of I by A) by B,
!     where I is identity matrix and A and B general matrices,
!     to form resultant general matrix C
!***********************************************************************
!
!     A     - INPUT . General matrix, 2-dimensioned with      - REAL*8
!                     IA rows and JA columns
!     B     - INPUT . General matrix, 2-dimensioned with      - REAL*8
!                     II*JA rows and JB columns
!     C     - OUTPUT. Resultant general matrix, 2-dimensioned - REAL*8
!                     with II*IA rows and JB columns
!     II    - INPUT . Order of the identity matrix which      - INTEGER
!                     directly pre multiplies A
!     IA    - INPUT . Number of rows in A                     - INTEGER
!     JA    - INPUT . Number of columns in A                  - INTEGER
!     JB    - INPUT . Number of columns in B and C            - INTEGER
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------
!
!     Note: the "direct" product of I by A is taken as follows
!     (assuming, e.g., order of I is 3 and order of A is 2):
!        a(1,1)  a(1,2)    0       0       0       0
!        a(2,1)  a(2,2)    0       0       0       0
!          0       0     a(1,1)  a(1,2)    0       0
!          0       0     a(2,1)  a(2,2)    0       0
!          0       0       0       0     a(1,1)  a(1,2)
!          0       0       0       0     a(2,1)  a(2,2)
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: ii, ia, ja, jb
   REAL(KIND=8), INTENT(IN) :: a(ia,ja), b(ii*ja,jb)
   REAL(KIND=8), INTENT(OUT) :: c(ii*ia,jb)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   REAL(KIND=8) :: sumx
   INTEGER :: i, i0, j, k, k0, l

   !--------------------------------------------------------------------

   i0 = 0
   k0 = 0

   do l = 1, ii

      do j = 1, jb
         do i = 1, ia
            sumx = zero

            do k = 1, ja
               sumx = sumx + a(i,k) * b(k0+k,j)
            end do
            c(i0+i,j) = sumx

         end do
      end do

      i0 = i0 + ia
      k0 = k0 + ja
   end do

END SUBROUTINE mikabc
