SUBROUTINE msbc( s, b, c, m, n )

!***********************************************************************
!     - msbc -
!     multiplies symmetric matrix S (stored in triangular form)
!     by general matrix B to form resultant general matrix C
!***********************************************************************
!
!     S     - INPUT . Symmetric matrix (in triangular form)   - REAL*8
!     B     - INPUT . General matrix (columnwise)             - REAL*8
!     C     - OUTPUT. Resultant general matrix (columnwise)   - REAL*8
!     M     - INPUT . Number of rows and columns in S,        - INTEGER
!                     of rows in B and C
!     N     - INPUT . Number of columns in B and C            - INTEGER
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: m, n
   REAL(KIND=8), INTENT(IN) :: s(m*(m+1)/2), b(m*n)
   REAL(KIND=8), INTENT(OUT) :: c(m*n)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   INTEGER :: ik, is, jk, k, km1

   !--------------------------------------------------------------------

      ! Note: the elements of the triangle of the symmetric matrix
      ! are accessed in their sequential order

   km1 = 1 - m

   do k = 1, n
      km1 = km1 + m
      is  = 1
      c(km1) = s(1) * b(km1)

      do ik = km1+1, km1-1+m
         c(ik) = zero

         do jk = km1, ik-1
            is = is + 1
            c(ik) = c(ik)  +  s(is) * b(jk)
            c(jk) = c(jk)  +  s(is) * b(ik)
         end do

         is = is + 1
         c(ik) = c(ik)  +  s(is) * b(ik)
      end do

   end do

END SUBROUTINE msbc
