SUBROUTINE msinv( xmat, n, ierr )

!***********************************************************************
!     - msinv -
!     computes the inverse of a symmetric positive definite matrix
!***********************************************************************
!
!     XMAT  - INPUT . The original matrix, of order N         - REAL*8
!                     (in triangular form)
!             OUTPUT. Resultant inverted matrix
!                     (in triangular form)
!     N     - INPUT . Order of XMAT                           - INTEGER
!     IERR  - OUTPUT. 0 if no error                           - INTEGER
!                     <0 : the abs(IERR)th pivot block is
!                          singular, computations are stopped
!                     >0 : XMAT was not positive definite:
!                          IERR returns the number of
!                          non-positive eigenvalues
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        DSPDIX  DSPFA  (adapted from LINPACK library)
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: n
   REAL(KIND=8), INTENT(INOUT) :: xmat(n*(n+1)/2)
   INTEGER, INTENT(OUT) :: ierr

   !  Local declarations
   !  ------------------

   REAL(KIND=8) :: det(2), work(n)
   INTEGER :: iner(3), kpvt(n)

   !--------------------------------------------------------------------

   ! Factor xmat
   ! -----------
   call dspfa( xmat, n, kpvt, ierr )
   if( ierr .ne. 0 )then
      ierr = -ierr
      return
   endif

   ! Compute inverse
   ! ---------------
   call dspdix( xmat, n, kpvt, det, iner, work, 101 )
   ierr = n - iner(1)

END SUBROUTINE msinv
