SUBROUTINE mtaas( a, s, m, n )

!***********************************************************************
!     - mtaas -
!     pre-multiplies general matrix A by its transpose to form
!     resultant symmetric matrix S (stored in triangular form)
!***********************************************************************
!
!     A     - INPUT . General matrix (columnwise)             - REAL*8
!     S     - OUTPUT. Resultant symmetric matrix              - REAL*8
!                     (in triangular form)
!     M     - INPUT . Number of rows in A                     - INTEGER
!     N     - INPUT . Number of columns in A, of rows and     - INTEGER
!                     columns in S
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        SUM
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: m, n
   REAL(KIND=8), INTENT(IN) :: a(m*n)
   REAL(KIND=8), INTENT(OUT) :: s(n*(n+1)/2)

   !  Local declarations
   !  ------------------

   INTEGER :: i, is, j, ki, kj

   !--------------------------------------------------------------------

   is = 0
   ki = 0

   do i = 1, n
      kj = 0

      do j = 1, i
         is  = is + 1
         s(is) = sum( a(ki+1:ki+m) * a(kj+1:kj+m) )
         kj = kj + m
      end do

      ki = ki + m
   end do

END SUBROUTINE mtaas
