SUBROUTINE mtasaz( a, s, z, m, n )

!***********************************************************************
!     - mtasaz -
!     computes the product  transpose(A) * S * A  where A is a general
!     matrix, S a symmetric matrix (stored in triangular form) and
!     result is a symmetric matrix Z (stored in triangular form)
!***********************************************************************
!
!     A     - INPUT . General matrix (columnwise)             - REAL*8
!     S     - INPUT . Symmetric matrix (in triangular form)   - REAL*8
!     Z     - OUTPUT. Resultant symmetric matrix              - REAL*8
!                     (in triangular form)
!     M     - INPUT . Number of rows in A, of rows and        - INTEGER
!                     columns in S
!     N     - INPUT . Number of columns in A, of rows and     - INTEGER
!                     columns in Z
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        SUM
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: m, n
   REAL(KIND=8), INTENT(IN) :: a(m*n), s(m*(m+1)/2)
   REAL(KIND=8), INTENT(OUT) :: z(n*(n+1)/2)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   REAL(KIND=8) :: sumx, tot
   INTEGER :: i, ij, im, j, jm, k, kl, l, lk

   !--------------------------------------------------------------------

   ij = 0
   im = 0

   do i = 1, n
      jm = 0

      do j = 1, i
         tot = zero
         lk = 0

         do l = 1, m
            sumx = sum( a(im+1:im+l) * s(lk+1:lk+l) )
            lk = lk + l
            kl = lk

            do k = l+1, m
               sumx = sumx + a(im+k) * s(kl+l)
               kl = kl + k
            end do

            tot = tot  +  sumx * a(l+jm)
         end do

         jm = jm + m
         ij = ij + 1
         z(ij) = tot
      end do

      im = im + m
   end do

END SUBROUTINE mtasaz
