SUBROUTINE mxbs( x, b, s, m, n )

!***********************************************************************
!     - mxbs -
!     multiplies the rectangular part of subset X of a symmetric matrix
!     (stored in triangular form) by general matrix B to form resultant
!     symmetric matrix S (stored in triangular form)
!***********************************************************************
!
!     X     - INPUT . Subset of a symmetric matrix            - REAL*8
!                     (in partial triangular form)
!     B     - INPUT . General matrix (columnwise)             - REAL*8
!     S     - OUTPUT. Resultant symmetric matrix              - REAL*8
!                     (in triangular form)
!     M     - INPUT . Number of rows in B                     - INTEGER
!     N     - INPUT . Number of columns in B,                 - INTEGER
!                     of rows and columns in S
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------
!
!     Explanation :
!     X is a subset of a symmetric matrix (stored in triangular form,
!     of order at least N + M), which has been transmitted from
!     location  N*(N+1)/2 + 1 in calling module (thus X has partial
!     triangular form with length (N+M)*(N+M+1)/2 - N*(N+1)/2).
!     Let  M = 4  and  N = 3, for example. With numbers representing
!     the indices of the elements, X will be:
!         1  5 10 16
!         2  6 11 17
!         3  7 12 18
!         4  8 13 19
!            9 14 20
!              15 21
!                 22
!     Then this subroutine premultiplies matrix B by the rectangular
!     matrix delimited by indices 1, 16, 18 and 3.
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: m, n
   REAL(KIND=8), INTENT(IN) :: x(m*(m+n+n+1)/2), b(m*n)
   REAL(KIND=8), INTENT(OUT) :: s(n*(n+1)/2)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   REAL(KIND=8) :: sumx
   INTEGER :: i, ik, is, j, k, kj

   !--------------------------------------------------------------------

   is = 0

   do i = 1, n
      kj  = -m

      do j = 1, i
         is = is + 1
         kj = kj + m
         ik = i
         sumx = zero

         do k = 1, m
            sumx = sumx + x(ik) * b(k+kj)
            ik  = ik + n + k
         end do

         s(is) = sumx
      end do

   end do

END SUBROUTINE mxbs
