SUBROUTINE prtmat( iout, lchk, pmat, drow, nrow, ncol, hrow, hcol )

!***********************************************************************
!     - prtmat -
!     PRinTs a MATrix (with row and column labels)
!***********************************************************************
!
!     IOUT  - INPUT . Unit number for printing                - INTEGER
!     LCHK  - INPUT . If .false. no call to CHKPG             - LOGICAL
!     PMAT  - INPUT . The matrix to be printed                - REAL*8
!     DROW  - INPUT . Number of rows used to dimension PMAT   - INTEGER
!                     in the calling module
!     NROW  - INPUT . Number of rows in PMAT to be used       - INTEGER
!                     and number of elements in HROW
!     NCOL  - INPUT . Number of columns in PMAT               - INTEGER
!                     and number of elements in HCOL
!     HROW  - INPUT . PMAT row labels    (8 characters long)  - CHAR
!     HCOL  - INPUT . PMAT column labels (8 characters long)  - CHAR
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG
!     External functions called:
!        NONE
!     Fortran  functions called:
!        MIN
!
!-----------------------------------------------------------------------

   USE gmpint

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: iout, drow, nrow, ncol
   LOGICAL, INTENT(IN) :: lchk
   REAL(KIND=8), INTENT(IN) :: pmat(drow,ncol)
   CHARACTER(LEN=*), INTENT(IN) :: hrow(nrow), hcol(ncol)

   !  Local declarations
   !  ------------------

   INTEGER :: i, l, linpp, ll, maxil, nbeg, nend, nladd, nlr, nmor
   LOGICAL :: more, newp

   !--------------------------------------------------------------------

      ! Maximum number of items per line (format g14, 10 col. skipped)
      ! Note that hrow and hcol are assumed to be 8 characters long
   maxil = (lrecp-10-1) / 14

   if ( lchk ) then
      linpp = linep
   else
      linpp = 99999
   end if

   more = .false.
   nmor = 2
   nend = 0
   do      ! Loop while nend < ncol
      nbeg = nend + 1
      nend = min( nend+maxil, ncol )
      if ( lchk ) call chkpg( nmor+1, .false., newp )
      if ( more ) write(iout,111) ' '
      write(iout,111) (hcol(i)   ,i=nbeg,nend)
      write(iout,111) ('--------',i=nbeg,nend)
      lines = lines + nmor
         ! nlr  = no. of lines still to be printed
         ! nladd= no. of lines added in current page
      nlr = nrow
      ll  = 0
      do      ! Loop while nlr > 0
         nladd = min( nlr, linpp-lines )
         do l = 1, nladd
            ll = ll + 1
            write(iout,222) hrow(ll), (pmat(ll,i),i=nbeg,nend)
         end do
         lines = lines + nladd
         nlr   = nlr   - nladd
         if ( nlr <= 0 ) exit
         if ( lchk ) call chkpg( 0, .true., newp )
      end do

      more = .true.
      nmor = 3
      if ( nend >= ncol ) exit
   end do

   111 format(10x,10(2x,a8,4x))
   222 format(1x,a8,1x,10g14.7)

END SUBROUTINE prtmat
