      CHARACTER*(*) FUNCTION RFMTA ( NC    , NN    , X     , ZERTOL )
!***********************************************************************
!     - RFMTA -
!     format adjustment according to the value of X (real type) :
!     . if 10**NN<|X|<10**6, F-type format with 6 characters and
!     centered in an area of NC characters : F7+n.m (n and m computed
!     from the value of X)
!     . if not, E-type : Enc.nc-7
!     The character string receiving RFMTA must be at least 6 characters
!     long and NC must be > 7
!***********************************************************************
!
!     NC     - INPUT . Number of characters for printing of X - INTEGER
!                      (must be > 7)
!     NN     - INPUT . (must be stictly lower than NC)        - INTEGER
!                      the upper limit for power of 10 under  - INTEGER
!                      which E-type format is used rather than
!                      F-type. Normally NN is negative and
!                      serves to avoid only zeros printed when
!                      X is small in absolute value (e.g. NN=-4
!                      means E-type is used if |X| < 10**-4 )
!                      NN is inactive if |X| < ZERTOL
!     X      - INPUT . The value to be printed                - REAL
!     ZERTOL - INPUT . A tolerance for zero : if |X| < ZERTOL - REAL
!                      X is printed with F-type format. This
!                      is usefull when one prefers having, e.g.
!                      3*10**-31, printed as 0.000000 rather
!                      than 0.3E-30
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        ABS  LOG10  MAX  SIGN
!
!-----------------------------------------------------------------------
!
!
!        ARGUMENTS
!        ---------
      REAL              X     , ZERTOL
      INTEGER           NC    , NN
!
!        LOCAL DECLARATIONS
!        ------------------
      REAL              ALGX  , ONEPLS, ZERO
      INTEGER           IDEC  , LGX
      CHARACTER*6       TEMP
!
      PARAMETER (ONEPLS=1.0E0+1.0E-6, ZERO=0.0E0)
!        ONEPLS must be slightly greater than 1, in order to
!        counter-balance such things as log10(100) = 1.99999,
!        so that Int( ONEPLS + log10(100) ) = 3
!
!        Note that TEMP is used to avoid direct writing into RFMTA
!        which causes some trouble with some compilers
!
!     ------------------------------------------------------------------
!
      ALGX = LOG10( MAX( ABS(X), ZERTOL) )
      LGX  = ALGX + SIGN( ONEPLS, ALGX )
      IDEC = MAX( 1, LGX )
      IF( IDEC+2 .GT. NC .OR. LGX .GT.5 .OR. LGX .LE. NN )THEN
         WRITE(TEMP,1) 'E', NC, NC-7
      else
         WRITE(TEMP,1) 'F', NC/2+4, 6-IDEC
!                          (NC/2+4 = (NC-7-1)/2 + 7+1)
      end if
      RFMTA = TEMP
!
    1 FORMAT(A1,I2,'.',I2)
!
      END
