SUBROUTINE TQL2X( N, A, D, E, Z, VCTR, IERR )

! ----------------------------------------------------------------------
!     THIS IS A MODIFIED VERSION OF SUBROUTINE TQL2 FROM EISPACK PACKAGE
!
!     MAINLY, THE EISPACK SUBROUTINES TRED3 AND TRBAK3 HAVE BEEN
!     INCLUDED IN TQL2 AND SOME ARGUMENTS HAVE BEEN SUPRESSED
! ----------------------------------------------------------------------
!
!     ON INPUT
!
!     N    IS THE ORDER OF THE MATRIX.
!
!     VCTR MUST BE SET TO .TRUE. IF EIGENVECTORS ARE DESIRED,
!          TO .FALSE. OTHERWISE
!
!     A    CONTAINS THE LOWER TRIANGLE OF THE REAL SYMMETRIC
!          INPUT MATRIX, STORED ROW-WISE AS A ONE-DIMENSIONAL
!          ARRAY, IN ITS FIRST N*(N+1)/2 POSITIONS.
!
!     E    WORK VECTOR OF LENGTH N
!
!     Z    IF VCTR IS .TRUE., THEN Z MUST BE A N*N MATRIX,
!          OTHERWISE IT WILL NOT BE USED
!
!     ON OUTPUT
!
!     A    IS DESTROYED
!
!     D    CONTAINS THE EIGENVALUES IN DESCENDING ORDER.  IF AN
!          ERROR EXIT IS MADE, THE EIGENVALUES ARE CORRECT BUT
!          UNORDERED FOR INDICES 1,2,...,IERR-1.
!
!     Z    IF VCTR IS .TRUE., THEN Z CONTAINS EIGENVECTORS OF
!          THE SYMMETRIC MATRIX.
!          IF AN ERROR EXIT IS MADE, Z CONTAINS THE EIGENVECTORS
!          ASSOCIATED WITH THE STORED EIGENVALUES
!
!     IERR IS SET TO
!          ZERO      FOR NORMAL RETURN,
!          J         IF THE J-TH EIGENVALUE HAS NOT BEEN
!                    DETERMINED AFTER 30 ITERATIONS.
!
!     NOTE THAT TRBAK3-SECTION PRESERVES VECTOR EUCLIDEAN NORMS.
!
!     CALLS PYTHAG FOR  DSQRT(A*A + B*B) .
!
! ----------------------------------------------------------------------

      LOGICAL :: VCTR
      INTEGER :: I,IERR,II,IK,IZ,J,JK,K,L,L1,L2,M,N
      real(kind=8) :: a(N*(N+1)/2),D(N),E(N),Z(N,N)
      real(kind=8) :: C,C2,C3,DL1,EL1,F,G,H,HH,P,R,S,SCALE,S2,TST1,TST2
      real(kind=8), external :: PYTHAG
!     FORTRAN FUNCTIONS: DOUBLE PRECISION abs,sign,sqrt

      IERR = 0
!        SPECIAL CASE
      IF (N .EQ. 1)THEN
         D(1) = A(1)
         IF( VCTR ) Z(1,1) = 1.0_8
         RETURN
      ENDIF

!     ------------------------------------------------------------------
!     - THIS SECTION CORRESPONDS TO SUBROUTINE TRED3
!     ------------------------------------------------------------------
!
!     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE TRED3,
!     NUM. MATH. 11, 181-195(1968) BY MARTIN, REINSCH, AND WILKINSON.
!     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 212-226(1971).
!
!     THIS SUBROUTINE REDUCES A REAL SYMMETRIC MATRIX, STORED AS
!     A ONE-DIMENSIONAL ARRAY, TO A SYMMETRIC TRIDIAGONAL MATRIX
!     USING ORTHOGONAL SIMILARITY TRANSFORMATIONS.
!
!     ON INPUT
!
!        N  IS THE ORDER OF THE MATRIX.
!
!        NV MUST BE SET TO THE DIMENSION OF THE ARRAY PARAMETER A
!           AS DECLARED IN THE CALLING PROGRAM DIMENSION STATEMENT.
!
!        A  CONTAINS THE LOWER TRIANGLE OF THE REAL SYMMETRIC
!           INPUT MATRIX, STORED ROW-WISE AS A ONE-DIMENSIONAL
!           ARRAY, IN ITS FIRST N*(N+1)/2 POSITIONS.
!
!     ON OUTPUT
!
!        A  CONTAINS INFORMATION ABOUT THE ORTHOGONAL
!           TRANSFORMATIONS USED IN THE REDUCTION.
!
!        D  CONTAINS THE DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX.
!
!        E  CONTAINS THE SUBDIAGONAL ELEMENTS OF THE TRIDIAGONAL
!           MATRIX IN ITS LAST N-1 POSITIONS.  E(1) IS SET TO ZERO.
!
!        E2 CONTAINS THE SQUARES OF THE CORRESPONDING ELEMENTS OF E.
!           E2 MAY COINCIDE WITH E IF THE SQUARES ARE NOT NEEDED.
!
!     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
!     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
!
!     THIS VERSION DATED AUGUST 1983.
!
!     ------------------------------------------------------------------

!     .......... FOR I=N STEP -1 UNTIL 1 DO -- ..........
      DO 300 I = N, 1, -1
         L = I - 1
         IZ = (I * L) / 2
         H = 0.0_8
         SCALE = 0.0_8
         IF (L .LT. 1) GO TO 130
!     .......... SCALE ROW (ALGOL TOL THEN NOT NEEDED) ..........
         DO 120 K = 1, L
            IZ = IZ + 1
            D(K) = A(IZ)
            SCALE = SCALE + abs(D(K))
  120    CONTINUE

         IF (SCALE .NE. 0.0_8) GO TO 140
  130    E(I) = 0.0_8
!!!!     E2(I) = 0.0_8
         GO TO 290

  140    DO 150 K = 1, L
            D(K) = D(K) / SCALE
            H = H + D(K) * D(K)
  150    CONTINUE

!!!!     E2(I) = SCALE * SCALE * H
         F = D(L)
         G = -sign(sqrt(H),F)
         E(I) = SCALE * G
         H = H - F * G
         D(L) = F - G
         A(IZ) = SCALE * D(L)
         IF (L .EQ. 1) GO TO 290
         E(1) = A(1) * D(1)
         JK = 2

         DO 240 J = 2, L
            F = D(J)
            G = 0.0_8

            DO 200 K = 1, J-1
               G = G + A(JK) * D(K)
               E(K) = E(K) + A(JK) * F
               JK = JK + 1
  200       CONTINUE

            E(J) = G + A(JK) * F
            JK = JK + 1
  240    CONTINUE
!     .......... FORM P ..........
         F = 0.0_8

         DO 245 J = 1, L
            E(J) = E(J) / H
            F = F + E(J) * D(J)
  245    CONTINUE

         HH = F / (H + H)
!     .......... FORM Q ..........
         DO 250 J = 1, L
  250    E(J) = E(J) - HH * D(J)

         JK = 1
!     .......... FORM REDUCED A ..........
         DO 280 J = 1, L
            F = D(J)
            G = E(J)

            DO 260 K = 1, J
               A(JK) = A(JK) - F * E(K) - G * D(K)
               JK = JK + 1
  260       CONTINUE

  280    CONTINUE

  290    D(I) = A(IZ+1)
         A(IZ+1) = SCALE * sqrt(H)
  300 CONTINUE

!     ------------------------------------------------------------------
!     - THIS SECTION CORRESPONDS TO SUBROUTINE TQL2
!     ------------------------------------------------------------------
!
!     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE TQL2,
!     NUM. MATH. 11, 293-306(1968) BY BOWDLER, MARTIN, REINSCH, AND
!     WILKINSON.
!     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 227-240(1971).
!
!     THIS SUBROUTINE FINDS THE EIGENVALUES AND EIGENVECTORS
!     OF A SYMMETRIC TRIDIAGONAL MATRIX BY THE QL METHOD.
!     THE EIGENVECTORS OF A FULL SYMMETRIC MATRIX CAN ALSO
!     BE FOUND IF  TRED2  HAS BEEN USED TO REDUCE THIS
!     FULL MATRIX TO TRIDIAGONAL FORM.
!
!     ON INPUT
!
!        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
!           ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
!           DIMENSION STATEMENT.
!
!        N  IS THE ORDER OF THE MATRIX.
!
!        D  CONTAINS THE DIAGONAL ELEMENTS OF THE INPUT MATRIX.
!
!        E  CONTAINS THE SUBDIAGONAL ELEMENTS OF THE INPUT MATRIX
!           IN ITS LAST N-1 POSITIONS.  E(1) IS ARBITRARY.
!
!        Z  CONTAINS THE TRANSFORMATION MATRIX PRODUCED IN THE
!           REDUCTION BY  TRED2, IF PERFORMED.  IF THE EIGENVECTORS
!           OF THE TRIDIAGONAL MATRIX ARE DESIRED, Z MUST CONTAIN
!           THE IDENTITY MATRIX.
!
!      ON OUTPUT
!
!        D  CONTAINS THE EIGENVALUES IN DESCENDING ORDER.  IF AN
!           ERROR EXIT IS MADE, THE EIGENVALUES ARE CORRECT BUT
!           UNORDERED FOR INDICES 1,2,...,IERR-1.
!
!        E  HAS BEEN DESTROYED.
!
!        Z  CONTAINS ORTHONORMAL EIGENVECTORS OF THE SYMMETRIC
!           TRIDIAGONAL (OR FULL) MATRIX.  IF AN ERROR EXIT IS MADE,
!           Z CONTAINS THE EIGENVECTORS ASSOCIATED WITH THE STORED
!           EIGENVALUES.
!
!        IERR IS SET TO
!           ZERO       FOR NORMAL RETURN,
!           J          IF THE J-TH EIGENVALUE HAS NOT BEEN
!                      DETERMINED AFTER 30 ITERATIONS.
!
!     CALLS PYTHAG FOR  DSQRT(A*A + B*B) .
!
!     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
!     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
!
!     THIS VERSION DATED AUGUST 1983.
!
!     ------------------------------------------------------------------

!     .......... FORM IDENTITY MATRIX ..........
      DO 1100 J = 1, N-1
         E(J) = E(J+1)
         IF ( VCTR ) THEN
            DO 1010 I = J+1, N
               Z(I,J) = 0.0_8
               Z(J,I) = 0.0_8
 1010       CONTINUE
            Z(J,J) = 1.0_8
         ENDIF
 1100 CONTINUE
      E(N) = 0.0_8
      IF ( VCTR ) Z(N,N) = 1.0_8

      F = 0.0_8
      TST1 = 0.0_8

      DO 1240 L = 1, N
         J = 0
         H = abs(D(L)) + abs(E(L))
         IF (TST1 .LT. H) TST1 = H
!     .......... LOOK FOR SMALL SUB-DIAGONAL ELEMENT ..........
         DO 1110 M = L, N
            TST2 = TST1 + abs(E(M))
            IF (TST2 .EQ. TST1) GO TO 1120
!     .......... E(N) IS ALWAYS ZERO, SO THERE IS NO EXIT
!                THROUGH THE BOTTOM OF THE LOOP ..........
 1110    CONTINUE

 1120    IF (M .EQ. L) GO TO 1220
 1130    IF (J .EQ. 30) THEN
!     .......... SET ERROR -- NO CONVERGENCE TO AN
!                EIGENVALUE AFTER 30 ITERATIONS ..........
            IERR = L
            GOTO 2000
         ENDIF
         J = J + 1
!     .......... FORM SHIFT ..........
         L1 = L + 1
         L2 = L1 + 1
         G = D(L)
         P = (D(L1) - G) / (E(L) + E(L))
         R = PYTHAG(P,1.0_8)
         D(L) = E(L) / (P + sign(R,P))
         D(L1) = E(L) * (P + sign(R,P))
         DL1 = D(L1)
         H = G - D(L)
         IF (L2 .GT. N) GO TO 1145

         DO 1140 I = L2, N
 1140    D(I) = D(I) - H

 1145    F = F + H
!     .......... QL TRANSFORMATION ..........
         P = D(M)
         C = 1.0_8
         C2 = C
         EL1 = E(L1)
         S = 0.0_8
!     .......... FOR I=M-1 STEP -1 UNTIL L DO -- ..........
         DO 1200 I = M-1, L, -1
            C3 = C2
            C2 = C
            S2 = S
            G = C * E(I)
            H = C * P
            R = PYTHAG(P,E(I))
            E(I+1) = S * R
            S = E(I) / R
            C = P / R
            P = C * D(I) - S * G
            D(I+1) = H + S * (C * G + S * D(I))
!     .......... FORM VECTOR ..........
            IF ( VCTR ) THEN
               DO 1180 K = 1, N
                  H = Z(K,I+1)
                  Z(K,I+1) = S * Z(K,I) + C * H
                  Z(K,I) = C * Z(K,I) - S * H
 1180          CONTINUE
            ENDIF

 1200    CONTINUE

         P = -S * S2 * C3 * EL1 * E(L) / DL1
         E(L) = S * P
         D(L) = C * P
         TST2 = TST1 + abs(E(L))
         IF (TST2 .GT. TST1) GO TO 1130
 1220    D(L) = D(L) + F
 1240 CONTINUE
!     .......... ORDER EIGENVALUES AND EIGENVECTORS ..........
      DO 1300 II = 2, N
         I = II - 1
         K = I
         P = D(I)

         DO 1260 J = II, N
            IF (D(J) .LE. P) GO TO 1260
            K = J
            P = D(J)
 1260    CONTINUE

         IF (K .EQ. I) GO TO 1300
         D(K) = D(I)
         D(I) = P

         IF ( VCTR ) THEN
            DO 1280 J = 1, N
               P = Z(J,I)
               Z(J,I) = Z(J,K)
               Z(J,K) = P
 1280       CONTINUE
         ENDIF

 1300 CONTINUE

 2000 CONTINUE
      IF ( .NOT. VCTR ) RETURN

!     ------------------------------------------------------------------
!     - THIS SECTION CORRESPONDS TO SUBROUTINE TRBAK3
!     ------------------------------------------------------------------
!
!     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE TRBAK3,
!     NUM. MATH. 11, 181-195(1968) BY MARTIN, REINSCH, AND WILKINSON.
!     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 212-226(1971).
!
!     THIS SUBROUTINE FORMS THE EIGENVECTORS OF A REAL SYMMETRIC
!     MATRIX BY BACK TRANSFORMING THOSE OF THE CORRESPONDING
!     SYMMETRIC TRIDIAGONAL MATRIX DETERMINED BY  TRED3.
!
!     ON INPUT
!
!        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
!          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
!          DIMENSION STATEMENT.
!
!        N IS THE ORDER OF THE MATRIX.
!
!        NV MUST BE SET TO THE DIMENSION OF THE ARRAY PARAMETER A
!          AS DECLARED IN THE CALLING PROGRAM DIMENSION STATEMENT.
!
!        A CONTAINS INFORMATION ABOUT THE ORTHOGONAL TRANSFORMATIONS
!          USED IN THE REDUCTION BY  TRED3  IN ITS FIRST
!          N*(N+1)/2 POSITIONS.
!
!        M IS THE NUMBER OF EIGENVECTORS TO BE BACK TRANSFORMED.
!
!        Z CONTAINS THE EIGENVECTORS TO BE BACK TRANSFORMED
!          IN ITS FIRST M COLUMNS.
!
!     ON OUTPUT
!
!        Z CONTAINS THE TRANSFORMED EIGENVECTORS
!          IN ITS FIRST M COLUMNS.
!
!     NOTE THAT TRBAK3 PRESERVES VECTOR EUCLIDEAN NORMS.
!
!     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
!     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
!
!     THIS VERSION DATED AUGUST 1983.
!
!     ------------------------------------------------------------------

      IF (IERR .EQ. 0) THEN
         M = N
      else
         M = IERR - 1
      ENDIF
      IF (M .EQ. 0) RETURN

      IZ = 0

      DO 2140 I = 2, N
         L = I - 1
         IZ = IZ + L
         IK = IZ + I
         H = A(IK)
         IF (H .NE. 0.0_8) THEN

            DO 2130 J = 1, M
               S = 0.0_8
               IK = IZ

               DO 2110 K = 1, L
                  IK = IK + 1
                  S = S + A(IK) * Z(K,J)
 2110          CONTINUE
!     .......... DOUBLE DIVISION AVOIDS POSSIBLE UNDERFLOW ..........
               S = (S / H) / H
               IK = IZ

               DO 2120 K = 1, L
                  IK = IK + 1
                  Z(K,J) = Z(K,J) - S * A(IK)
 2120          CONTINUE

 2130       CONTINUE

         ENDIF
 2140 CONTINUE

END SUBROUTINE TQL2X
