SUBROUTINE chkpg( nladd, new, newp )

!***********************************************************************
!     - chkpg -
!     checks a new page is necessary for next output on unit imp
!     if yes, page count is updated and line count is reset to 0 (screen)
!     or 2 (printer).
!     if screen output, "new page" means a prompt to user on unit imp
!     and possible screen-clearing (if subroutine sline0 is available)
!***********************************************************************
!
!     NLADD - INPUT . Number of new lines to be added on unit - INTEGER
!                     number IMP
!     NEW   - INPUT . If .true.  a new page (or "new screen") - LOGICAL
!                     is got whatever the value of NLADD,
!                     if .false. a new page (or "new screen")
!                     is got only if total no. of lines to be
!                     printed in current page (LINES + NLADD)
!                     would exceed the maximum allowed (LINEP)
!     NEWP  - OUTPUT. .true. if a new page (or "new screen")  - LOGICAL
!                     was got
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines  page
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        SLINE0 (screen clearing; machine dependent)
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE gmplgl
   USE termnl

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN)  :: nladd
   LOGICAL, INTENT(IN)  :: new
   LOGICAL, INTENT(OUT) :: newp

   !  Local declarations
   !  ------------------

   CHARACTER(LEN=1) :: rep

   !--------------------------------------------------------------------

   newp = .false.

   if ( new .or. lines+nladd > linep ) then
      newp = .true.
      page = page + 1
      if ( linter ) then
         ! Prompt user on unit imp to press "enter" key
         write(imp,1) ' .../...'
         read(lec1,1) rep   ! Get response from unit lec1 (standard unit)
         call sline0   ! Clear screen
         lines = 0
      else
         ! Get a new page on printer
         write(imp,1) '1Page', page, ' - ', titl2(:lrecp-12)
         lines = 1
      endif

   endif

   1 format(a,i4,2a)

END SUBROUTINE chkpg
