CHARACTER(LEN=6) FUNCTION dfmtc( nc, nn, x, zertol )

!***********************************************************************
!     - dfmtc -
!     adjusted format according to the value of x (real*8 type):
!     . if 10**nn<|x|<10**(nc-1) (or <10**(nc-2) if x<0) f-type format:
!     fnc.m (m derived from value of x)
!     . otherwise, e-type format: enc.nc-m (m derived from range of x)
!     The character string receiving dfmtc must be at least 6 characters
!     long and nc must be >= 8
!***********************************************************************
!
!     NC     - INPUT . Number of characters for printing of X - INTEGER
!                      (must be >= 8)
!     NN     - INPUT . (must be stictly lower than NC)        - INTEGER
!                      the upper limit for power of 10 under  - INTEGER
!                      which E-type format is used rather than
!                      F-type. Normally NN is negative and
!                      serves to avoid having only zeros printed
!                      when X is small in absolute value (e.g.
!                      NN=-4 means E-type is used if |X|<10**-4)
!                      NN is inactive if |X| <= ZERTOL
!     X      - INPUT . The value to be printed                - REAL*8.
!     ZERTOL - INPUT . A tolerance for zero: if |X| <= ZERTOL - REAL*8.
!                      X is printed with F-type format. This
!                      is usefull when one prefers having, e.g.
!                      3*10**-51, printed as 0.000000 rather
!                      than 0.3E-50
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        ABS  LOG10  MAX  SIGN
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   REAL(KIND=8), INTENT(IN) :: x, zertol
   INTEGER, INTENT(IN) :: nc, nn

   !  Local declarations
   !  ------------------

   REAL(KIND=8) :: ax, algx
   REAL(KIND=8), PARAMETER :: zero=0.0_8
   REAL(KIND=8), PARAMETER :: onepls=1.0_8+1.0e-12_8
   INTEGER :: idec, ineg, lgx
   CHARACTER(LEN=6) :: temp

   ! onepls must be slightly greater than 1, in order to
   ! counter-balance such things as log10(100) = 1.999999999999,
   ! so that int( onepls + log10(100) ) = 3

   ! Temp is used to avoid direct writing into dfmtc which causes
   ! some trouble with some compilers

   !--------------------------------------------------------------------

   if ( x < zero ) then
      ineg = 1
   else
      ineg = 0
   end if
   ax = abs(x)
   if ( ax <= abs(zertol) ) then
      write(temp,1) 'f', nc, nc-2-ineg
   else
      algx = log10(ax)
      lgx  = algx + sign( onepls, algx )
      idec = max(1,lgx) + 1 + ineg
      if ( idec > nc .or. lgx+1 <= nn ) then
         idec = log10(abs(algx)) + onepls
         if ( algx < 0 ) idec = idec + 1
         write(temp,1) 'e', nc, nc-(4+max(3,idec))
      else
         write(temp,1) 'f', nc, nc-idec
      end if
   end if
   dfmtc = temp

   1 format(a1,i2,'.',i2)

END FUNCTION dfmtc
