SUBROUTINE filopn( kode, unt, filnam, stat, frmtd, rdonly, lerr )

!***********************************************************************
!     - filopn -
!     opens sequential external file
!***********************************************************************
!
!     KODE  - INPUT . Code number coding calling module       - INTEGER
!                     and/or special action to be taken.
!                     The codes for modules of the general
!                     section are:
!                      0 : unformatted scratch file
!                     12 : GMPCOM - command file
!                     21 : GMPPRO - input data file
!                     31 : GMPSAV - save file
!     UNT   - INPUT . Unit number                             - INTEGER
!     FILNAM- INPUT . File specifier.                         - CHAR
!                     If 1st character is blank, no file
!                     specifier will be used in OPEN statement
!     STAT  - INPUT . Status specifier                        - CHAR
!     FRMTD - INPUT . Form specifier                          - CHAR
!     RDONLY- INPUT . .true. if file is to be read only,      - LOGICAL
!                     .false. otherwise
!     LERR  - OUTPUT. .true. if error when opening file       - LOGICAL
!
!     --------------------------------------------------------
!     This routine may be machine dependent because some
!     compilers offer usefull extensions to the norm for the
!     OPEN statement.
!     For instance the specifier ACTION is not a standard one.
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        FILERR
!     External functions called:
!        NONE
!     Fortran  functions called:
!        LEN
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: kode, unt
   LOGICAL, INTENT(IN) :: rdonly
   CHARACTER(LEN=*), INTENT(IN) :: filnam, stat, frmtd
   LOGICAL, INTENT(OUT) :: lerr

   !  Local declarations
   !  ------------------

   INTEGER :: io_status
   INCLUDE 'gmpopnp.'

   !--------------------------------------------------------------------

   lerr = .false.

   if ( len(stat) >= 7 .and. stat(1:7) == 'SCRATCH' ) then
         ! Scratch file
      open(unit=unt,iostat=io_status,status='SCRATCH',form=frmtd)
!!    open(unit=unt,iostat=io_status,status='SCRATCH',form=frmtd,recl=16000)

   else if ( rdonly ) then

      if ( filnam(1:1) /= ' ' ) then
         open(unit=unt,iostat=io_status,file=filnam,status=stat,form=frmtd,action='READ')
!!       open(unit=unt,iostat=io_status,file=filnam,status=stat,form=frmtd,action='READ,DENYNONE') ! Lahey77
!!       open(unit=unt,iostat=io_status,file=filnam,status=stat,form=frmtd,action='READ',share='DENYNONE') ! Watcom
      else
         open(unit=unt,iostat=io_status,status=stat,form=frmtd,action='READ')
!!       open(unit=unt,iostat=io_status,status=stat,form=frmtd,action='READ,DENYNONE') ! Lahey77
!!       open(unit=unt,iostat=io_status,status=stat,form=frmtd,action='READ',share='DENYNONE') ! Watcom
      end if

   else

      if ( filnam(1:1) /= ' ' ) then
         open(unit=unt,iostat=io_status,file=filnam,status=stat,form=frmtd)
      else
         open(unit=unt,iostat=io_status,status=stat,form=frmtd)
      end if

   end if

   if ( io_status > 0 ) then   ! Open failed
                               ! Print message and set lerr
      call filerr( opn, unt, filnam, frmtd, 'SEQUENTIAL', stat )
      lerr = .true.
   end if

END SUBROUTINE filopn
