SUBROUTINE gmpcat( k1, kn, lerr, nvals )

!***********************************************************************
!     - gmpcat -
!     analyzes CATEG paragraph, with commands filtering
!     allocates arrays vals and noms
!***********************************************************************
!
!     K1    - INPUT . Starting location in CAR of the zone    - INTEGER
!                     analyzed. unchanged on return
!     KN    - INPUT . Ending   location in CAR of the zone    - INTEGER
!                     analyzed (end-of-paragraph symbol)
!             OUTPUT. Destroyed
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!     NVALS - INPUT . Initial length of VALS and NOMS         - INTEGER
!             OUTPUT. Updated length of VALS and NOMS
!
!     --------------------------------------------------------
!     Module elements modified:
!     carl   : car
!     gmpint : lines
!     memst1 : vals  lval  lnom
!     memsta : noms
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        AVECTY  CHKPG  CMPRSS  FILTER  GMP999  GMPWRN
!     External functions called:
!        DFMTC  LSCALX
!     Fortran  functions called:
!        ABS  ALLOCATE  ALLOCATED  DEALLOCATE  MAX  MIN  MOD  NINT
!
!-----------------------------------------------------------------------

   USE carl
   USE gmpint
   USE gmplgl
   USE memst1
   USE memsta
   USE spcval

   IMPLICIT NONE

   !  Interface block
   !  ---------------

   INTERFACE
      SUBROUTINE avecty( mot, longm, long, ideb, iend, kode, &
                         nomv, iv, minv, maxv, nind, maxx,   &
                         nx, icol, jcol, lerr,               &
                         dx, rx, ix, lx, cx )
         IMPLICIT NONE
         INTEGER, INTENT(IN)  :: longm, long, ideb, iend, kode, minv, maxv, maxx
         CHARACTER(LEN=*), INTENT(IN) :: mot
         CHARACTER(LEN=*), DIMENSION(minv:maxv), INTENT(IN) :: nomv
         INTEGER, INTENT(OUT) :: nind, nx, icol, jcol
         LOGICAL, INTENT(OUT) :: lerr
         INTEGER, DIMENSION(maxv-minv+1), INTENT(OUT) :: iv
         REAL(KIND=8)    , DIMENSION(maxx), INTENT(OUT), OPTIONAL :: dx
         REAL            , DIMENSION(maxx), INTENT(OUT), OPTIONAL :: rx
         INTEGER         , DIMENSION(maxx), INTENT(OUT), OPTIONAL :: ix
         LOGICAL         , DIMENSION(maxx), INTENT(OUT), OPTIONAL :: lx
         CHARACTER(LEN=*), DIMENSION(maxx), INTENT(OUT), OPTIONAL :: cx
      END SUBROUTINE avecty
   END INTERFACE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: k1
   INTEGER, INTENT(INOUT) :: kn, nvals
   LOGICAL, INTENT(OUT) :: lerr

   !  Local declarations
   !  ------------------

   LOGICAL, EXTERNAL :: lscalx
   CHARACTER(LEN=6), EXTERNAL :: dfmtc

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   INTEGER, PARAMETER :: ndcc=500
      ! ndcc = maximum number of codes/cutpoints allowed for each var.
      !        must be < kodtxt
   INTEGER :: i, icol, ik, j, jcol, k, kol, kolf, l, lj, lok
   INTEGER :: ni, nind, nj, nkar, nladd, nnoms, nx, status
   LOGICAL :: lpcat, newp
   CHARACTER(LEN=8) :: fmtn='(      )'

      ! Work arrays:
   INTEGER, DIMENSION(nvarmx) :: iwrk
   REAL(KIND=8), DIMENSION(-1:ndcc,nvarmx) :: wvals
   REAL(KIND=8), DIMENSION(ndcc) :: dwrk
   CHARACTER(LEN=nocw), DIMENSION(-1:ndcc,nvarmx) :: wnoms
   CHARACTER(LEN=nocw), DIMENSION(ndcc) :: cwrk

   INCLUDE 'gmpcatp.'

   ! nmotc - number of keywords in this paragraph
   ! motc  - the keywords (possibly preceded by sep2 character) of the
   !         paragraph, in chronological order of their analysis
   ! lngm(.,1) - full length of elements of motc
   ! lngm(.,2) - length of maximum abreviated form of elements of motc


   !    NOTES.
   !
   ! LVAL and LNOM are arrays of locations of sub-arrays in arrays
   ! VALS and NOMS.
   ! i = LVAL(j) is the location in VALS of sub-array of codes
   ! (or cutpoints) for the jth variable.
   ! 0 codes for no codes/cutpoints for the variable.
   ! i.e. VALS(i) contains the 1st code or cutpoint of jth variable.
   !
   ! VALS(i-2) contains j, the index of the variable.
   !    If numeric (or character) codes were stated:
   ! VALS(i-1) is negative and its absolute value is the number of
   ! stated codes (in case of character codes, VALS(i-1) is
   ! decremented by KODTXT for distinction from numeric codes)
   !    If cutpoints were stated:
   ! VALS(i-1) contains the number of stated cutpoints + 1 (a last
   ! cutpoint, set to XOUT, is added to represent the last upper
   ! bound).
   !
   ! Idem for LNOM and NOMS with few differences:
   ! For NOMS the number of category names is always positive,
   ! even for codes-names;
   ! and due to character-type of array, NOMS(i-2) and NOMS(i-1) are
   ! filled in by writing into them with format I4 (category names
   ! of numeric and character codes are treated in the same maner).
   !
   ! Note that, due to the way of processing, the elements in vals
   ! are stored in increasing order of rank of variables (i.e., if
   ! i<j, then codes/cutpts/texts for variable i are stored in vals
   ! before those for variable j).
   ! The only exception is in case of input with an old version of gmp
   ! file and vals is unchanged.

   !-------------------------------------------------------------------

   lerr = .false.

   lok = 1   ! Starting location in noms for storage of default category names

   ! If nothing stated, just check consistency of previous information

   if ( k1+1 < kn ) then

      ! Keep previous categ information ?
      ! ---------------------------------
      lpcat = lscalx( motc(1), lngm(1,1), lngm(1,2), k1, kn, .true., icol, jcol, lerr )
      if ( lerr ) then
         call gmpwrn( pargr, linter, motc(1:), nmotc )
         return
      end if

      ! Copy previous categ information in temporary arrays if it is to
      ! be kept, so that new info will complete or replace previous ones
      ! Note that it's meaningfull for input gmp file only, as otherwise
      ! all concerned arrays and parameters have been reset to zero

      wvals = zero
      wnoms = " "
      iwrk  = 0      ! Initialize these 3 work arrays,
      dwrk  = zero   ! although not necessary
      cwrk  = " "    !

      if ( lpcat ) then
         if ( lcode .and. allocated(vals) .and. allocated(noms) ) then
               ! Fill in wvals with existing vals
            j = 1
            do
               if ( j >= nvals ) exit
               lj = nint(vals(j))
               nx = mod( abs(nint(vals(j+1))), kodtxt )
               wvals(-1,lj) = vals(j)
               wvals( 0,lj) = vals(j+1)
               do k = 1, nx
                  wvals(k,lj) = vals(j+1+k)
               end do
               j = j + 2 + nx
            end do
               ! Fill in wnoms with existing noms
            j = 1
            do
               if ( j >= nvals ) exit
               read(noms(j  ),1111) lj
               read(noms(j+1),1111) nx
               wnoms(-1,lj) = noms(j)
               wnoms( 0,lj) = noms(j+1)
               do k = 1, nx
                  wnoms(k,lj) = noms(j+1+k)
               end do
               j = j + 2 + nx
            end do
         end if
      else
         nvals = 0
         lval = 0
         lnom = 0
         if ( allocated(vals) ) deallocate(vals)
         if ( allocated(noms) ) deallocate(noms)
      end if

      ! Analysis of numeric codes
      ! -------------------------
      kolf = k1

      do
         kol = kolf
         call avecty( motc(2), lngm(2,1), lngm(2,2), kol, kn, 1,    &
                      vnom(1:nvarmx), iwrk, 1, nvarmx, nind, ndcc,  &
                      nx, icol, kolf,                               &
                      lerr, dx=dwrk )
         if ( lerr ) then
            call gmpwrn( pargr, linter, motc(2:), nmotc-1 )
            return
         end if
         if ( icol <= 0 ) exit
         if ( nind <= 0 ) then
!!          call chkpg( 3, .false., newp )
            write(lgu,*) mrker1
            write(lgu,222) motc(2)(2:)
            write(lgu,*) mrker2
!!          lines = lines + 3
            lerr  = .true.
            return
         end if
            ! Cancel replications of indices, if any
         call filter( iwrk, nind, -1 )
            ! Fill in temporary array for vals
         do i = 1, nind
            j = iwrk(i)
            wvals(-1,j)= j
            wvals( 0,j)= -nx
            do k = 1, nx
               wvals(k,j) = dwrk(k)
            end do
         end do
         if ( kolf >= kn ) exit
         ! Cycle for next appearance of keyword
      end do

      ! Analysis of character codes (not converted to capitals)
      ! ---------------------------
      kolf = k1

      do
         kol = kolf
         call avecty( motc(3), lngm(3,1), lngm(3,2), kol, kn, 6,     &
                      vnom(1:nvarmx), iwrk, 1, nvarmx, nind, ndcc,   &
                      nx, icol, kolf,                                &
                      lerr, dx=dwrk )
         if ( lerr ) then
            call gmpwrn( pargr, linter, motc(3:), nmotc-2 )
            return
         end if
         if ( icol <= 0 ) exit
         if ( nind <= 0 ) then
!!          call chkpg( 3, .false., newp )
            write(lgu,*) mrker1
            write(lgu,222) motc(3)(2:)
            write(lgu,*) mrker2
!!          lines = lines + 3
            lerr  = .true.
            return
         end if
            ! Cancel replications of indices, if any
         call filter( iwrk, nind, -1 )
            ! Fill in temporary array for vals
         do i = 1, nind
            j = iwrk(i)
            wvals(-1,j)= j
            wvals( 0,j)= -(nx+kodtxt)
            do k = 1, nx
               wvals(k,j) = dwrk(k)
            end do
         end do
         if ( kolf >= kn ) exit
         ! Cycle for next appearance of keyword
      end do

      ! Analysis of cutpoints
      ! ---------------------
      kolf = k1

      do
         kol = kolf
         call avecty( motc(4), lngm(4,1), lngm(4,2), kol, kn, 1,     &
                      vnom(1:nvarmx), iwrk, 1, nvarmx, nind, ndcc-1, &
                      nx, icol, kolf,                                &
                      lerr, dx=dwrk )
         if ( lerr ) then
            call gmpwrn( pargr, linter, motc(4:), nmotc-3 )
            return
         end if
         if ( icol <= 0 ) exit
         if ( nind <= 0 ) then
!!          call chkpg( 3, .false., newp )
            write(lgu,*) mrker1
            write(lgu,222) motc(4)(2:)
            write(lgu,*) mrker2
!!          lines = lines + 3
            lerr  = .true.
            return
         end if
            ! Cancel replications of indices, if any
         call filter( iwrk, nind, -1 )
            ! Fill in temporary array for vals
         nx = nx + 1
         do i = 1, nind
            j = iwrk(i)
            wvals(-1,j)= j
            wvals( 0,j)= nx
            do k = 1, nx-1
               wvals(k,j) = dwrk(k)
            end do
            wvals(nx,j) = xout
         end do
         if ( kolf >= kn ) exit
         ! Cycle for next appearance of keyword
      end do

         ! Number of elements in array wvals
      nj = 0
      do i = 1, nvarmx
         if ( wvals(-1,i) > zero ) then
            nx = mod( abs(nint(wvals(0,i))), kodtxt )
            nj = nj + 2 + nx
         end if
      end do

      if ( nj > 0 ) then
            ! Allocate array vals
         nvals = nj
         if ( allocated(vals) ) deallocate(vals)
         allocate( vals(1:nvals), stat=status )
         if ( status /= 0 ) then   ! Fatal error
            write(lgu,*) mrker1
            write(lgu,101) " (GmpCat_1)."
            write(lgu,*) mrker2
            call gmp999            ! Release memory space and files
            stop                   ! and stop
         end if
            ! Fill in arrays lval and vals
         nind = 1
         do i = 1, nvarmx
            j = nint(wvals(-1,i))
            if ( j > 0 ) then
               nx = mod( abs(nint(wvals(0,i))), kodtxt )
               lval(j) = nind + 2
               vals(nind) = j
               do k = 0, nx
                  vals(nind+1+k) = wvals(k,i)
               end do
               nind = nind + 2 + nx
            end if
         end do
      end if

      ! Analysis of category names (not converted to capitals)
      ! --------------------------
      kolf = k1

      do
         kol = kolf
         call avecty( motc(5), lngm(5,1), lngm(5,2), kol, kn, 5,     &
                      vnom(1:nvarmx), iwrk, 1, nvarmx, nind, ndcc,   &
                      nx, icol, kolf,                                &
                      lerr, cx=cwrk )
         if ( lerr ) then
            call gmpwrn( pargr, linter, motc(5:), nmotc-4 )
            return
         end if
         if ( icol <= 0 ) exit
         if ( nind <= 0 ) then
!!          call chkpg( 3, .false., newp )
            write(lgu,*) mrker1
            write(lgu,222) motc(5)(2:)
            write(lgu,*) mrker2
!!          lines = lines + 3
            lerr  = .true.
            return
         end if
            ! Cancel replications of indices, if any
         call filter( iwrk, nind, -1 )
            ! Fill in temporary array for noms
         do i = 1, nind
            j = iwrk(i)
            write(wnoms(-1,j),1111) j
            write(wnoms( 0,j),1111) nx
            do k = 1, nx
               wnoms(k,j) = cwrk(k)
            end do
         end do
         if ( kolf >= kn ) exit
         ! Cycle for next appearance of keyword
      end do

      if ( kol > k1 .and. nvals > 0 ) then
            ! Allocate array noms
         if ( allocated(noms) ) deallocate(noms)
         allocate( noms(1:nvals), stat=status )
         if ( status /= 0 ) then   ! Fatal error
            write(lgu,*) mrker1
            write(lgu,101) " (GmpCat_2)."
            write(lgu,*) mrker2
            call gmp999            ! Release memory space and files
            stop                   ! and stop
         end if
            ! Fill in arrays lnom and noms
         nind = 1
         do i = 1, nvarmx
            read(wnoms(-1,i),1111) j
            if ( j > 0 ) then
               read(wnoms(0,i),1111) nx
               if ( nind+1+nx > nvals ) then
!!                call chkpg( 4, .false., newp )
                  write(lgu,*) mrker1
                  write(lgu,555)
                  write(lgu,*) mrker2
!!                lines = lines + 4
                  lerr  = .true.
                  return
               end if
               lnom(j) = nind + 2
               write(noms(nind),1111) j
               do k = 0, nx
                  noms(nind+1+k) = wnoms(k,i)
               end do
               nind = nind + 2 + nx
            end if
         end do
         lok = nind
      end if

   end if

   ! Default category names if codes/cutpts/texts given without names
   ! Check consistency between codes/cutpts/texts and names, if given
   ! ----------------------------------------------------------------

      ! Allocate noms if not done yet, as needed for default names
   if ( .not. allocated(noms) ) then
      allocate( noms(1:max(nvals,1)), stat=status )
      if ( status /= 0 ) then   ! Fatal error
         write(lgu,*) mrker1
         write(lgu,101) " (GmpCat_3)."
         write(lgu,*) mrker2
         call gmp999            ! Release memory space and files
         stop                   ! and stop
      end if
      lok = 1
   end if

   do i = 1, nvarmx
      if ( lval(i) > 0 ) then

         if ( lnom(i) <= 0 ) then   ! Generate default names
            lnom(i) = lok + 2
            lj      = lval(i) - 1
            k       = nint(vals(lj))
            ik      = abs(k)
            write(noms(lok),1111) i
            if ( ik <= kodtxt ) then   ! Names for numeric codes or cutpoints
               write(noms(lok+1),1111) ik
               do l = 1, ik
                  fmtn(2:7) = dfmtc( nocw, 4-nocw,vals(lj+l), tzero )
                  write(noms(lok+1+l),fmtn) vals(lj+l)
               end do
            else                       ! Names for character codes
               ik = ik - kodtxt
               write(noms(lok+1),1111) ik
               do l = 1, ik
                  write(noms(lok+1+l),666) vals(lj+l)
               end do
            end if
            lok = lok + 2 + ik
            if ( k > 0 ) noms(lok-1) = '*MAXIMUM'

         else                       ! Check codes/cutpts/texts and names
            ni = nint(vals(lval(i)-1))
            lj = lnom(i) - 1
            read(noms(lj),1111) nj
            if ( nj /= mod(abs(ni),kodtxt) ) then
!!             call chkpg( 4, .false., newp )
               write(lgu,*) mrker1
               write(lgu,333) vnom(i)
               write(lgu,*) mrker2
!!             lines = lines + 4
               lerr  = .true.
               return
            end if
         end if

      else if ( lnom(i) > 0 ) then
!!        call chkpg( 4, .false., newp )
          write(lgu,*) mrker1
          write(lgu,444) vnom(i)
          write(lgu,*) mrker2
!!        lines = lines + 4
          lerr  = .true.
          return

      end if
   end do

   nnoms = lok - 1      ! Number of elements in array noms

   if ( nnoms /= nvals ) then
!!    call chkpg( 4, .false., newp )
      write(lgu,*) mrker1
      write(lgu,555)
      write(lgu,*) mrker2
!!    lines = lines + 4
      lerr  = .true.
      return
   end if

   ! Commands filtering
   ! ------------------
   nkar = ncar
   call cmprss( k1, kn-1 )
   kn = kn - nkar + ncar
   if ( kn > k1 ) then
      nladd = 3 + (kn-(k1+1))/(lrecp-1)
      call chkpg( nladd, .false., newp )
      write(imp,111) (car(i:min(i+lrecp-2,kn)),i=k1+1,kn,lrecp-1)
      lines = lines + nladd
      write(lgu,*) mrker1//mrker1
      write(lgu,111) (car(i:min(i+lrecp-2,kn)),i=k1+1,kn,lrecp-1)
      write(lgu,*) mrker2//mrker2
      car(k1:kn-1) = ' '
      call cmprss( k1, kn-1 )
   end if

   INCLUDE 'allocf.'
   INCLUDE 'gmpf111.'
   INCLUDE 'gmpcatf.'
   INCLUDE 'gmpnomsf.'

END SUBROUTINE gmpcat
