SUBROUTINE gmpcmd( card, lcard, lprt, cend, nend, kend, kard, nread, eof, lerr )

!***********************************************************************
!     - gmpcmd -
!     (derived from leccom routine of ANACOM7 library by adding
!     call to chkpg to control page skipping when lprt=.true.)
!     reads commands, changes tabs to blanks, suppresses blanks not
!     inbedded into quotes and suppresses delimiting quotes.
!     Commands are stored in character string car (in module carl)
!     cend = keyword for end of commands (initiates normal return);
!     this keyword is kept in car.
!     ikcom = iachar rank of character coding comments: the zone in
!     card beyond this character is ignored. This character is not
!     kept in car.
!***********************************************************************
!
!     CARD  - INPUT . A record on LEC                         - CHAR
!                     (already read if LCARD=.true.)
!             OUTPUT. Last record read on LEC
!     LCARD - INPUT . .true. if CARD has already been read    - LOGICAL
!     LPRT  - INPUT . If .true. commands read are printed     - LOGICAL
!     CEND  - INPUT . Keyword for end of commands             - CHAR
!     NEND  - INPUT . Number of characters in CEND            - INTEGER
!     KEND  - INPUT . IACHAR rank of 1st character of CEND    - INTEGER
!     KARD  - OUTPUT. Location in CARD of last character of   - INTEGER
!                     CEND (0 if end of file reached in batch)
!     NREAD - OUPUT . Number of records read on LEC           - INTEGER
!     EOF   - OUTPUT. .true. if end of file reached before    - LOGICAL
!                     CEND is found
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!                     (too many commands or end of file in
!                     batch and unbalanced quotes)
!
!     --------------------------------------------------------
!     Module elements modified:
!     carl   : car  ncar  com
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG
!     External functions called:
!        NONE
!     Fortran  functions called:
!        IACHAR  INDEX  LEN_TRIM  MAX  MIN
!
!-----------------------------------------------------------------------

   USE carl
   USE gmpint

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: nend, kend
   LOGICAL, INTENT(IN) :: lcard, lprt
   CHARACTER(LEN=*), INTENT(IN) :: cend
   CHARACTER(LEN=*), INTENT(INOUT) :: card
   INTEGER, INTENT(OUT) :: kard, nread
   LOGICAL, INTENT(OUT) :: eof, lerr

   !  Local declarations
   !  ------------------

   INTEGER :: i, icf, j, jcf, ncf
   LOGICAL :: lend, llcard, lqte, l6imp, newp, prec
   INTEGER, PARAMETER :: iblan=IACHAR(" "), iquote=IACHAR("'")
   INTEGER, PARAMETER :: itab=9   ! IACHAR(horizontal tabulation)

   !--------------------------------------------------------------------

   kard = 0
   nread= 0
   eof  = .false.
   lerr = .false.
   lqte = .false.
   prec = .false.
   lend = .false.
   l6imp = l6 == imp
   if ( lprt ) then
      if ( l6imp ) call chkpg( 2, .false., newp )
      write(l6,222)
      if ( l6imp ) lines = lines + 2
   endif
   llcard = lcard

   ! Loop on records of unit lec
   ! ---------------------------
   record: do
      ! If card already read, skip reading
      if ( llcard ) then
         llcard = .false.
      else
         read(lec,'(a)',end=10) card(:lrecl)
         nread = nread + 1
      end if
      if ( lprt ) then
         j = max( len_trim( card(:lrecl) ), 1 )
         do i = 1, j, lrecp
            if ( l6imp ) call chkpg( 1, .false., newp )
            write(l6,*) card(i:min(i+lrecp-1,j))
            if ( l6imp ) lines = lines + 1
         end do
      endif

      ! Horizontal tab is replaced by blank character
      ! Suppress blanks except in string delimited by quotes
      ! lqte and com()=.true. if current column is inside quotes
      ! prec=.true. if previous column is a quote
      ! lend=.true. if current column may be part of end keyword

      ! Loop on characters of record card
      ! ---------------------------------
      do i = 1, lrecl

         j = iachar(card(i:i))

         if ( j == iquote ) then

            lqte = .not. lqte
            if ( lend ) lend = .false.
            if ( prec ) then
               ! It's a double quote, thus to be kept in car
               ncar = ncar + 1
               if ( ncar > incar ) then
                  write(l6,*) mrker1
                  write(l6,111) incar
                  write(l6,*) mrker2
                  lerr = .true.
                  return
               endif

               car(ncar:ncar) = card(i:i)
               com(ncar)      = lqte
               prec = .false.
            else
               prec = .true.
            endif

         else

            if ( j == itab ) then
               j = iblan
               card(i:i) = ' '
            endif
            prec = .false.
            if ( .not. lqte ) then
               ! We are in an "active" zone (if blank, skip it)

               if ( j /= iblan ) then
                  ! If it is the beginning of a comment zone,
                  ! skip remaining part of record and go to next record
                  if ( j == ikcom ) cycle record

                  ncar = ncar + 1
                  if ( ncar > incar ) then
                     write(l6,*) mrker1
                     write(l6,111) incar
                     write(l6,*) mrker2
                     lerr = .true.
                     return
                  endif

                  car(ncar:ncar) = card(i:i)
                  com(ncar)      = lqte
                  ! End keyword ?
                  if ( lend ) then
                     ncf = ncf + 1
                     if ( ncf == nend ) then
                        if ( car(icf:ncar) == cend(:nend) ) then
                           ! The end, normal return
                           kard = i
                           return
                          !------!
                        endif
                        ! Beginning of end keyword in icf+1,...?
                        jcf = index(car(icf+1:ncar),cend(1:1))
                        if ( jcf <= 0 ) then
                           lend = .false.
                        else
                           icf  = icf + jcf
                           ncf  = ncf - jcf
                        endif
                     endif
                    ! First character of end keyword ?
                  else if ( j == kend ) then
                      ! If length of end keyword is 1:
                      ! the end, normal return
                      if ( nend == 1 ) then
                         kard = i
                         return
                        !------!
                      endif
                      icf  = ncar
                      ncf  = 1
                      lend = .true.
                  endif

               endif

           else
               ! We are in a zone delimited by quotes
               ncar = ncar + 1
               if ( ncar > incar ) then
                  write(l6,*) mrker1
                  write(l6,111) incar
                  write(l6,*) mrker2
                  lerr = .true.
                  return
               endif

               car(ncar:ncar) = card(i:i)
               com(ncar)      = lqte
            endif

         endif

      end do

      ! End of current record scanning, go back to read next record
   end do record

10 continue           ! End of file (batch)
   eof = .true.
   if ( lqte ) then   ! If quotes are not balanced it's considered as an error
      write(l6,*) mrker1
      write(l6,333)
      write(l6,*) mrker2
      lerr = .true.
   endif

   INCLUDE 'gmpcmdf.'

END SUBROUTINE gmpcmd
