SUBROUTINE gmpext( mvarmx, lerr )

!***********************************************************************
!     - gmpext -
!     when gmp file on input and nvarmx has changed, extends/compresses
!     mem1 arrays (xmis, xmin, xmax, vuse, lval, lnom, vsav, vals)
!     and mema arrays (vnom, unom, noms)
!     Note that arrays vuse, vsav and unom are not filled in here, as
!     reset to default later on
!***********************************************************************
!
!     MVARMX- INPUT . nvarmx from previous problem            - INTEGER
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     memst1 : xmis   xmin   xmax   vals   vuse   lval   lnom   vsav
!              l_xmis l_xmin l_xmax l_vals l_vuse l_lval l_lnom l_vsav
!     memsta : vnom   unom   noms
!              l_vnom l_unom l_noms
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        ABS  ALLOCATE  DEALLOCATE  MOD  NINT
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE memst1
   USE memsta
   USE spcnom
   USE spcval

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: mvarmx
   LOGICAL, INTENT(OUT) :: lerr


   !  Local declarations
   !  ------------------

   INTEGER :: i, j, k, l, nvals, nx, status
   LOGICAL :: done
      ! Work arrays:
   INTEGER, DIMENSION(mvarmx) :: iwrk
   REAL(KIND=8), DIMENSION(mvarmx) :: dwrk
   REAL(KIND=8), DIMENSION(MAX(1,l_vals)) :: wals
   CHARACTER(LEN=nocw), DIMENSION(0:mvarmx) :: cwrk
   CHARACTER(LEN=nocw), DIMENSION(MAX(1,l_vals)) :: woms

   !--------------------------------------------------------------------

   lerr = .false.

   if ( nvarmx > mvarmx ) then

      ! Extension
      ! ---------
         !-- xmis
      dwrk = xmis
      deallocate(xmis)
      allocate( xmis(nvarmx), stat=status )
      if ( status /= 0 ) then
         write(lgu,*) ermrk1
         write(lgu,101) " (GmpExt_1)."
         write(lgu,*) ermrk2
         lerr = .true.
         return
      end if
      xmis(1:mvarmx) = dwrk
         !-- xmin
      dwrk = xmin
      deallocate(xmin)
      allocate( xmin(nvarmx), stat=status )
      if ( status /= 0 ) then
         write(lgu,*) ermrk1
         write(lgu,101) " (GmpExt_2)."
         write(lgu,*) ermrk2
         lerr = .true.
         return
      end if
      xmin(1:mvarmx) = dwrk
         !-- xmax
      dwrk = xmax
      deallocate(xmax)
      allocate( xmax(nvarmx), stat=status )
      if ( status /= 0 ) then
         write(lgu,*) ermrk1
         write(lgu,101) " (GmpExt_3)."
         write(lgu,*) ermrk2
         lerr = .true.
         return
      end if
      xmax(1:mvarmx) = dwrk
         !-- vuse
      deallocate(vuse)
      allocate( vuse(nvarmx), stat=status )
      if ( status /= 0 ) then
         write(lgu,*) ermrk1
         write(lgu,101) " (GmpExt_4)."
         write(lgu,*) ermrk2
         lerr = .true.
         return
      end if
         !-- lval
      iwrk = lval
      deallocate(lval)
      allocate( lval(nvarmx), stat=status )
      if ( status /= 0 ) then
         write(lgu,*) ermrk1
         write(lgu,101) " (GmpExt_5)."
         write(lgu,*) ermrk2
         lerr = .true.
         return
      end if
      lval(1:mvarmx) = iwrk
         !-- lnom
      iwrk = lnom
      deallocate(lnom)
      allocate( lnom(nvarmx), stat=status )
      if ( status /= 0 ) then
         write(lgu,*) ermrk1
         write(lgu,101) " (GmpExt_6)."
         write(lgu,*) ermrk2
         lerr = .true.
         return
      end if
      lnom(1:mvarmx) = iwrk
         !-- vsav
      deallocate(vsav)
      allocate( vsav(nvarmx), stat=status )
      if ( status /= 0 ) then
         write(lgu,*) ermrk1
         write(lgu,101) " (GmpExt_7)."
         write(lgu,*) ermrk2
         lerr = .true.
         return
      end if
         !-- vnom
      cwrk = vnom(1:mvarmx)
      deallocate(vnom)
      allocate( vnom(0:nvmxst), stat=status )
      if ( status /= 0 ) then
         write(lgu,*) ermrk1
         write(lgu,101) " (GmpExt_8)."
         write(lgu,*) ermrk2
         lerr = .true.
         return
      end if
      vnom(1:mvarmx) = cwrk
         !-- unom
      deallocate(unom)
      allocate( unom(nvarmx), stat=status )
      if ( status /= 0 ) then
         write(lgu,*) ermrk1
         write(lgu,101) " (GmpExt_9)."
         write(lgu,*) ermrk2
         lerr = .true.
         return
      end if
         ! N.B. vals and noms are left unchanged at the moment

         ! Initialize elements added
      xmis(mvarmx+1:nvarmx) = xout
      xmin(mvarmx+1:nvarmx) = -xout
      xmax(mvarmx+1:nvarmx) = xout
      lval(mvarmx+1:nvarmx) = 0
      lnom(mvarmx+1:nvarmx) = 0
      do i = mvarmx+1, nvarmx
         vnom(i) = ' '
         write(vnom(i),"('X',i3.3)") i
      end do

   else if ( nvarmx < mvarmx ) then

      ! Compression
      ! -----------
         !-- vals and noms are compressed, if relevant
      if ( l_vals > 0 ) then
         wals = vals
         woms = noms
         nvals = l_vals
         done = .false.
         do l = nvarmx+1, mvarmx
            if ( lval(l) > 0 ) then
               done = .true.
               j = lval(l)
               nx = mod( abs(nint(wals(j-1))), kodtxt )
               k = nx + 2
               wals(j-2:nvals-k) = wals(j+nx:nvals)
                  ! Update locations
               lval(l) = 0
               do i = 1, nvarmx
                  if ( lval(i) > j ) lval(i) = lval(i) - k
               end do
               j = lnom(l)
               woms(j-2:nvals-k) = woms(j+nx:nvals)
                  ! Update locations
               lnom(l) = 0
               do i = 1, nvarmx
                  if ( lnom(i) > j ) lnom(i) = lnom(i) - k
               end do
               nvals = nvals - k
            end if
         end do
         if ( done ) then
            l_vals = nvals
            deallocate( vals, noms )
            allocate( vals(l_vals), noms(l_vals), stat=status )
            if ( status /= 0 ) then
               write(lgu,*) ermrk1
               write(lgu,101) " (GmpExt_10)."
               write(lgu,*) ermrk2
               lerr = .true.
               return
            end if
            vals = wals(1:l_vals)
            noms = woms(1:l_vals)
         end if
      end if
         !-- xmis
      dwrk = xmis
      deallocate(xmis)
      allocate( xmis(nvarmx), stat=status )
      if ( status /= 0 ) then
         write(lgu,*) ermrk1
         write(lgu,101) " (GmpExt_11)."
         write(lgu,*) ermrk2
         lerr = .true.
         return
      end if
      xmis = dwrk(1:nvarmx)
         !-- xmin
      dwrk = xmin
      deallocate(xmin)
      allocate( xmin(nvarmx), stat=status )
      if ( status /= 0 ) then
         write(lgu,*) ermrk1
         write(lgu,101) " (GmpExt_12)."
         write(lgu,*) ermrk2
         lerr = .true.
         return
      end if
      xmin = dwrk(1:nvarmx)
         !-- xmax
      dwrk = xmax
      deallocate(xmax)
      allocate( xmax(nvarmx), stat=status )
      if ( status /= 0 ) then
         write(lgu,*) ermrk1
         write(lgu,101) " (GmpExt_13)."
         write(lgu,*) ermrk2
         lerr = .true.
         return
      end if
      xmax = dwrk(1:nvarmx)
         !-- vuse
      deallocate(vuse)
      allocate( vuse(nvarmx), stat=status )
      if ( status /= 0 ) then
         write(lgu,*) ermrk1
         write(lgu,101) " (GmpExt_14)."
         write(lgu,*) ermrk2
         lerr = .true.
         return
      end if
         !-- lval
      iwrk = lval
      deallocate(lval)
      allocate( lval(nvarmx), stat=status )
      if ( status /= 0 ) then
         write(lgu,*) ermrk1
         write(lgu,101) " (GmpExt_15)."
         write(lgu,*) ermrk2
         lerr = .true.
         return
      end if
      lval = iwrk(1:nvarmx)
         !-- lnom
      iwrk = lnom
      deallocate(lnom)
      allocate( lnom(nvarmx), stat=status )
      if ( status /= 0 ) then
         write(lgu,*) ermrk1
         write(lgu,101) " (GmpExt_16)."
         write(lgu,*) ermrk2
         lerr = .true.
         return
      end if
      lnom = iwrk(1:nvarmx)
         !-- vsav
      deallocate(vsav)
      allocate( vsav(nvarmx), stat=status )
      if ( status /= 0 ) then
         write(lgu,*) ermrk1
         write(lgu,101) " (GmpExt_17)."
         write(lgu,*) ermrk2
         lerr = .true.
         return
      end if
         !-- vnom
      cwrk = vnom(1:mvarmx)
      deallocate(vnom)
      allocate( vnom(0:nvmxst), stat=status )
      if ( status /= 0 ) then
         write(lgu,*) ermrk1
         write(lgu,101) " (GmpExt_18)."
         write(lgu,*) ermrk2
         lerr = .true.
         return
      end if
      vnom(1:nvarmx) = cwrk(1:nvarmx)
         !-- unom
      deallocate(unom)
      allocate( unom(nvarmx), stat=status )
      if ( status /= 0 ) then
         write(lgu,*) ermrk1
         write(lgu,101) " (GmpExt_19)."
         write(lgu,*) ermrk2
         lerr = .true.
         return
      end if

   end if

      ! Set the special "no variable" name to blank and
      ! add special variable names
   vnom(0) = ' '
   vnom(nvarmx+1) = vstat
   vnom(nvarmx+2) = vrang
   vnom(nvarmx+3) = vmanq
   vnom(nvarmx+4:nvmxst) = ' ' ! Initialize for possible temporary variable names

   l_xmis = nvarmx
   l_xmin = nvarmx
   l_xmax = nvarmx
   l_vuse = nvarmx
   l_lval = nvarmx
   l_lnom = nvarmx
   l_vsav = nvarmx
   l_vnom = nvmxst
   l_unom = nvarmx

   INCLUDE 'allocf.'

END SUBROUTINE gmpext
