SUBROUTINE gmpgfp( oldvrs, nvar, jlabel, jfrqcy, lerr )

!***********************************************************************
!     - gmpgfp -
!     reads parameters from the dictionnary of a gmp file
!***********************************************************************
!
!     oldvrs- OUTPUT. .true. if gmp-file version is F1.0      - LOGICAL
!     NVAR  - OUTPUT. The value of nvarmx                     - INTEGER
!                     at time the gmp-file was created
!     JLABEL- OUTPUT. The value of label                      - INTEGER
!                     at time the gmp-file was created
!     JFRQCY- OUTPUT. The value of ifrqcy                     - INTEGER
!                     at time the gmp-file was created
!     LERR  - OUTPUT. .true. if an error has been detected    - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpchr : codee
!     gmpint : lines   nvarin
!     memst1 : l_xmis to l_vsav
!     memsta : l_vnom   l_unom   l_noms
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        LEN_TRIM  MIN
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE memst1
   USE memsta
   USE pgmgmp

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(OUT) :: nvar, jlabel, jfrqcy
   LOGICAL, INTENT(OUT) :: oldvrs, lerr

   !  Local declarations
   !  ------------------

   REAL(KIND=8) :: zout
   INTEGER :: i, jidw, jirw, jocw, jsz1, jsza, null, nvsp, nvtmp
   LOGICAL :: newp
   CHARACTER(LEN=8) :: vrsion
   INTEGER, DIMENSION(9) :: isize1   ! For F1.0 version only

   !--------------------------------------------------------------------

   lerr = .false.

   read(inpux) codee
   read(inpux) vrsion, zout, jirw, jidw, jocw, jsz1, jsza
   oldvrs = vrsion == hdrgmp//'F1.0'
   if ( vrsion /= gmpvrs .and. .not. oldvrs ) then
      lerr = .true.
!!    call chkpg( 5, .false., newp )
      write(lgu,*) ermrk1
      write(lgu,111) hdrgmp, gmpvrs, vrsion
      write(lgu,333)
      write(lgu,*) ermrk2
!!    lines = lines + 5
      return
   end if
   if ( jocw /= nocw ) then
      lerr = .true.
!!    call chkpg( 6, .false., newp )
      i = min( len_trim(codee), lrecp-1 )
      write(lgu,*) ermrk1
      write(lgu,222) hdrgmp, codee(:i)
      write(lgu,333)
      write(lgu,*) ermrk2
!!    lines = lines + 6
      return
   end if
   read(inpux) nvar, nvsp, nvtmp, nvarin, jlabel, jfrqcy
   if ( oldvrs ) then
      read(inpux) (null,i=1,9)           ! Old mem1 pointers
      read(inpux) isize1(1:jsz1)         ! Old sizes of mem1 arrays
      read(inpux) (null,i=1,jsza)        ! Old mema pointers
      l_xmis = isize1(1)
      l_xmin = isize1(2)
      l_xmax = isize1(3)
      l_vals = isize1(9)
      l_vuse = isize1(4)
      l_lval = isize1(5)
      l_lnom = isize1(6)
      l_vsav = isize1(7)
      l_valpha = isize1(1)
   else
      read(inpux) l_xmis, l_xmin, l_xmax, l_vals,   &
                  l_vuse, l_lval, l_lnom, l_vsav, l_valpha
   end if
   read(inpux) l_vnom, l_unom, l_noms    ! Sizes of mema arrays

      ! Update size if nvarsp changed since gmp-file creation
   if ( nvsp /= nvarsp ) l_vnom = l_vnom + nvarsp - nvsp
      ! Update size if nvtemp changed since gmp-file creation
   if ( nvtmp /= nvtemp ) l_vnom = l_vnom + nvtemp - nvtmp

   INCLUDE 'gmpgfpf.'

END SUBROUTINE gmpgfp
