SUBROUTINE gmpin1( header, kardl, lerr )

!***********************************************************************
!     - gmpin1 -
!     initializes some module items
!     (this subroutine is called only once, when starting execution)
!***********************************************************************
!
!     header- INPUT . External subroutine for header printout - EXTERNAL
!     kardl - INPUT . Maximum record length on unit used for  - INTEGER
!                     command input
!                     (the record length that will be actually
!                     used will not exceed this value, but it
!                     may be lower: in any case it will not
!                     exceed 80 if batch run)
!     lerr  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  CVUCT  FILOPN  HEADER
!     External functions called:
!        NONE
!     Fortran  functions called:
!        EPSILON  HUGE  IACHAR  LEN_TRIM  LOG  MAX  MIN  RANGE  TINY
!
!-----------------------------------------------------------------------

   USE carl
   USE gmpchr
   USE gmpint
   USE gmplgl
   USE memst1
   USE memsta
   USE opprm
   USE pgmgmp
   USE spcnom
   USE spcval
   USE termnl
   USE trancb

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN)  :: kardl
   LOGICAL, INTENT(OUT) :: lerr
   EXTERNAL header

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: ten=10.0_8
   INTEGER :: nladd
   LOGICAL :: newp
   CHARACTER(LEN=incdw) :: cblan
   CHARACTER(LEN=6) :: fmta='(A   )'
   CHARACTER(LEN=3) :: rep
   INCLUDE 'gmpin1p.'

   !--------------------------------------------------------------------

   lerr = .false.

   ! INPUT/OUTPUT UNITS ( TERMNL,  GMPINT, OPPRM, CARL )
   ! ------------------
      ! Remember basic initializations were performed in Main program

      ! Adapt screen characteristics (no. of lines & columns on
      ! input/output)
      ! ncsco must be at least 77, because of outputs in gmpprp,
      ! but 79 is safer, as maybe required in program-specific section
   nlscr = max( nlscr, 20 )
   ncsci = min( ncsci, kardl )
   ncsco = ncsci - 1

      ! Unit number and record length for input of commands
      ! (the maximum record length allowed is 80)
   lrecl1 = min( 80, kardl )
   lec    = lec1
   lrecl  = lrecl1

      ! Output width and height (see also at the bottom of routine)
   lrecp = ncprt
   linep = nlprt

      ! Unit number aliases
   iook = imp   ! Output file
   io   = lgu   ! Log file (Transfo)
   l6   = lgu   ! Log file (Anacom7)

   ! OPEN SCRATCH FILE
   ! -----------------
      ! Only iu1 is needed in general section
      ! iu2 to iu4 are to be opened in program-specific section if needed
   call filopn( 0, iu1, ' ', 'SCRATCH', 'UNFORMATTED', .false., lerr )
   if ( lerr ) return

   ! SPECIAL VALUES ( SPCVAL )
   ! --------------
   write(fmta(3:5),'(i3)') incdw
   cblan  = ' '
   read(cblan,fmta) xblan
   zerolk = epsilon(ten)
   xminim = tiny(ten)
   xmaxim = huge(ten)
   explow = log(xminim)
   expupp = log(xmaxim)
   gamlow = 2.0_8**(-1023)
   gamupp = 171._8
   hsincos= 710._8
   maxint = huge(1)
   tzero  = xminim * 2
   xout   = ten**(range(ten)-1)

   ! SPECIAL NAMES ( SPCNOM )
   ! -------------
   vstat = zvstat
   vrang = zvrang
   vmanq = zvmanq

      ! Choose end-of-paragraph character, name of ending paragraph,
      ! comment-flag character ( GMPCHR, GMPINT )
   cfin  ='$'
   pfin  = cfin // '/'
   nfin  = len_trim( pfin )
   ikfin = iachar(cfin)
   ikcom = iachar('#')

      ! Choose special characters in commands ( CARL )
   sep1  = ','
   sep2  = ';'
   par1  = '('
   par2  = ')'
   repet = '*'
   to    = '>'
   ncto  = len_trim( to )
   by    = ':'
   ncby  = len_trim( by )

      ! Choose error marks ( GMPCHR, CARL, OPPRM )
   ermrk1= '{'
   ermrk2= '}'
   mrker1= ermrk1
   mrker2= ermrk2
   ewmrk1= ermrk1
   ewmrk2= ermrk2

      ! Version label to identify storage mode on GMP file ( GMPCHR )
      ! Important: do not modify unless storage mode has changed
   gmpvrs= hdrgmp // 'F2.0 '   ! Previous version was F1.0

   ! INITIALIZE MODULE ITEMS NOT DEPENDING ON PROGRAMMER'S CHOICE
   ! ------------------------------------------------------------

      ! GMPINT, GMPLGL, GMPCHR
   lines  = 1
   page   = 0
   nprob  = 0
   nvarin = 0
   nvarmx = 0
   nalpha = 0
   nvaru  = 0
   nvarsp = 3
   nvtemp = 20
   label  = 0
   ifrqcy = 0
   lectx  = 1
   lrwde  = .true.
   lcmpl  = .true.
   lbref  = .false.
   lpdat  = .false.
   lgmpf  = .false.
   filee  = '        '
   codee  = '        '

   ! PRINT PROGRAM HEADER
   ! --------------------
   call header( imp , nladd )
   lines = lines + nladd

   ! LRECL1, LEC, LRECL, LRECP, LINEP, NCSCI/O, NLSCR ARE ADJUSTED
   ! -------------------------------------------------------------
   if ( linter ) then
      lrecl1 = ncsci
      lrecl  = ncsci
      lrecp  = ncsco
      linep  = nlscr - nprmpt

      call chkpg( 4 , .false. , newp )
      write(imp,111) ncsci, nlscr
      lines = lines + 3
      read(lec,222) rep
      call cvuct( rep )
      if ( rep == batu ) then
            ! Batch mode is specified
         linter = .false.
         lrecp  = ncprt
         linep  = nlprt
         lines  = 1
      else if ( rep(:1) == nou ) then
            ! Update screen characteristics
         call chkpg( 0 , .true. , newp )
         write(imp,333) colin, ncsci
         read(lec,222) rep(:1)
         call cvuct( rep(:1) )
         if ( rep(:1) /= yesu ) then
            do
               write(imp,444) colin, ' ', kardl
               read(lec,*) ncsci
               ncsci = max( ncsci, 1 )
               ncsci = min( ncsci, kardl )
               write(imp,555) colin, ncsci
               read(lec,222) rep(:1)
               call cvuct( rep(:1) )
               if ( rep(:1) == yesu ) exit
            end do
         endif
         ncsco = ncsci - 1
         write(imp,333) colout, ncsco
         read(lec,222) rep(:1)
         call cvuct( rep(:1) )
         if ( rep(:1) /= yesu ) then
            do
               write(imp,444) colout//' (79 -> 132)', ':'
               read(lec,*) ncsco
               ncsco = max( ncsco, 79 )
               if ( ncsco .gt. 132 ) ncsco = 80
               write(imp,555) colout, ncsco
               read(lec,222) rep(:1)
               call cvuct( rep(:1) )
               if ( rep(:1) == yesu ) exit
            end do
         endif
         write(imp,333) zlin, nlscr
         read(lec,222) rep(:1)
         call cvuct( rep(:1) )
         if ( rep(:1) /= yesu ) then
            do
               write(imp,444) zlin, ':'
               read(lec,*) nlscr
               if ( nlscr .le. 0 ) nlscr = nlprt
               write(imp,555) zlin, nlscr
               read(lec,222) rep(:1)
               call cvuct( rep(:1) )
               if ( rep(:1) == yesu ) exit
            end do
         endif
         nlscr  = max( nlscr, 20 )
         lrecl1 = ncsci
         lrecl  = ncsci
         lrecp  = ncsco
         linep  = nlscr - nprmpt
      endif
   endif

   lrecp = min( 132, lrecp )      ! Output width protection

   INCLUDE 'gmpin1f.'

END SUBROUTINE gmpin1
