SUBROUTINE gmpprd( iout, xmis, xmin, xmax, vals, vuse,   &
                   lval, lnom, vnom, noms, nvals, lchk )
!***********************************************************************
!     - gmpprd -
!     prints the dictionnary of variables used
!***********************************************************************
!
!     IOUT  - INPUT . Unit number for printing                - INTEGER
!     XMIS  - INPUT .    see definition of MEM1 arrays        - REAL*8
!     XMIN  - INPUT .    see definition of MEM1 arrays        - REAL*8
!     XMAX  - INPUT .    see definition of MEM1 arrays        - REAL*8
!     VALS  - INPUT .    see definition of MEM1 arrays        - REAL*8
!     VUSE  - INPUT .    see definition of MEM1 arrays        - INTEGER
!     LVAL  - INPUT .    see definition of MEM1 arrays        - INTEGER
!     LNOM  - INPUT .    see definition of MEM1 arrays        - INTEGER
!     VNOM  - INPUT .    see definition of MEMA arrays        - CHAR
!     NOMS  - INPUT .    see definition of MEMA arrays        - CHAR
!     NVALS - INPUT . Length of arrays VALS and NOMS          - INTEGER
!     LCHK  - INPUT . If .false. no call to CHKPG             - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG
!     External functions called:
!        NONE
!     Fortran  functions called:
!        ABS  MIN  MOD  NINT
!
!-----------------------------------------------------------------------

   USE gmpint

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: nvals
   LOGICAL, INTENT(IN) :: lchk
   REAL(KIND=8), DIMENSION(nvarmx), INTENT(IN) :: xmis, xmin, xmax
   REAL(KIND=8), DIMENSION(nvals) , INTENT(IN) :: vals
   INTEGER, DIMENSION(nvarmx), INTENT(IN) :: vuse, lval, lnom
   CHARACTER(LEN=*), DIMENSION(nvarmx), INTENT(IN) :: vnom
   CHARACTER(LEN=*), DIMENSION(nvals) , INTENT(IN) :: noms

   !  Local declarations
   !  ------------------

   INTEGER :: i, ii, inom, iout, ival, j, k, la, lz
   INTEGER :: ncod, ndisp, nkod, nladd, npas
   INTEGER, PARAMETER :: minic=2   ! Minimum number of codes per var.
                                   ! to be printed in one page
   LOGICAL :: newp
   CHARACTER(LEN=25) :: fmt
   CHARACTER(LEN=25), PARAMETER :: fmt5="(31X,G14.7,'   (',A,')' )"
   CHARACTER(LEN=25), PARAMETER :: fmt6="(35X,A   ,5X,'(',A,')' ) "
                      ! voir 35X et 5X de fmt6 si incdw /= 8 ?????
   INCLUDE 'gmpprdp.'

   !--------------------------------------------------------------------

   newp = .false.

   if ( lchk ) then
         ! Check there's enough place in page to print at least 1 var.
      if ( lval(vuse(1)) == 0 ) then
         nladd = 9
      else
         nladd = 10 + minic
      end if
      call chkpg( nladd, .false., newp )
   end if
   write(iout,111)
   write(iout,222)
   lines = lines + 6

   do ii = 1, nvaru
      i    = vuse(ii)
      ival = lval(i)
      if ( ival /= 0 ) then
         inom  = lnom(i)
         ncod  = nint(vals(ival-1))
         nkod  = abs(ncod)
         nkod  = mod( nkod, kodtxt )
         nladd = 4 + min( nkod, minic )
         if ( lchk ) then
            call chkpg( nladd, .false., newp )
            if ( newp ) then
               write(iout,222)
               lines = lines + 2
            end if
         end if
         if ( ncod >= -kodtxt ) then
            fmt = fmt5
         else
            fmt = fmt6
            write(fmt(7:9),'(i3)') incdw
         end if
         write(iout,333) i, vnom(i), xmis(i), xmin(i), xmax(i)
         if ( ncod < 0 ) then
            write(iout,444) msgcod
         else
            write(iout,444) msgbor
         end if
         lines = lines + 4
         ndisp = linep - lines
         lz = min(nkod,ndisp)
         do j = 0, lz-1
            write(iout,fmt) vals(ival+j), noms(inom+j)
         end do
         lines = lines + lz
         if ( nkod > ndisp ) then
            npas = (nkod-ndisp-1)/linep + 1
            la   = ndisp
            do k = 1, npas
               if ( lchk ) call chkpg( 0, .true., newp )
               lz = min(nkod,la+linep) - 1
               do j = la, lz
                  write(iout,fmt) vals(ival+j), noms(inom+j)
               end do
               la = lz + 1
            end do
            lines = lines + lz - la + 1
         end if
      else
         if ( lchk ) then
            call chkpg( 3, .false., newp )
            if ( newp ) then
               write(iout,222)
               lines = lines + 2
            end if
         end if
         write(iout,333) i, vnom(i), xmis(i), xmin(i), xmax(i)
         lines = lines + 3
      end if
   end do

   INCLUDE 'gmpprdf.'

END SUBROUTINE gmpprd
