SUBROUTINE gmppro( k1, kn, lerr )

!***********************************************************************
!     - gmppro -
!     analyzes PROB paragraph, with commands filtering
!***********************************************************************
!
!     K1    - INPUT . Starting location in CAR of the zone    - INTEGER
!                     analyzed. Unchanged on return
!     KN    - INPUT . Ending   location in CAR of the zone    - INTEGER
!                     analyzed (end-of-paragraph symbol)
!             OUTPUT. Destroyed
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     carl   : car
!     gmpchr : contin  filee   titre   titl2   fmtin
!     gmpint : lrecl   lrecp   linep   lines   inpux
!              nvarin  nvadd   nindv   lectx   kontin
!     gmplgl : lcode   lrwde   lbref   lgmpf
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  CHKUNT  CMPRSS  CSCALX  CVUCT  FILOPN  GMPWRN
!     External functions called:
!        ISCALX  LSCALX
!     Fortran  functions called:
!        ADJUSTL  MIN
!
!-----------------------------------------------------------------------

   USE carl
   USE gmpchr
   USE gmpint
   USE gmplgl
   USE pgmgmp
   USE systm
   USE termnl

   IMPLICIT NONE

   !  Interface block
   !  ---------------

   INTERFACE
      SUBROUTINE chkunt( lerr,iout,iunt,u1,u2,u3,u4,u5,u6,u7,u8,u9,u10,u11,u12 )
         IMPLICIT NONE
         INTEGER, INTENT(IN) :: iout, iunt
         INTEGER, INTENT(IN), OPTIONAL :: u1, u2, u3, u4, u5, u6, u7, u8, u9, u10, u11, u12
         LOGICAL, INTENT(OUT) :: lerr
      END SUBROUTINE chkunt
   END INTERFACE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: k1
   INTEGER, INTENT(INOUT) :: kn
   LOGICAL, INTENT(OUT) :: lerr

   !  Local declarations
   !  ------------------

   INTEGER, EXTERNAL :: iscalx
   LOGICAL, EXTERNAL :: lscalx

   INTEGER :: i, icof, icol, icov, jcol, jnpux, nkar, nladd, numi
   LOGICAL :: lexis, lopen, newp
   CHARACTER(LEN=LEN(filee)) :: oldnm
   CHARACTER(LEN=11) :: frm

   INCLUDE 'gmpprop.'

   ! nmotc - number of keywords in this paragraph
   ! motc  - the keywords (possibly preceded by sep2 character) of the
   !         paragraph, in chronological order of their analysis
   ! lngm(.,1) - full length of elements of motc
   ! lngm(.,2) - length of maximum abreviated form of elements of motc

   !--------------------------------------------------------------------

   lerr = .false.

   nvarin = iscalx( motc(1), lngm(1,1), lngm(1,2), k1, kn, nvarin, icov, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(1:), nmotc )
      return
   end if

   nvadd = iscalx( motc(2), lngm(2,1), lngm(2,2), k1, kn, nvadd, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(2:), nmotc-1 )
      return
   end if

   nindv = iscalx( motc(3), lngm(3,1), lngm(3,2), k1, kn, nindv, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(3:), nmotc-2 )
      return
   end if

      ! Save previous input unit number
   jnpux = inpux
   inpux = iscalx( motc(4), lngm(4,1), lngm(4,2), k1, kn, inpux, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(4:), nmotc-3 )
      return
   end if
   lgmpf = inpux == jnpux

      ! Save previous file name
   if ( lgmpf ) oldnm = filee
   filee = ' '
   call cscalx( motc(5), lngm(5,1), lngm(5,2), k1, kn, filee, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(5:), nmotc-4 )
      return
   end if
   filee = adjustl( filee )
      ! Convert file name to capitals if not unix system
   if ( .not. lunix .and. icol > 0 ) call cvuct( filee )
   if ( lgmpf ) lgmpf = filee == oldnm

   call cscalx( motc(6), lngm(6,1), lngm(6,2), k1, kn, fmtin, icof, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(6:), nmotc-5 )
      return
   end if
      ! Convert format to capitals
   if ( icof > 0 ) call cvuct( fmtin )

   call cscalx( motc(7), lngm(7,1), lngm(7,2), k1, kn, titre, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(7:), nmotc-6 )
      return
   end if
   titl2(lphdr+1:) = ' - ' // titre(:lrecp-(3+lphdr))

   call cscalx( motc(8), lngm(8,1), lngm(8,2), k1, kn, contin, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(8:), nmotc-7 )
      return
   end if
      ! Reset kontin code to proper value according to contin
   if ( icol > 0 ) then
         ! Convert to capitals
      call cvuct( contin )
      if ( contin(:3) /= zdata ) kontin = 0
   end if

   lrwde = lscalx( motc(9), lngm(9,1), lngm(9,2), k1, kn, lrwde, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(9:), nmotc-8 )
      return
   end if
   lgmpf = lgmpf .and. .not. lrwde

   lbref = lscalx( motc(10), lngm(10,1), lngm(10,2), k1, kn, lbref, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(10:), nmotc-9 )
      return
   end if

   lpdat = lscalx( motc(11), lngm(11,1), lngm(11,2), k1, kn, lpdat, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(11:), nmotc-10 )
      return
   end if

   ! Commands filtering
   ! ------------------
   nkar = ncar
   call cmprss( k1, kn-1 )
   kn = kn - nkar + ncar
   if ( kn > k1 ) then
      nladd = 3 + (kn-(k1+1))/(lrecp-1)
      call chkpg( nladd, .false., newp )
      write(imp,111) (car(i:min(i+lrecp-2,kn)),i=k1+1,kn,lrecp-1)
      lines = lines + nladd
      write(lgu,*) mrker1//mrker1
      write(lgu,111) (car(i:min(i+lrecp-2,kn)),i=k1+1,kn,lrecp-1)
      write(lgu,*) mrker2//mrker2
      car(k1:kn-1) = ' '
      call cmprss( k1, kn-1 )
   end if

   ! Prepare input data file
   ! -----------------------
      ! What kind of data reading ?
   if ( fmtin(1:3) == hdrgmp ) then
         ! gmp file
      lcode = .true.
      frm   = 'UNFORMATTED'
      lectx = 3
   else
      if ( icov+icof > 0 ) then
         lcode = .false.
         frm   = 'FORMATTED  '
         if ( fmtin(1:3) == zfree ) then
               ! Free format
            lectx = 1
         else if ( fmtin(1:1) == '(' ) then
               ! Fortran format
            lectx = 2
         else if ( fmtin(1:3) == 'BIN' .or. fmtin(2:4) == 'BIN' ) then
               ! Binary (unformatted) input, single or double precision
            frm   = 'UNFORMATTED'
            lectx = 4
            if ( fmtin(1:1) == 'S' ) lectx = 5
         else
               ! Unknown
!!          call chkpg( 2, .false., newp )
            write(lgu,*) mrker1
            write(lgu,555)
            write(lgu,*) mrker2
!!          lines = lines + 2
            lerr = .true.
            return
         end if
      else
            ! All values from previous problem, lectx is unchanged
         if ( lectx > 2 ) then
            frm = 'UNFORMATTED'
         else
            frm = 'FORMATTED  '
         end if
      end if
      if ( .not. lcode .and. nvarin <= 0 .and. kontin > 0 ) then
!!       call chkpg( 2, .false., newp )
         write(lgu,*) mrker1
         write(lgu,222)
         write(lgu,*) mrker2
!!       lines = lines + 2
         lerr = .true.
         return
      end if
   end if

      ! Check input unit number is permitted
   call chkunt( lerr, imp, inpux, iu1, iu2, iu3, iu4, imp, ksave, lec, lec1, lec2 )
   if ( lerr ) return

      ! Open input unit with or without filename, unless input is by
      ! the same gmp file as in previous problem and data are not to
      ! be rewinded
   lgmpf = lgmpf .and. lcode
   if ( .not. lgmpf ) then
      if ( filee(1:1) == ' ' ) then
         call filopn( 21, inpux, filee, 'OLD', frm, .true., lerr )
         if ( lerr ) return
      else
            ! Check the file already exists
         inquire ( file=filee, exist=lexis, opened=lopen, number=numi )
         if ( .not. (lopen .and. lexis .and. numi == inpux) ) then
            if ( lopen .and. lexis .and. numi /= inpux ) then
               call chkpg( 7, .false., newp )
               write(imp,333)
               write(imp,*) filee(:len_trim(filee))
               write(imp,444) numi, inpux
               lines = lines + 7
               write(lgu,*) mrker1//mrker1
               write(lgu,333)
               write(lgu,*) filee(:len_trim(filee))
               write(lgu,444) numi, inpux
               write(lgu,*) mrker2//mrker2
               close (numi)
            end if
            call filopn( 21, inpux, filee, 'OLD', frm, .true., lerr )
            if ( lerr ) return
         end if
      end if
   end if

   INCLUDE 'gmpf111.'
   INCLUDE 'gmpprof.'

END SUBROUTINE gmppro
