SUBROUTINE gmpprx( ierr )

!***********************************************************************
!     - gmpprx -
!     prints problem and dictionnary information and directs reading
!     (and saving) of input data; deallocates "transformations" arrays
!     if relevant
!***********************************************************************
!
!     IERR  - OUTPUT. Error code. 0 if no error               - INTEGER
!                     1: content of input GMP-file is not data
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!     gmplgl : lgmpf
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  GMPGFW  GMPPRD  GMPPRP  GMPXIN
!     External functions called:
!        NONE
!     Fortran  functions called:
!        DEALLOCATE  LEN_TRIM  MAX  MIN
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE gmplgl
   USE memst1
   USE memsta
   USE pgmgmp
   USE spcnom
   USE trancb

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(OUT) :: ierr

   !  Local declarations
   !  ------------------

   INTEGER :: i, iout, irw, itemp, jendf, kodsv, kontex
   INTEGER :: l, lc, lf, ll, lll
   INTEGER :: nerr, nladd, nladp, nlr, nmis, nout, nsav, nval, nvpl
   LOGICAL :: lchk, newp, flnam
   INCLUDE 'gmpprxp.'

   !--------------------------------------------------------------------

   ierr  = 0
   iout  = imp
   nval  = max( 1, l_vals )

   ! PRINT PROBLEM INFORMATION AND DICTIONNARY OF VARIABLES
   ! ------------------------------------------------------
   if ( linter ) call chkpg( 0, .true., newp )
   call gmpprp( iout, vuse, unom, vnom(1:), .true. )
   if ( .not. lbref ) then
      call gmpprd( iout, xmis, xmin, xmax, vals, vuse, lval, lnom,   &
                   vnom(1:), noms, nval, .true. )
   end if

   if ( ltran ) then
!!    call chkpg( 2, .false., newp )
      write(lgu,111)
!!    lines = lines + 2
   end if

   ! PREPARE SAVE FILE AND READ INPUT DATA
   ! -------------------------------------
      ! Open of input file was done in routine gmppro
      ! If input file is a gmp one, note that dictionnary information
      ! stored before data has already been read in, and so the file
      ! must not be rewinded

   if ( lcode .and. .not. lgmpf ) then
         ! Check content of input gmp file is data, as expected
      read(inpux) kontex
      if ( kontex /= 1 ) then
!!       call chkpg( 4, .false., newp )
         write(lgu,*) ermrk1
         write(lgu,1111) hdrgmp, kontex
         write(lgu,*) ermrk2
!!       lines = lines + 4
         ierr  = 1
         return
      end if
      lgmpf = .true.
   else
      if ( lrwde ) rewind inpux
   end if

      ! Take proper action if data saving is requested (flagged by kodsv)
   if ( lsave .and. kontsv > 0 ) then
      if ( lrwds ) then
         rewind isave
         irw = 1
      else
         irw = 2
      end if
      if ( lcods ) then
            ! Read data and save on gmp save file
         kodsv = 3
            ! Save dictionnary information on gmp save file
         call gmpgfw( nval )
            ! Store content code on gmp save file
         write(isave) kontsv
      else
            ! Read data and save on user's save file
            ! Fortran format
         kodsv = 2
         if ( fmtsv(:3) == zfree ) then
               ! Free format
            kodsv = 1
         else
            if ( fmtsv(:3) == 'BIN' .or. fmtsv(2:4) == 'BIN' ) then
                  ! Binary, real*8 or real
               kodsv = 4
               if ( fmtsv(1:1) == 'S' ) kodsv = 5
            end if
         end if
      end if

   else
         ! Read data but don't save it
      kodsv = 0

   end if

   ! READ DATA AND POSSIBLY PRINT AND SAVE DATA
   ! ------------------------------------------
   call gmpxin( nerr, nmis, nsav, kodsv )

   nout  = nindv - nindvu
   nladd = 7
   call chkpg( nladd, .false., newp )
   write(imp,222) nindv, nout, nerr, nindvu, nmis
   lines = lines + nladd

   if ( kodsv > 0 ) then

      nladd = 4
      flnam = files(1:1) /= ' '
      if ( flnam ) then
         lf = len_trim( files )
         nladd = nladd + 1 + (lf-1)/(lrecp-1) + 1
      end if
      if ( lcods ) then
         lc = len_trim( codes )
      else
         if ( kodsv == 2 ) then
               ! Find actual end of output format (to avoid printing
               ! of irrelevant lines)
            jendf = len_trim( fmtsv )
         else
            jendf = 12
         end if
         nladd = nladd + (jendf-1)/(lrecp-1) + 1
      end if

      lchk = .true.
         ! Print variables saved if not all
      if ( .not. savall ) then
            ! Extended output (variable numbers and names)
            ! nvpl = no. of variables printed/line(see format 333)
            ! nlr  = no. of lines still to be printed
            ! nladp= no. of lines in current page
         if ( lchk ) then
            call chkpg( 3, .false., newp )
         else
            itemp = lines
         end if
         write(imp,3333)
         lines = lines + 2
         nvpl  = 4
         nlr   = (nvsav-1)/nvpl + 1
         lll   = 0
         do
            if ( lchk ) then
               nladp = min( nlr, max( linep-lines, 1 ) )
            else
               nladp = nlr
            end if
            do l = 1, nladp
               ll  = lll + 1
               lll = min( nvsav, lll+nvpl )
               write(imp,333) (i,vnom(vsav(i)),i=ll,lll)
            end do
            lines = lines + nladp
            nlr   = nlr   - nladp
            if ( nlr > 0 ) then
               if ( lchk ) call chkpg( 0, .true., newp )
               cycle
            end if
            exit
         end do
            if ( .not. lchk ) lines = itemp
      end if
         ! Print other informations
      if ( lchk ) call chkpg( nladd, .false., newp )
      write(imp,444) isave, rep(irw)
      if ( flnam ) then
         write(imp,555)
         write(imp,2222) (files(i:min(i+lrecp-2,lf)),i=1,lf,lrecp-1)
      end if
      if ( lcods ) then
         write(imp,666) hdrgmp
         write(imp,2222) codes(1:min(lrecp-1,lc))
      else
            ! Print output format
         write(imp,777)
         if ( kodsv < 4 ) then
            write(imp,2222) (fmtsv(i:min(i+lrecp-2,jendf)),i=1,jendf,lrecp-1)
         else
            write(imp,888) binsv(kodsv-3)
         end if
      end if
      write(imp,999) nsav
      lchk = .false.

      lines = lines + nladd
   end if

   ! IF RELEVANT, DEALLOCATE ARRAYS USED FOR TRANSFORMATIONS
   ! -------------------------------------------------------
   if ( ltran ) then
      deallocate( xtra, adr, adrv, nato, nope, nvaf, next1, next2, pilif, pilel, lcond )
   end if

   INCLUDE 'gmpprxf.'

END SUBROUTINE gmpprx
