SUBROUTINE transn( car, com, lastk, nf )

!***********************************************************************
!     - transn -
!     (over)estimates the number of formulas used for "transformations"
!***********************************************************************
!
!     car   - INPUT . Characters string containing the        - CHAR
!                     transformations (with delimiting
!                     apostrophs removed)
!     com   - INPUT . A logical vector such that com(i) is    - LOGICAL
!                     .true. if car(i:i) was inbeded in
!                     apostrophs, and .false. otherwise
!     lastk - INPUT . Length of car and com                   - INTEGER
!     nf    - OUTPUT. Number of formulas found                - INTEGER
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        INDEX
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: lastk
   CHARACTER(LEN=*), INTENT(IN) :: car
   LOGICAL, DIMENSION(lastk), INTENT(IN) :: com
   INTEGER, INTENT(OUT) :: nf

   !  Local declarations
   !  ------------------

   INTEGER :: i, kol

   !--------------------------------------------------------------------

      ! Note.
      ! An IF()THEN statement corresponds to 1 formula,
      ! a DO-loop corresponds to 5 formulas,
      ! and an assignement (=) corresponds to 1 or 2 formulas
      ! (2 if the variable operator is present on left-hand size of = sign)

   nf = 0

      ! Count conditions ( IF(...)THEN )
      ! --------------------------------
   kol = 1
   do
      i = index( car(kol:lastk), 'IF(' ) + kol-1
      if ( i >= kol ) then
         kol = i + 3
         if ( com(i) ) cycle
         nf = nf + 1
         if ( kol >= lastk ) exit
      else
         exit
      end if
   end do

      ! Count do-loops
      ! --------------
   kol = 1
   do
      i = index( car(kol:lastk), 'DO_' ) + kol-1
      if ( i >= kol ) then
         kol = i + 3
         if ( com(i) ) cycle
         nf = nf + 5
         if ( kol >= lastk ) exit
      else
         exit
      end if
   end do

      ! Count algebraic formulas
      ! ------------------------
   kol = 1
   do
      i = index( car(kol:lastk), '=' ) + kol-1
      if ( i >= kol ) then
         kol = i + 1
         if ( com(i) ) cycle
         nf = nf + 2      ! Always assume 2 formulas
         if ( kol >= lastk ) exit
      else
         exit
      end if
   end do

END SUBROUTINE transn
