SUBROUTINE varid1( nbreva, nomvar, lonvar, adrv, numv, nom, ll, ic, lerr )

!***********************************************************************
!     - varid1 -
!     searches for a name of variable, the length of target name being
!     known (search is done by descending order of variable-name
!     lengths), and updates array of addresses of variables
!     N.B. Search is insensitive to case type of letters
!***********************************************************************
!
!     NBREVA- INPUT . Number of variables                     - INTEGER
!     NOMVAR- INPUT . Names of variables                      - CHAR
!     LONVAR- INPUT . Their actual length                     - INTEGER
!     ADRV  - INPUT . Array of addresses of variables         - INTEGER
!             OUTPUT. Updated
!     NUMV  - INPUT . Array of indices of variables sorted    - INTEGER
!                     in descending order of name length
!     NOM   - INPUT . Name being tested                       - CHAR
!     LL    - INPUT . Its length                              - INTEGER
!     IC    - INPUT . Counter                                 - INTEGER
!             OUTPUT. + 1 if name has been found
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     opprm : natom   nvarf
!
!----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        SAMSTR
!     Fortran  functions called:
!        MIN
!
!-----------------------------------------------------------------------

   USE opprm

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: nbreva, ll
   CHARACTER(LEN=*), INTENT(IN) :: nom
   INTEGER, DIMENSION(nbreva), INTENT(IN) :: lonvar, numv
   CHARACTER(LEN=*), DIMENSION(nbreva), INTENT(IN) :: nomvar
   INTEGER, INTENT(INOUT) :: ic
   INTEGER, DIMENSION(2,maxva), INTENT(INOUT) :: adrv
   LOGICAL, INTENT(OUT) :: lerr

   !  Local declarations
   !  ------------------

   LOGICAL, EXTERNAL :: samstr
   INTEGER :: i, j
   INCLUDE 'varid12p.'

   !--------------------------------------------------------------------

   lerr = .false.

   do j = 1, nbreva
      i = numv(j)
      if ( ll > lonvar(i) ) exit
      if ( ll == lonvar(i) ) then
         if ( samstr( nom(1:ll), nomvar(i)(1:ll) ) ) then
            natom = natom + 1
            nvarf = nvarf + 1
            if ( natom > maxat ) then
               write(io,*) ewmrk1
               write(io,222) msgato
               write(io,*) ewmrk2
               lerr = .true.
               return
            end if
            if ( nvarf > maxva ) then
               write(io,*) ewmrk1
               write(io,222) msgvar
               write(io,*) ewmrk2
               lerr = .true.
               return
            end if
            adrv(1,nvarf) = i
            adrv(2,nvarf) = natom
            ic = ic + 1
            return
         end if
      end if
   end do

   write(io,*) ewmrk1
   write(io,111) nom(1:min(ll,30))
   write(io,*) ewmrk2
   lerr = .true.

   INCLUDE 'varid12f.'

END SUBROUTINE varid1
