SUBROUTINE varid3( nvar, nvart, nvatp, nomvar, lonvar, nom, lon, numv, lerr )

!***********************************************************************
!     - varid3 -
!     finds the index in nomvar of a variable name (with possible
!     updating of nomvar)
!     N.B. Search is insensitive to case type of letters
!***********************************************************************
!
!     NVAR  - INPUT . Number of variables                     - INTEGER
!     NVART - INPUT . Dimension of NOMVAR, LONVAR             - INTEGER
!                     If NVART > NVAR, then the difference
!                     is the maximum number of temporary
!                     variables that can be used
!     NVATP - INPUT . NVAR + number of temporary variables    - INTEGER
!                     already used
!             OUTPUT. Updated value (+1 if new temp. var.)
!     NOMVAR- INPUT . Names of variables                      - CHAR
!             OUTPUT. Updated list if NOM is a new temp. var.
!     LONVAR- INPUT . Their actual length                     - INTEGER
!             OUTPUT. Updated list if NOM is a new temp. var.
!     NOM   - INPUT . Name being searched for                 - CHAR
!     LON   - INPUT . Its length                              - INTEGER
!     NUMV  - OUTPUT. The index of NOM in NOMVAR              - INTEGER
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        SAMSTR
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------

   USE opprm

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: nvar, nvart, lon
   CHARACTER(LEN=*), INTENT(IN) :: nom
   INTEGER, INTENT(INOUT) :: nvatp
   INTEGER, DIMENSION(nvart), INTENT(INOUT) :: lonvar
   CHARACTER(LEN=*), DIMENSION(nvart), INTENT(INOUT) :: nomvar
   INTEGER, INTENT(OUT) :: numv
   LOGICAL, INTENT(OUT) :: lerr

   !  Local declarations
   !  ------------------

   LOGICAL, EXTERNAL :: samstr
   INTEGER :: i, lv

   !--------------------------------------------------------------------

   lerr = .false.

   do i = 1, nvatp
      lv = lonvar(i)
      if ( lon == lv ) then
         if ( samstr( nom(1:lv), nomvar(i)(1:lv) ) ) then
            numv = i
            return
         end if
      end if
   end do

   if ( nvart > nvar ) then
      if ( nvatp < nvart ) then
         nvatp         = nvatp + 1
         nomvar(nvatp) = nom(:lon)
         lonvar(nvatp) = lon
         numv          = nvatp
      else
         write(io,*) ewmrk1
         write(io,222) nvart-nvar, nom
         write(io,*) ewmrk2
         lerr = .true.
      end if
   else
      write(io,*) ewmrk1
      write(io,111) nom
      write(io,*) ewmrk2
      lerr = .true.
   end if

   INCLUDE 'varid3f.'

END SUBROUTINE varid3
