SUBROUTINE getdfc( cov, nw, nr, dfc )

!***********************************************************************
!     - getdfc -
!     computes a symmetric matrix of indices of Departure From
!     Circularity assumption for a given covariance matrix
!***********************************************************************
!
!     COV   - INPUT . Covariance matrix of order NW*NR,       - REAL*8
!                     stored in triangular form
!     NW    - INPUT . Number of W-contrasts                   - INTEGER
!     NR    - INPUT . Number of dependent variables           - INTEGER
!     DFC   - OUTPUT. Matrix of indices of departure from     - REAL*8
!                     circularity, of order NR,
!                     stored in triangular form
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: nw, nr
   REAL(KIND=8), INTENT(IN) :: cov(nw*nr*(nw*nr+1)/2)
   REAL(KIND=8), INTENT(OUT) :: dfc(nr*(nr+1)/2)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8, one=1.0_8, two=2.0_8
   REAL(KIND=8) :: s, t, w
   INTEGER :: i, ij, iw, j, jw, k, l, m

   !--------------------------------------------------------------------

      ! Calculate the matrix of indices of departure (dfc)
      ! (the index is a ratio with square of trace as numerator, and
      ! nw times the sum of square elements as denominator)
   w  = nw
   ij = 0
   iw = 0
   do i = 1, nr
      jw = 0
      do j = 1, i
         t = zero
         s = zero
         do k = 1, nw
            m = (iw+k)*(iw+k-1)/2 + jw
            t = t + cov(m+k)
            s = s + cov(m+k)**2
            do l = 1, k-1
               s = s + two*cov(m+l)**2
            end do
         end do
         ij = ij + 1
         if ( s > zero ) then
            dfc(ij) = t*t / (w*s)
         else
            dfc(ij) = one
         end if
         jw = jw + nw
      end do
      iw = iw + nw
   end do

END SUBROUTINE getdfc
