SUBROUTINE pacaf2( form, nk, kol, ikar, gstr, tstr, gmod, tmod,   &
                   lexs, flag, mata, kfac, vide, lerr )
!***********************************************************************
!     - pacaf2 -
!     analyzes "union of simple parts" in the formula,
!     i.e. a sequence   x_y_z etc   where x y z are "simple parts".
!     Updates the logical G&T structure-matrix mata
!***********************************************************************
!
!     FORM  - INPUT . The formula                             - CHAR
!     NK    - INPUT . Location in FORM of the last relevant   - INTEGER
!                     character
!     KOL   - INPUT . Starting location in FORM of the zone   - INTEGER
!                     analyzed
!             OUTPUT. Location in FORM of the character
!                     following the "union"
!     IKAR  - OUTPUT. JCHAR rank of FORM(KOL:KOL), or IACHAR  - INTEGER
!                     rank of blank if the end of formula
!                     has been reached
!                     (JCHAR rank is IACHAR rank but with lower
!                     case letter returning same rank as the
!                     corresponding upper case letter)
!     GSTR  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     TSTR  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     GMOD  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     TMOD  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     LEXS  - INPUT . A logical G&T matrix                    - LOGICAL
!                     where existing cells are .true.
!     FLAG  - INPUT . Blank string used to enhance error      - CHAR
!                     messages (of length > 50)
!     MATA  - INPUT . Logical G&T structure-matrix            - LOGICAL
!             OUTPUT. Cells not indexed by modalities in the
!                     union are reset to .false.
!     KFAC  - OUTPUT.    see definition in PACAF1             - INTEGER
!     VIDE  - OUTPUT. .true. if all cells in MATA are .false. - LOGICAL
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  PACAF1
!     External functions called:
!        NONE
!     Fortran  functions called:
!        ANY  MAX  MIN
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE gmplgl
   USE pacint
   USE pacrnk

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: nk
   CHARACTER(LEN=*), INTENT(IN) :: form, flag
   INTEGER, INTENT(IN) :: gstr(ngrp,ngfac), tstr(ntrt,ntfac)
   INTEGER, INTENT(IN) :: gmod(ngfac), tmod(ntfac)
   LOGICAL, INTENT(IN) :: lexs(ntrt,ngrp)
   INTEGER, INTENT(INOUT) :: kol
   LOGICAL, INTENT(INOUT) :: mata(ntrt,ngrp)
   INTEGER, INTENT(OUT) :: ikar, kfac
   LOGICAL, INTENT(OUT) :: vide, lerr

   !  Local declarations
   !  ------------------

   INTEGER :: kola, kolz
   LOGICAL :: newp
   CHARACTER(LEN=1), PARAMETER :: mark='$'
   LOGICAL :: matb(ntrt,ngrp)

   !--------------------------------------------------------------------

   lerr = .false.

   ! Analyze first term
   ! ------------------
   call pacaf1( form, nk, kol, ikar, gstr, tstr, gmod, tmod,   &
                flag, mata, kfac, vide, lerr )
   if ( lerr ) return
      ! If only one "simple part", no union needed
   if ( ikar /= kor .or. kfac /= 0 ) return

   do   ! Looping on union terms
        ! ----------------------

         ! Analyze current term (resultant matrix is matb)
         ! First, initialize matb
      matb = lexs
      kol = kol + 1
      call pacaf1( form, nk, kol, ikar, gstr, tstr, gmod, tmod,   &
                   flag, matb, kfac, vide, lerr )
      if ( lerr ) return
      if ( kfac /= 0 ) then
            ! If a factor appears here, it is an error
         kola = max( kol-26, 1 )
         kolz = min( kol+24, nk )
!!       call chkpg( 6, .false., newp )
         write(lgu,*) ermrk1
         write(lgu,111) form(kola:kolz), flag(:kol-kola), mark
         write(lgu,222)
         write(lgu,*) ermrk2
!!       lines = lines + 6
         lerr = .true.
         return
      end if
         ! Take the union of current term (in matb) with previous
         ! ones (in mata) (result in mata)
      mata = mata .or. matb
      vide = .not. any(mata)

         ! More terms ?
      if ( ikar /= kor ) exit
   end do

   INCLUDE 'pacaf2f.'

END SUBROUTINE pacaf2
