SUBROUTINE pacaf4( form, lenf, kol, ikar, gstr, tstr, gmod, tmod,   &
                   lexs, flag, part, npar, allgt, lerr )
!***********************************************************************
!     - pacaf4 -
!     analyzes a "between-parts comparison",
!     i.e. a sequence   a,b,c etc   where a, b, c are some "composition"
!     The analysis results in a matrix representing the partition
!     of G&T corresponding to the comparison
!***********************************************************************
!
!     FORM  - INPUT . The formula                             - CHAR
!     LENF  - INPUT . Location in FORM of the last relevant   - INTEGER
!                     character
!     KOL   - INPUT . Starting location in FORM of the zone   - INTEGER
!                     analyzed
!             OUTPUT. Location in FORM of the character
!                     following the "composition"
!     IKAR  - OUTPUT. JCHAR rank of FORM(KOL:KOL), or IACHAR  - INTEGER
!                     rank of blank if the end of formula
!                     has been reached
!                     (JCHAR rank is IACHAR rank but with lower
!                     case letter returning same rank as the
!                     corresponding upper case letter)
!     GSTR  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     TSTR  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     GMOD  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     TMOD  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     LEXS  - INPUT . A logical G&T matrix                    - LOGICAL
!                     where existing cells are .true.
!     FLAG  - INPUT . Blank string used to enhance error      - CHAR
!                     messages (of length > 50)
!     PART  - OUTPUT. G&T partition-matrix                    - INTEGER
!     NPAR  - OUTPUT. Number of parts in the partition, i.e.  - INTEGER
!                     number of terms in the comparison
!                     (the complementary part is not counted)
!     ALLGT - INPUT . .true. if all cells in LEXS are .true.  - LOGICAL
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  PACAF0  PACAF3
!     External functions called:
!        NONE
!     Fortran  functions called:
!        MAX  MIN
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE gmplgl
   USE pacint
   USE pacrnk

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: lenf
   CHARACTER(LEN=*), INTENT(IN) :: form, flag
   LOGICAL, INTENT(IN) :: allgt
   INTEGER, INTENT(IN) :: gstr(ngrp,ngfac), tstr(ntrt,ntfac)
   INTEGER, INTENT(IN) :: gmod(ngfac), tmod(ntfac)
   LOGICAL, INTENT(IN) :: lexs(ntrt,ngrp)
   INTEGER, INTENT(INOUT) :: kol
   INTEGER, INTENT(OUT) :: ikar, npar
   LOGICAL, INTENT(OUT) :: lerr
   INTEGER, INTENT(OUT) :: part(ntrt,ngrp)

   !  Local declarations
   !  ------------------

   INTEGER :: i, j, kfac, kola, kolz
   LOGICAL :: more, newp, none, ovlp, vide
   CHARACTER(LEN=1), PARAMETER :: mark='$'
   LOGICAL :: mat1(ntrt,ngrp)

   !--------------------------------------------------------------------

   ! Initialize
   ! ----------
   lerr = .false.
   part = 0
   mat1 = lexs
   more = .false.
   npar = 0
   kol  = kol - 1

   do   ! Looping on between-comparison terms
        ! -----------------------------------
      kol = kol + 1

         ! Analyze new term
      call pacaf3( form, lenf, kol, ikar, gstr, tstr, gmod, tmod,   &
                   lexs, flag, mat1, kfac, vide, lerr )
      if ( lerr ) return

      if ( kfac /= 0 ) then
         if ( more ) then
               ! If a complete factor appears here, it is an error
            kola = max( kol-27, 1 )
            kolz = min( kol+23, lenf )
!!          call chkpg( 6, .false., newp )
            write(lgu,*) ermrk1
            write(lgu,111) form(kola:kolz), flag(:kol-1-kola), mark
            write(lgu,333)
            write(lgu,*) ermrk2
!!          lines = lines + 6
            lerr = .true.
            return
         else
               ! Special analysis for complete factors
            call pacaf0( form, lenf, kol, ikar, gstr, tstr, gmod, tmod,   &
                         lexs, flag, kfac, part, npar, allgt, lerr )
            return
         end if
      end if

      if ( .not. vide ) then
            ! The term does exist, unless complete overlapping with
            ! previous parts (which will be flagged by none)
            ! Update part and initialize mat1 for next pass
         none = .true.
         ovlp = .false.
         npar = npar + 1
         do j = 1, ngrp
            do i = 1, ntrt
               if ( mat1(i,j) ) then
                  if ( part(i,j) <= 0 ) then
                     part(i,j) = npar
                     none = .false.
                  else  ! Part-overlapping within a comparison:
                        ! the previous part-number is kept
                     ovlp = .true.
                  end if
               else
                  mat1(i,j) = lexs(i,j)
               end if
            end do
         end do
         if ( none ) then
            npar = npar - 1
            vide = .true.
         end if
         if ( ovlp ) then
            kola = max( kol-25, 1 )
            kolz = min( kol+25, lenf )
            call chkpg( 6, .false., newp )
            write(imp,222) form(kola:kolz), flag(1:kol-kola+1), mark
            write(imp,555)
            lines = lines + 6
            write(lgu,*) ermrk1//ermrk1
            write(lgu,222) form(kola:kolz), flag(1:kol-kola+1), mark
            write(lgu,555)
            write(lgu,*) ermrk2//ermrk2
         end if
      end if

      if ( vide ) then
            ! The term doesn't really exist, and so is skipped
         kola = max( kol-25, 1 )
         kolz = min( kol+25, lenf )
         call chkpg( 5, .false., newp )
         write(imp,222) form(kola:kolz), flag(1:kol-kola+1), mark
         write(imp,444)
         lines = lines + 5
         write(lgu,*) ermrk1//ermrk1
         write(lgu,222) form(kola:kolz), flag(1:kol-kola+1), mark
         write(lgu,444)
         write(lgu,*) ermrk2//ermrk2
            ! Initialize mat1 for next pass
         mat1 = lexs
      end if

         ! More terms ?
      more = .true.
      if ( ikar /= kcomp ) exit
   end do

   INCLUDE 'pacaf4f.'

END SUBROUTINE pacaf4
