SUBROUTINE pacaf5( form, lenf, kol, ikar, gstr, tstr, gmod, tmod,   &
                   lexs, flag, part, npart, allgt, lerr )
!***********************************************************************
!     - pacaf5 -
!     Analyzes a "between parts comparison" made of products of
!     "individual between parts comparisons", i.e. a sequence  a*b*c etc
!     where a, b, c are some "between parts comparisons".
!     The analysis results in a G&T partition matrix
!***********************************************************************
!
!     FORM  - INPUT . The formula                             - CHAR
!     LENF  - INPUT . Location in FORM of the last relevant   - INTEGER
!                     character
!     KOL   - INPUT . Starting location in FORM of the zone   - INTEGER
!                     analyzed
!             OUTPUT. Location in FORM of the character
!                     following the product
!     IKAR  - OUTPUT. JCHAR rank of FORM(KOL:KOL), or IACHAR  - INTEGER
!                     rank of blank if the end of formula
!                     has been reached
!                     (JCHAR rank is IACHAR rank but with lower
!                     case letter returning same rank as the
!                     corresponding upper case letter)
!     GSTR  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     TSTR  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     GMOD  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     TMOD  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     LEXS  - INPUT . A logical G&T matrix                    - LOGICAL
!                     where existing cells are .true.
!     FLAG  - INPUT . Blank string used to enhance error      - CHAR
!                     messages (of length > 50)
!     PART  - OUTPUT. G&T partition-matrix                    - INTEGER
!     NPART - OUTPUT. Number of parts of this partition       - INTEGER
!     ALLGT - INPUT . .true. if all cells in LEXS are .true.  - LOGICAL
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  CONSEC  PACAF4
!     External functions called:
!        NONE
!     Fortran  functions called:
!        MAX  MIN
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE gmplgl
   USE pacint
   USE pacrnk
   USE spcval

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: lenf
   CHARACTER(LEN=*), INTENT(IN) :: form, flag
   LOGICAL, INTENT(IN) :: allgt
   INTEGER, INTENT(IN) :: gstr(ngrp,ngfac), tstr(ntrt,ntfac)
   INTEGER, INTENT(IN) :: gmod(ngfac), tmod(ntfac)
   LOGICAL, INTENT(IN) :: lexs(ntrt,ngrp)
   INTEGER, INTENT(INOUT) :: kol
   INTEGER, INTENT(OUT) :: ikar, npart
   LOGICAL, INTENT(OUT) :: lerr
   INTEGER, INTENT(OUT) :: part(ntrt,ngrp)

   !  Local declarations
   !  ------------------

   INTEGER :: i, j, k, kola, kolz, npar
   LOGICAL :: newp
   CHARACTER(LEN=1), PARAMETER :: mark='$'
   INTEGER :: pwrk(ntrt,ngrp)

   !--------------------------------------------------------------------

   lerr = .false.

   ! Analyze first term
   ! ------------------
   call pacaf4( form, lenf, kol, ikar, gstr, tstr, gmod, tmod,   &
                lexs, flag, part, npart, allgt, lerr )
   if ( lerr ) return
      ! Check for an empty partition
   if ( npart <= 0 ) then
      kola = max( kol-25, 1 )
      kolz = min( kol+25, lenf )
!!    call chkpg( 5, .false., newp )
      write(lgu,*) ermrk1
      write(lgu,111) form(kola:kolz), flag(1:kol-kola+1), mark
      write(lgu,222)
      write(lgu,*) ermrk2
!!    lines = lines + 5
      lerr = .true.
      return
   end if

   ! Analyze the product (if any)
   ! ----------------------------
   if ( ikar == kprod .or. ikar == knest ) then

      do   ! Looping on product terms
         kol = kol + 1

            ! Analyze new term
         call pacaf4( form, lenf, kol, ikar, gstr, tstr, gmod, tmod,   &
                     lexs, flag, pwrk, npar, allgt, lerr )
         if ( lerr ) return
            ! Check for an empty partition
         if ( npar <= 0 ) then
            kola = max( kol-25, 1 )
            kolz = min( kol+25, lenf )
!!          call chkpg( 5, .false., newp )
            write(lgu,*) ermrk1
            write(lgu,111) form(kola:kolz), flag(1:kol-kola+1), mark
            write(lgu,222)
            write(lgu,*) ermrk2
!!          lines = lines + 5
            lerr = .true.
            return
         end if

            ! Update part
         do j = 1, ngrp
            do i = 1, ntrt
               if ( part(i,j) > 0 ) then
                  if ( pwrk(i,j) > 0 ) then
                     part(i,j) = part(i,j)  +  (pwrk(i,j)-1) * npart
                  else
                     part(i,j) = 0
                  end if
               end if
            end do
         end do
         npart = npart * npar

            ! More product terms ?
         if ( ikar /= kprod .and. ikar /= knest ) exit
      end do

         ! Reset partition numbers to consecutive numbers:
         ! Gaps may arise when product of partitions occurs and
         ! when the corresponding cells do not all exist in the
         ! design (whether because originally empty or because
         ! a condition caused some groups/treatments to "disappear"
         ! or because of a nesting structure)
         ! On return from consec, npart is the actual number of
         ! parts in the partition-matrix
      call consec( part, ntrt*ngrp, 0, maxint, k, npart )
   end if

   INCLUDE 'pacaf5f.'

END SUBROUTINE pacaf5
