SUBROUTINE pacarf( form, lenr, gstr, tstr, gmod, tmod, lexs, flag, lres, lerr )

!***********************************************************************
!     - pacarf -
!     Analyzes the "Right part" of the Formula (the condition), which
!     follows the conditioning symbol,
!     i.e. a sequence like   a&b&c etc   where a, b, c are some
!     "unions of simple parts".
!     The analysis results in a "restricted" logical matrix where the
!     cells corresponding to the restriction are set to .true. and all
!     other ones to .false.
!***********************************************************************
!
!     FORM  - INPUT . The conditioning part of formula,       - CHAR
!                     including conditioning symbol
!     LENR  - INPUT . Location in FORM of the last relevant   - INTEGER
!                     character
!     GSTR  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     TSTR  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     GMOD  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     TMOD  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     LEXS  - INPUT . A logical G&T matrix                    - LOGICAL
!                     where existing cells in the design
!                     are .true.
!     FLAG  - INPUT . Blank string used to enhance error      - CHAR
!                     messages (of length > 50)
!     LRES  - OUTPUT. A logical G&T matrix                    - LOGICAL
!                     where existing cells in the design and
!                     in the condition are .true.
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  PACAF3
!     external functions called:
!        NONE
!     Fortran  functions called:
!        MAX  MIN
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE gmplgl
   USE pacint
   USE pacrnk

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: lenr
   CHARACTER(LEN=*), INTENT(IN) :: form, flag
   INTEGER, INTENT(IN) :: gstr(ngrp,ngfac), tstr(ntrt,ntfac)
   INTEGER, INTENT(IN) :: gmod(ngfac), tmod(ntfac)
   LOGICAL, INTENT(IN) :: lexs(ntrt,ngrp)
   LOGICAL, INTENT(OUT) :: lerr
   LOGICAL, INTENT(OUT) :: lres(ntrt,ngrp)

   !  Local declarations
   !  ------------------

   CHARACTER(LEN=1), PARAMETER :: mark='$'
   INTEGER :: ikar, kfac, kol, kola, kolz
   LOGICAL :: newp, vide

   !--------------------------------------------------------------------

   ! Initialize
   ! ----------
   lres = lexs

   lerr = .false.
   kol  = 2

   ! Analyze the composition
   ! -----------------------
   call pacaf3( form, lenr, kol, ikar, gstr, tstr, gmod, tmod,   &
                lexs, flag, lres, kfac, vide, lerr )
   if ( lerr ) return
      ! Check for empty composition
   if ( vide ) then
      kola = max( kol-25, 1 )
      kolz = min( kol+25, lenr )
!!    call chkpg( 5, .false., newp )
      write(lgu,*) ermrk1
      write(lgu,111) form(kola:kolz), flag(:kol-kola+1), mark
      write(lgu,222)
      write(lgu,*) ermrk2
!!    lines = lines + 5
      lerr = .true.
      return
   end if

   ! End of scanning, check end of formula has been reached
   ! ------------------------------------------------------
   if ( ikar /= kblan ) then
      kola = max( kol-25, 1 )
      kolz = min( kol+25, lenr )
!!    call chkpg( 5, .false., newp )
      write(lgu,*) ermrk1
      write(lgu,111) form(kola:kolz), flag(1:kol-kola+1), mark
      write(lgu,333)
      write(lgu,*) ermrk2
!!    lines = lines + 5
      lerr = .true.
   end if

   INCLUDE 'pacarff.'

END SUBROUTINE pacarf
