SUBROUTINE paccom( k1, kn, lerr, ndf1, lform, baylev, outlev, maxou, noutlv, rcoef, maxrc, nrco )

!***********************************************************************
!     - paccom -    ???????????? revoir dl>1 par vecteur w
!     analyzes COMp paragraph, with commands filtering.
!     In case of entry by vectors, arrays v and w are allocated and
!     entered.
!     Arrays hyp0, xlim possibly entered.
!***********************************************************************
!
!     K1    - INPUT . Starting location in CAR of the zone    - INTEGER
!                     analyzed. Unchanged on return
!     KN    - INPUT . Ending   location in CAR of the zone    - INTEGER
!                     analyzed (end-of-paragraph symbol)
!             OUTPUT. Destroyed
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!     NDF1  - OUTPUT. Degrees of freedom of the comparison    - INTEGER
!                     Relevant only for a comparison entered
!                     by vectors (otherwise not yet defined)
!     LFORM - INPUT . .true. if entry by formula in previous  - LOGICAL
!                     COMP paragragh
!             OUTPUT. .true. if entry by formula in current
!                     COMP paragragh
!     BAYLEV- INPUT . Probability level for bayesian          - REAL*8
!                     statements
!             OUTPUT.
!     OUTLEV- INPUT . Desired outputs for descriptive stat.   - CHAR
!             OUTPUT.
!     MAXOU - INPUT . Maximum length of OUTLEV                - INTEGER
!     NOUTLV- OUTPUT. Number of entries in OUTLEV             - INTEGER
!                     Unchanged if none
!     RCOEF - INPUT . Regression coefficients                 - REAL*8
!             OUTPUT.
!     MAXRC - INPUT . Maximum length of RCOEF                 - INTEGER
!     NRCO  - OUTPUT. Number of coefficients entered in RCOEF - INTEGER
!                     (returns MAXRC if none)
!
!     --------------------------------------------------------
!     Module elements modified:
!     carl   : car
!     gmpint : lines
!     memst2 : hyp0    xlim    v       w
!     pacchr : derfil  cmptit  derfmt  cmpgwo  cmptwo  cmpfor
!     pacint : ncv     ncw     nszu    ideriv  gpond   tpond
!     paclgl : lupvw   lbayes  lderiv  lrwdd   lspecf  limit
!              lfrapp  lanova  luniv   lmulti  lreduc  lpmean
!              lptabl  listdp
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        AVECTX  CHKPG  CHKUNT  CMPRSS  CSCALX  CVUCT  FILOPN  GMPWRN
!        GMP999  LOCAL  PAC999
!     External functions called:
!        ISCALX  LSCALX
!     Fortran  functions called:
!        ADJUSTL  ALLOCATE  ALLOCATED  DEALLOCATE  LEN_TRIM  MAX  MIN
!
!-----------------------------------------------------------------------

   USE carl
   USE gmpint
   USE gmplgl
   USE memst2
   USE pacchr
   USE pacint
   USE paclgl
   USE systm
   USE termnl

   IMPLICIT NONE

   !  Interface block
   !  ---------------

   INTERFACE
      SUBROUTINE avectx( mot, longm, long, ideb, iend, kode, nomv, lnom, &
                         incr, minx, maxx, imin, imax, icol, jcol, lerr, &
                         dx, rx, ix, lx, cx )
         INTEGER, INTENT(IN)  :: longm, long, ideb, iend, kode, incr, minx, maxx
         LOGICAL, INTENT(IN)  :: lnom
         CHARACTER(LEN=*), INTENT(IN) :: mot
         CHARACTER(LEN=*), DIMENSION(minx:*), INTENT(IN) :: nomv
         INTEGER, INTENT(OUT) :: imin, imax, icol, jcol
         LOGICAL, INTENT(OUT) :: lerr
         REAL(KIND=8)    , DIMENSION(minx:maxx), INTENT(OUT), OPTIONAL :: dx
         REAL            , DIMENSION(minx:maxx), INTENT(OUT), OPTIONAL :: rx
         INTEGER         , DIMENSION(minx:maxx), INTENT(OUT), OPTIONAL :: ix
         LOGICAL         , DIMENSION(minx:maxx), INTENT(OUT), OPTIONAL :: lx
         CHARACTER(LEN=*), DIMENSION(minx:maxx), INTENT(OUT), OPTIONAL :: cx
      END SUBROUTINE avectx

      SUBROUTINE chkunt( lerr,iout,iunt,u1,u2,u3,u4,u5,u6,u7,u8,u9,u10,u11,u12 )
         IMPLICIT NONE
         INTEGER, INTENT(IN) :: iout, iunt
         INTEGER, INTENT(IN), OPTIONAL :: u1, u2, u3, u4, u5, u6, u7, u8, u9, u10, u11, u12
         LOGICAL, INTENT(OUT) :: lerr
      END SUBROUTINE chkunt

      SUBROUTINE pac999( u1, u2, u3 )
         IMPLICIT NONE
         INTEGER, INTENT(IN), OPTIONAL :: u1, u2, u3
      END SUBROUTINE pac999
   END INTERFACE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: k1, maxou, maxrc
   REAL(KIND=8), INTENT(INOUT) :: baylev
   INTEGER, INTENT(INOUT) :: kn, noutlv
   LOGICAL, INTENT(INOUT) :: lform
   REAL(KIND=8), INTENT(INOUT) :: rcoef(maxrc)
   CHARACTER(LEN=*), INTENT(INOUT) :: outlev(maxou)
   INTEGER, INTENT(OUT) :: ndf1, nrco
   LOGICAL, INTENT(OUT) :: lerr

   !  Local declarations
   !  ------------------

   INTEGER, EXTERNAL :: iscalx
   LOGICAL, EXTERNAL :: lscalx

   REAL(KIND=8), PARAMETER :: zero=0.0_8, one=1.0_8
   INTEGER :: i, icof, icol, icov, icow, imax, imin, irep, jcol
   INTEGER :: ngv, nkar, nladd, nlocg, nloct, nlocu, ntw, numd, status
   LOGICAL :: lexis, lnamd, lopen, lnewd, lrwnd, newp
   CHARACTER(LEN=1) :: reptmp, septmp

   INCLUDE 'pacin1p.'
   INCLUDE 'paccomp.'

   ! nmotc - number of keywords in this paragraph
   ! motc  - the keywords (possibly preceded by sep2 character) of the
   !         paragraph, in chronological order of their analysis
   ! lngm(.,1) - full length of elements of motc
   ! lngm(.,2) - length of maximum abreviated form of elements of motc
   ! alias - number of alias keywords (used when the beginning of
   !         some keywords may interfere with the paragrah name:
   !         e.g. the sequence $COMP ANOVA; will be interpreted as
   !         $COMPA NOVA; and therefore transformed as .....;NOVA;
   !         thus NOVA must be taken as an alias for ANOVA for the
   !         request to be correctly analyzed)
   !         These aliases should take place after ordinary keywords

   !--------------------------------------------------------------------

   lerr = .false.

   cmptit = ' '
   call cscalx( motc(1), lngm(1,1), lngm(1,2), k1, kn, cmptit, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(1:), nmotc )
      lderiv = .false.
      return
   end if

      ! See if derived protocol is to be saved
      ! (save file is prepared at end of routine, if needed)
   ideriv = iscalx( motc(2), lngm(2,1), lngm(2,2), k1, kn, ideriv, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(2:), nmotc-1 )
      lderiv = .false.
      return
   end if
   lderiv = icol > 0 .and. ideriv > 0

   call cscalx( motc(3), lngm(3,1), lngm(3,2), k1, kn, derfil, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(3:), nmotc-2 )
      lderiv = .false.
      return
   end if
   derfil = adjustl( derfil )
      ! Convert file name to capitals if not unix system
   if ( .not. lunix .and. icol > 0 ) call cvuct( derfil )
   lderiv = lderiv .or. icol > 0

   call cscalx( motc(4), lngm(4,1), lngm(4,2), k1, kn, derfmt, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(4:), nmotc-3 )
      lderiv = .false.
      return
   end if
      ! Convert format to capitals
   if ( icol > 0 ) call cvuct( derfmt )

   lrwnd = lscalx( motc(5), lngm(5,1), lngm(5,2), k1, kn, .false., icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(5:), nmotc-4 )
      lderiv = .false.
      return
   end if

      ! Enter formulas
      ! Note that the list-item-separator and the repetition symbol
      ! are modified in order to allow use of comma and asterik in the
      ! formulas without inbedding them into quotes
   septmp = sep1
   reptmp = repet
   sep1   = '?'
   repet  = '%'
   call avectx( motc(6), lngm(6,1), lngm(6,2), k1, kn, 5,           &
                cnul, .false., 1, 1, maxnf, imin, imax, icof, jcol, &
                lerr, cx=cmpfor )
      ! Restaure original list-item-separator and repetition symbol
   sep1  = septmp
   repet = reptmp
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(6:), nmotc-5 )
      lderiv = .false.
      return
   end if

   call cscalx( motc(7), lngm(7,1), lngm(7,2), k1, kn, cmpgwo, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(7:), nmotc-6 )
      lderiv = .false.
      return
   end if
   if ( icol > 0 ) then
         ! Set code for group weights (default set in pacin1)
      call cvuct( cmpgwo )
      if ( cmpgwo(:3) == typwg2 ) then
         gpond = 2      ! Size weights
      else
         gpond = 1      ! Equal weights
         if ( cmpgwo(:3) /= typwg1 ) then
            call chkpg( 3, .false., newp )
            write(imp,222) cmpgwo(:16)
            lines = lines + 3
            write(lgu,*) mrker1//mrker1
            write(lgu,222) cmpgwo(:16)
            write(lgu,*) mrker2//mrker2
         end if
      end if
   end if

   call cscalx( motc(8), lngm(8,1), lngm(8,2), k1, kn, cmptwo, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(8:), nmotc-7 )
      lderiv = .false.
      return
   end if
   if ( icol > 0 ) then
         ! Set code for treatment weights (default set in pacin1)
      call cvuct( cmptwo )
      if ( cmptwo(:3) == typwg2 ) then
         tpond = 2      ! Size weights
      else
         tpond = 1      ! Equal weights
         if ( cmptwo(:3) /= typwg1 ) then
            call chkpg( 3, .false., newp )
            write(imp,222) cmptwo(:16)
            lines = lines + 3
            write(lgu,*) mrker1//mrker1
            write(lgu,222) cmptwo(:16)
            write(lgu,*) mrker2//mrker2
         end if
      end if
   end if

   if ( icof > 0 ) then
      ! Entry by formula
      ! ----------------
      lform = .true.

   else
      ! Maybe it is an entry by contrast-vectors
      ! ----------------------------------------
      if ( lform ) then
            ! Check vectors are given
         call local( motc(12), lngm(12,2), k1, kn, .false., 1, nlocg, inul )
         call local( motc(13), lngm(13,2), k1, kn, .false., 1, nloct, inul )
         call local( motc(15), lngm(15,2), k1, kn, .false., 1, nlocu, inul )
         lform = nlocg+nloct+nlocu <= 0
      end if
      if ( .not. lform ) then
            ! Yes, it is an entry by contrast-vectors

         lupvw = lscalx( motc(9), lngm(9,1), lngm(9,2), k1, kn, .true., icol, jcol, lerr )
         if ( lerr ) then
            call gmpwrn( pargr, linter, motc(9:), nmotc-8 )
            lderiv = .false.
            return
         end if

         if ( lupvw ) then   ! The comparison is product form

            ncv = iscalx( motc(10), lngm(10,1), lngm(10,2), k1, kn, 1, icol, jcol, lerr )
            if ( lerr ) then
               call gmpwrn( pargr, linter,motc(10:), nmotc-9 )
               lderiv = .false.
               return
            end if

            ncw = iscalx( motc(11), lngm(11,1), lngm(11,2), k1, kn, 1, icol, jcol, lerr )
            if ( lerr ) then
               call gmpwrn( pargr, linter, motc(11:), nmotc-10 )
               lderiv = .false.
               return
            end if

            ncv  = max( 1, ncv )
            ncw  = max( 1, ncw )
            ndf1 = ncv * ncw

               ! Update sizes of arrays
            ngv  = ngrp * ncv
            ntw  = ntrt * ncw
            nszu = ncw

               ! Allocate and initialize v, w
            if ( allocated(v) ) deallocate(v)
            if ( allocated(w) ) deallocate(w)
            allocate ( v(ngrp,ncv), w(ntrt,ncw), stat=status )
            if ( status /= 0 ) then   ! fatal error
               write(lgu,*) mrker1
               write(lgu,101) " (PacCom)."
               write(lgu,*) mrker2
               call gmp999            ! release memory space and files
               call pac999( 99 )
               stop
            end if
            v = zero
            if ( ngrp == 1 ) v(1,1) = one
            w = zero
            if ( ntrt == 1 ) w(1,1) = one
               ! Analysis of v
            call avectx( motc(12), lngm(12,1), lngm(12,2), k1, kn, 1, &
                         cnul, .false., 1,                            &
                         1, ngv, imin, imax, icov, jcol,              &
                         lerr, dx=v )
            if ( lerr ) then
               call gmpwrn( pargr, linter, motc(12:), nmotc-11 )
               lderiv = .false.
               return
            end if
               ! Analysis of w
            call avectx( motc(13), lngm(13,1), lngm(13,2), k1, kn, 1, &
                         cnul, .false., 1,                            &
                         1, ntw, imin, imax, icow, jcol,              &
                         lerr, dx=w )
            if ( lerr ) then
               call gmpwrn( pargr, linter, motc(13:), nmotc-12 )
               lderiv = .false.
               return
            end if
            if ( ngrp > 1 .and. icov <= 0 ) then
               write(lgu,*) mrker1
               write(lgu,333) msggrp
               write(lgu,*) mrker2
               lerr   = .true.
               lderiv = .false.
               return
            end if
            if ( ntrt > 1 .and. icow <= 0 ) then
               write(lgu,*) mrker1
               write(lgu,333) msgocc
               write(lgu,*) mrker2
               lerr   = .true.
               lderiv = .false.
               return
            end if

         else                ! The comparison is non-product form

            ndf1 = iscalx( motc(14), lngm(14,1), lngm(14,2), k1, kn, 1, icol, jcol, lerr )
            if ( lerr ) then
               call gmpwrn( pargr, linter,motc(14:), nmotc-13 )
               lderiv = .false.
               return
            end if
            ndf1 = max( 1, ndf1 )
!!?????????????????  a verifier:
            ncv  = ndf1
            ncw  = ndf1
            nszu = ndf1 * ngrp

               ! Allocate and initialize w
            if ( allocated(w) ) deallocate(w)
            allocate ( w(ntrt,nszu), stat=status )
            if ( status /= 0 ) then   ! Fatal error
               write(lgu,*) mrker1
               write(lgu,101) " (PacCom)."
               write(lgu,*) mrker2
               call gmp999            ! Release memory space and files
               call pac999( 99 )
               stop
            end if
            if ( ntrt*nszu == 1 ) then
               w(1,1) = one
            else
               w = zero
            end if
               ! Analysis of w
               ! (Note that treatments vary faster, then degrees of
               ! freedom and then groups)
            call avectx( motc(15), lngm(15,1), lngm(15,2), k1, kn, 1, &
                         cnul, .false., 1,                            &
                         1, ntrt*nszu, imin, imax, icol,jcol,         &
                         lerr, dx=w )
            if ( lerr ) then
               call gmpwrn( pargr, linter, motc(15:), nmotc-14 )
               lderiv = .false.
               return
            end if
            if ( icol <= 0 ) then
               write(lgu,*) mrker1
               write(lgu,444)
               write(lgu,*) mrker2
               lerr   = .true.
               lderiv = .false.
               return
            end if
         end if
      end if

   end if

      ! Analysis of baylev (bayes-fiducial probability)
   call avectx( motc(16), lngm(16,1), lngm(16,2), k1, kn, 1,    &
                cnul, .false., 1, 1, 1, imin, imax, icol, jcol, &
                lerr, dx=dnul )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(16:), nmotc-15 )
      lderiv = .false.
      return
   end if
   if ( icol > 0 ) baylev = dnul(1)

      ! Analysis of output level for descriptive statistics
   call avectx( motc(17), lngm(17,1), lngm(17,2), k1, kn, 5,        &
                cnul, .false., 1, 1, maxou, imin, imax, icol, jcol, &
                lerr, cx=outlev )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(17:), nmotc-16 )
      lderiv = .false.
      return
   end if
   if ( icol > 0 ) noutlv = imax - imin + 1

      ! Analysis of rcoef
   call avectx( motc(18), lngm(18,1), lngm(18,2), k1, kn, 1,        &
                cnul, .false., 1, 1, maxrc, imin, imax, icol, jcol, &
                lerr, dx=rcoef )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(18:), nmotc-17 )
      lderiv = .false.
      return
   end if
   nrco = maxrc
   if ( icol > 0 ) nrco = imax - imin + 1

      ! Analysis of hyp0
   call avectx( motc(19), lngm(19,1), lngm(19,2), k1, kn, 1,        &
                cnul, .false., 1, 1, nvtot, imin, imax, icol, jcol, &
                lerr, dx=hyp0 )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(19:), nmotc-18 )
      lderiv = .false.
      return
   end if

      ! Analysis of request to cancel previous limit-request
   limit = lscalx( motc(20), lngm(20,1), lngm(20,2), k1, kn, limit, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(20:), nmotc-19 )
      lderiv = .false.
      return
   end if
   if ( icol > 0 ) limit = .not. limit

      ! Analysis of xlim
   call avectx( motc(21), lngm(21,1), lngm(21,2), k1, kn, 1,        &
                cnul, .false., 1, 1, nvtot, imin, imax, icol, jcol, &
                lerr, dx=xlim )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(21:), nmotc-20 )
      lderiv = .false.
      return
   end if
   if ( icol > 0 ) limit = .true.

      ! Analysis of lanova (request for a standard anova table)
   lanova = lscalx( motc(22), lngm(22,1), lngm(22,2), k1, kn, lanova, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(22:), nmotc-21 )
      lderiv = .false.
      return
   end if
   if ( icol <= 0 ) then
         ! Search for the alias of preceding keyword
      lanova = lscalx( motc(nmotc+1),lngm(nmotc+1,1),lngm(nmotc+1,2), k1, kn, lanova, icol, jcol, lerr )
      if ( lerr ) then
         call gmpwrn( pargr, linter, motc(22:), nmotc-21 )
         lderiv = .false.
         return
      end if
   end if

      ! Analysis of luniv (request for multivariate procedures)
   luniv = lscalx( motc(23), lngm(23,1), lngm(23,2), k1, kn, luniv, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(23:), nmotc-22 )
      lderiv = .false.
      return
   end if

      ! Analysis of lmulti (request for multivariate procedures)
   lmulti = lscalx( motc(24), lngm(24,1), lngm(24,2), k1, kn, lmulti, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(24:), nmotc-23 )
      lderiv = .false.
      return
   end if

      ! Analysis of lreduc (request for "data reduction" analysis)
   lreduc = lscalx( motc(25), lngm(25,1), lngm(25,2), k1, kn, lreduc, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(25:), nmotc-24 )
      lderiv = .false.
      return
   end if

      ! Analysis of lpmean (request for printing means compared)
   lpmean = lscalx( motc(26), lngm(26,1), lngm(26,2), k1, kn, lpmean, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(26:), nmotc-25 )
      lderiv = .false.
      return
   end if

      ! Analysis of lptabl (request for printing the partition table)
   lptabl = lscalx( motc(27), lngm(27,1), lngm(27,2), k1, kn, lptabl, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(27:), nmotc-26 )
      lderiv = .false.
      return
   end if

      ! Analysis of listdp (request for derived protocol listing)
   listdp = lscalx( motc(28), lngm(28,1), lngm(28,2), k1, kn, listdp, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(28:), nmotc-27 )
      lderiv = .false.
      return
   end if

      ! Analysis of lspecf (request for specific approach)
   lspecf = lscalx( motc(29), lngm(29,1), lngm(29,2), k1, kn, lspecf, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(29:), nmotc-28 )
      lderiv = .false.
      return
   end if

      ! Analysis of lfrapp (request for frequentist approach)
   lfrapp = lscalx( motc(30), lngm(30,1), lngm(30,2), k1, kn, lfrapp, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(30:), nmotc-29 )
      lderiv = .false.
      return
   end if

   lbayes = lscalx( motc(31), lngm(31,1), lngm(31,2), k1, kn, lbayes, icol, jcol, lerr )
   if ( lerr ) then
      call gmpwrn( pargr, linter, motc(31:), nmotc-30 )
      lderiv = .false.
      return
   end if
   if ( ibayes < 0 ) lbayes = .false.

   ! Commands filtering
   ! ------------------
   nkar = ncar
   call cmprss( k1, kn-1 )
   kn = kn - nkar + ncar
   if ( kn > k1 ) then
      nladd = 3 + (kn-(k1+1))/(lrecp-1)
      call chkpg( nladd, .false., newp )
      write(imp,111) (car(i:min(i+lrecp-2,kn)),i=k1+1,kn,lrecp-1)
      lines = lines + nladd
      write(lgu,*) mrker1//mrker1
      write(lgu,111) (car(i:min(i+lrecp-2,kn)),i=k1+1,kn,lrecp-1)
      write(lgu,*) mrker2//mrker2
      car(k1:kn-1) = ' '
      call cmprss( k1, kn-1 )
   end if


   ! Prepare deriv file
   ! ------------------
   kn = max( 1, len_trim(derfil) )
   lnewd = lderiv
   if ( ideriv > 0 ) then

         ! Check unit number is correct
      call chkunt( lerr, imp, ideriv, iu1, iu2, iu3, iu4,     &
                   imp, lec, lec1, lec2, inpux, isave, ibayes )
      if ( lerr ) then
         lderiv = .false.
         return
      end if
      if ( derfil(1:1) == ' ' ) then
            ! Check the unit is connected
         inquire (unit=ideriv,exist=lexis,opened=lopen,named=lnamd)
         lnewd = .not. (lopen .and. lexis)
         if ( lopen .and. lexis .and. lnamd ) then
            close (ideriv)
            lnewd = .true.
         end if
      else
            ! Check the file already exists
         inquire (file=derfil,exist=lexis,opened=lopen,number=numd)
         lnewd = .not. (lopen .and. lexis)
         if ( lopen .and. lexis .and. numd /= ideriv ) then
!!          call chkpg( 5, .false., newp )
            write(lgu,*) mrker1
            write(lgu,555) derfil(:min(lrecp-1,kn)), numd, ideriv
            write(lgu,*) mrker2
!!          lines  = lines + 5
            lerr   = .true.
            lderiv = .false.
            return
         end if
      end if
      if ( lnewd ) then
         call filopn( 301, ideriv, derfil, 'UNKNOWN', 'FORMATTED', .false., lerr )
         if ( lerr ) then
            lderiv = .false.
            return
         end if
      end if

   else if ( derfil(1:1) /= ' ' ) then

         ! Use default save unit
      ideriv = kderiv
         ! Check the file already exists
      inquire(file=derfil,exist=lexis,opened=lopen,number=numd)
      lnewd = .not. (lopen .and. lexis)
      if ( lopen .and. lexis .and. numd /= ideriv ) ideriv = numd
      if ( lnewd ) then
         call filopn( 301, ideriv, derfil, 'UNKNOWN', 'FORMATTED', .false., lerr )
         if ( lerr ) then
            lderiv = .false.
            return
         end if
      end if

   end if

   lrwdd = lderiv .and. (lnewd .or. lrwnd)

   ! Rewind and print message about derive save file if used
   ! -------------------------------------------------------
   if ( lderiv ) then
      if ( lrwdd ) then
         rewind ideriv
         irep = 1
      else
         irep = 0
      end if
      jcol  = max( len_trim(derfmt), 1 )
      nladd = (kn-1)/lrecp + 1 + (jcol-1)/lrecp + 1
      call chkpg( 5+nladd, .false., newp )
      write(imp,666) ideriv, rep(irep)
      lines = lines + 3
      if ( derfil(:1) /= ' ' ) then
         write(imp,777) (derfil(i:min(i+lrecp-2,kn)),i=1,kn,lrecp-1)
         write(imp,888) (derfmt(i:min(i+lrecp-2,jcol)),i=1,jcol,lrecp-1)
         lines = lines + 2 + nladd
      end if
   end if

   INCLUDE 'allocf.'
   INCLUDE 'gmpf111.'
   INCLUDE 'paccomf.'

END SUBROUTINE paccom
