SUBROUTINE pacdpc( ius, jus, gnum, gsup, wess, nred, nsuj, nssav )

!***********************************************************************
!     - pacdpc -
!     computes and saves the "reduced" relevant Derived Protocol (i.e.
!     when discriminant analysis)
!***********************************************************************
!
!     IUS   - INPUT . Unit number for data reading            - INTEGER
!     JUS   - INPUT . Unit number for temporary storage of    - INTEGER
!                     derived protocol (only used if no user's
!                     save file)
!     GNUM  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     GSUP  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     WESS  - INPUT . The matrix of "essential" contrasts     - REAL*8
!                     with NTDC rows and NRED columns
!     NRED  - INPUT . Length of Y                             - INTEGER
!     NSUJ  - INPUT . Number of cases on input                - INTEGER
!     NSSAV - OUTPUT. Number of cases saved                   - INTEGER
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------

   USE gmpint
   USE pacchr
   USE pacint
   USE paclgl

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: ius, jus, nred, nsuj
   REAL(KIND=8), INTENT(IN) :: wess(ntdc,nred)
   INTEGER, INTENT(IN) :: gnum(ngrp), gsup(ngrp)
   INTEGER, INTENT(OUT) :: nssav

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   REAL(KIND=8) :: frq, rk
   INTEGER :: j, k, kase, ko, num
   CHARACTER(LEN=incdw) :: slab
   REAL(KIND=8) :: xg(ngfac), x(ntdc), y(nred)
                 ! xg - work vector for grouping variables of case read
                 ! x  - work vector for data of case read
                 ! y  - work vector for the derived reduced data

   !--------------------------------------------------------------------

   rewind ius
   if ( .not. lderiv ) rewind jus
   nssav = 0

   do kase = 1, nsuj
         ! num is sequential case-number in original input data file
         ! frq is case frequency
         ! ko  is group-number according to appearance order in data
         ! k   is group-number according to "actual group-table"
         ! Skip the case if it doesn't belong to a g-support group
      read(ius) num, slab, frq, ko, xg, x
      k = gnum(ko)
      if ( gsup(k) <= 0 ) cycle

      do j = 1, nred
         y(j) = sum( wess(:,j)*x )
      end do

         ! N.B. Correct weight not saved presently, replaced by 0.
      if ( lderiv ) then   ! Save on user's save-file
         rk = k
         write(ideriv,derfmt) slab, frq, zero, rk, xg, y
      else                 ! Temporary storage
         write(jus) slab, frq, zero, k, xg, y
      end if
      nssav = nssav + 1

   end do

END SUBROUTINE pacdpc
