SUBROUTINE paclcf( formul, last, loop, deli, ndel, expf, nexpf, lerr )

!***********************************************************************
!     - paclcf -
!     expands Loop(s) in a Comparison Formula.
!     Only loops on union and comparison operators are permitted.
!     A loop is indicated by doubling the operator.
!***********************************************************************
!
!     FORMUL- INPUT . Formula to be processed                 - CHAR
!     LAST  - INPUT . Length of that formula                  - INTEGER
!     LOOP  - INPUT . Array of 2-byte loop symbols            - CHAR
!                     LOOP(1) : "union loop" symbol
!                     LOOP(2) : "comparison loop" symbol
!     DELI  - INPUT . Array of 1-byte delimitors              - CHAR
!                     dimensioned max(NDEL(1),NDEL(2))
!     NDEL  - INPUT . Array of numbers of delimitors:         - INTEGER
!                     NDEL(1) : for "union loop"
!                     NDEL(2) : for "comparison loop"
!     EXPF  - OUTPUT. Expanded formula (a mere copy of the    - CHAR
!                     input formula if no loop)
!     NEXPF - OUTPUT. Length of expanded formula              - INTEGER
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     carl   : car
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  INUM
!     External functions called:
!        NONE
!     Fortran  functions called:
!        IACHAR  INDEX  LEN  MIN  SIGN
!
!-----------------------------------------------------------------------

   USE is_digit
   USE carl
   USE gmpint

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: last
   CHARACTER(LEN=*), INTENT(IN) :: formul
   INTEGER, INTENT(IN) :: ndel(2)
   CHARACTER(LEN=2), INTENT(IN) :: loop(2)
   CHARACTER(LEN=1), INTENT(IN) :: deli(*)
   INTEGER, INTENT(OUT) :: nexpf
   LOGICAL, INTENT(OUT) :: lerr
   CHARACTER(LEN=*), INTENT(OUT) :: expf

   !  Local declarations
   !  ------------------

   LOGICAL, EXTERNAL :: samstr

   INTEGER :: i, ic1, ik1, j, ja, jb, jc, k, kdif
   INTEGER :: lastc, la1, la2, lb1, lb2, mexpf
   LOGICAL :: error, lgl, newp
   CHARACTER(LEN=8) :: num

   INCLUDE 'pacdegp.'

   !--------------------------------------------------------------------

   lerr = .true.
   mexpf = min( len(expf), incar )
   if ( last > mexpf ) then
!!    call chkpg( 3, .false., newp )
      write(lgu,*) mrker1
      write(lgu,111)
      write(lgu,*) mrker2
!!    lines = lines + 3
      return
   end if

      ! The progressively expanded formula is stored into car with
      ! the substring not yet analyzed appended to it. This permits
      ! to treat the case of an ending term of loop being the first
      ! term of an immediately following loop.
      ! ik1 codes a location in formul
      ! ic1 codes a location in car
      ! kdif is the difference between locations in car and formul of
      ! the substring not yet analyzed.
   expf  = ' '
   nexpf = 0
   lastc = last
   ik1   = 1
   ic1   = 1
   kdif  = 0

   do

      car(ic1:lastc) = formul(ik1:last)
      ncar = lastc

         ! Any loop ?
         ! Note: A (true) loop operator cannot begin before location 3;
         ! e.g. a minimal string with loop would be of the type: a1,,5

      j = index( car(ic1:lastc), loop(1) )
      k = index( car(ic1:lastc), loop(2) )

      if ( j+k <= 0 ) then

         expf(nexpf+1:lastc) = car(ic1:lastc)
         nexpf = lastc

      else
            ! i codes for the operator in the loop
            ! k is the location of the 1st character of the loop symbol
         i = 2
         if ( j > 0 .and. (k <= 0 .or. j < k) ) then
            i = 1
            k = j
         end if
         k = k + ic1 - 1
         if ( k <= 1 ) then
!!          call chkpg( 4, .false., newp )
            write(lgu,*) mrker1
            write(lgu,222)
            write(lgu,333)
            write(lgu,*) car(1:min(lastc,76))
            write(lgu,*) mrker2
!!          lines = lines + 4
            return
         end if
            ! Store substring up to 1st term of loop (if not yet done)
         if ( nexpf < k-1 ) then
            nexpf = k - 1
            expf(ic1:nexpf) = car(ic1:nexpf)
         end if
            ! Get starting index (ja)
         call inum( nexpf, 1, -1, ' ', 1, ' ', 1, .false., la2, lgl, ja, error )
         if ( error ) return
         if ( la2 <= 0 ) then
!!          call chkpg( 4, .false., newp )
            write(lgu,*) mrker1
            write(lgu,222)
            write(lgu,444)
            write(lgu,*) car(1:min(lastc,76))
            write(lgu,*) mrker2
!!          lines = lines + 4
            return
         end if
            ! Look for delimitor
            ! la1 is starting location of character string to be
            ! expanded, and la2 is its ending location
            ! (delimitor is before location ic1 if 1st term of loop
            ! is the last term of an immediately preceding loop)
         la1 = 0
         do j = 1, ndel(i)
            la1 = max( la1, index( car(1:la2), deli(j), back=.true. ) )
         end do
         if ( la1 >= la2 ) then
!!          call chkpg( 4, .false., newp )
            write(lgu,*) mrker1
            write(lgu,222)
            write(lgu,555)
            write(lgu,*) car(la1:min(lastc,la1+75))
            write(lgu,*) mrker2
!!          lines = lines + 4
            return
         end if
         la1 = la1 + 1
            ! Scan for regression keywords
         if ( car(la1:la1) == '-' ) la1 = la1 + 1
         if ( la1+2 < la2 ) then
            if ( samstr( car(la1:la1+2), kwhel ) .or.    &
                 samstr( car(la1:la1+2), kwint ) .or.    &
                 samstr( car(la1:la1+2), kwpro ) .or.    &
                 samstr( car(la1:la1+2), alias(0) ) .or. &
                 samstr( car(la1:la1+2), alias(1) ) .or. &
                 samstr( car(la1:la1+2), alias(2) ) .or. &
                 samstr( car(la1:la1+2), alias(3) ) ) then
               la1 = la1 + 3
            else if ( samstr( car(la1:la1+2), kwgen(1) ) .or.  &
                       samstr( car(la1:la1+2), kwgen(2) ) ) then
               la1 = la1 + 3
               do
                  j = iachar( car(la1:la1) )
                  if ( ldigit(j) ) then
                     la1 = la1 + 1
                     if ( la1 < la2 ) cycle
                  end if
                  exit
               end do
            end if
         end if
         if ( la1 > la2 ) then
!!          call chkpg( 4, .false., newp )
            write(lgu,*) mrker1
            write(lgu,222)
            write(lgu,555)
            write(lgu,*) car(la1:min(lastc,la1+75))
            write(lgu,*) mrker2
!!          lines = lines + 4
            return
         end if
            ! Get ending index (jb)
         call inum( k+2, lastc, 1, ' ', 1, ' ', 1, .false., ic1, lgl, jb, error )
         if ( error ) return
            ! Get loop increment (jc)
         jc = sign( 1, jb-ja )
         if ( ic1+ncby <= lastc ) then
            if ( car(ic1:ic1+ncby-1) == by ) then
               call inum( ic1+1, lastc, 1, ' ', 1, ' ', 1, .false., ic1, lgl, jc, error )
               if ( error ) return
               if ( jc == 0 ) then
!!                call chkpg( 4, .false., newp )
                  write(lgu,*) mrker1
                  write(lgu,222)
                  write(lgu,666)
                  write(lgu,*) car(la1:min(ic1,lastc,la1+75))
                  write(lgu,*) mrker2
!!                lines = lines + 4
                  return
               end if
               jc = sign( jc, jb-ja )
            end if
         end if
            ! Expand
         do j = ja+jc, jb, jc
            call karin( j, num, lb1 )
            lb2 = nexpf + la2 - la1 + 2 + lb1
            if ( lb2 > mexpf ) then
!!             call chkpg( 3, .false., newp )
               write(lgu,*) mrker1
               write(lgu,222)
               write(lgu,777)
               write(lgu,*) mrker2
!!             lines = lines + 3
               return
            end if
            expf(nexpf+1:lb2) = car(k:k) // car(la1:la2) // num
            nexpf = lb2
         end do
         ik1 = ic1 - kdif
         if ( ik1 < last ) then
            lastc = lastc + nexpf + 1 - ic1
            if ( lastc > mexpf ) then
!!             call chkpg( 3, .false., newp )
               write(lgu,*) mrker1
               write(lgu,222)
               write(lgu,777)
               write(lgu,*) mrker2
!!             lines = lines + 3
               return
            end if
            ic1  = nexpf + 1
            kdif = ic1 - ik1
            car(k:nexpf) = expf(k:nexpf)
            cycle
         end if

      end if

      exit
   end do

      ! Normal return
   lerr = .false.

   INCLUDE 'paclcff.'

END SUBROUTINE paclcf
