SUBROUTINE paclid( iulid, lidfil, gsym, tsym, nomv, gnum, gmod, tmod, gstr, tstr )

!***********************************************************************
!     - paclid -
!     creates an Eyelid save file corresponding to current design
!***********************************************************************
!
!     IULID - INPUT . Unit number of the EYELID save file     - INTEGER
!             OUTPUT.
!     LIDFIL- INPUT . Name of the EYELID save file            - CHAR
!     GSYM  - INPUT . Array of symbols for G-factors          - CHAR
!     TSYM  - INPUT . Array of symbols for T-factors          - CHAR
!     NOMV  - INPUT . Names of variables                      - CHAR
!     GNUM  - INPUT .    See definition of MEM2 sub-arrays    - INTEGER
!     GMOD  - INPUT .    See definition of MEM2 sub-arrays    - INTEGER
!     TMOD  - INPUT .    See definition of MEM2 sub-arrays    - INTEGER
!     GSTR  - INPUT .    See definition of MEM2 sub-arrays    - INTEGER
!     TSTR  - INPUT .    See definition of MEM2 sub-arrays    - INTEGER
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  CHKUNT  FILOPN
!     External functions called:
!        NONE
!     Fortran  functions called:
!        LEN_TRIM  MIN  NINT  REAL
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE gmplgl
   USE pacint
   USE termnl

   IMPLICIT NONE

   !  Interface block
   !  ---------------

   INTERFACE
      SUBROUTINE chkunt( lerr,iout,iunt,u1,u2,u3,u4,u5,u6,u7,u8,u9,u10,u11,u12 )
         IMPLICIT NONE
         INTEGER, INTENT(IN) :: iout, iunt
         INTEGER, INTENT(IN), OPTIONAL :: u1, u2, u3, u4, u5, u6, u7, u8, u9, u10, u11, u12
         LOGICAL, INTENT(OUT) :: lerr
      END SUBROUTINE chkunt
   END INTERFACE

   !  Arguments
   !  ---------

   CHARACTER(LEN=*), INTENT(IN) :: lidfil
   INTEGER, INTENT(IN) :: gnum(ngrp), gmod(ngfac), tmod(ntfac)
   INTEGER, INTENT(IN) :: gstr(ngrp,ngfac), tstr(ntrt,ntfac)
   CHARACTER(LEN=*), INTENT(IN) :: nomv(nvtot)
   INTEGER, INTENT(INOUT) :: iulid
   CHARACTER(LEN=*), INTENT(INOUT) :: gsym(ngfac), tsym(ntfac)

   !  Local declarations
   !  ------------------

   REAL(KIND=8) :: frqcy
   INTEGER :: i, ip, iv, j, k, kase, ko, lenf, nu, num, numl
   LOGICAL :: lerr, lexis, lopen, lnamd, newp
   CHARACTER(LEN=incdw) :: slab
   CHARACTER(LEN=1) :: g1sym, t1sym
   REAL(KIND=8) :: xg(ngfac), x(ntdc)

   !--------------------------------------------------------------------

   lenf = len_trim( lidfil )
   lenf = min( lenf, 32 )

   ! Prepare Eyelid save file
   ! ------------------------
   if ( iulid >= 0 ) then

         ! Check unit number is correct
      call chkunt( lerr, imp, iulid, iu1, iu2, iu3, iu4,      &
                   imp, lec, lec1, lec2, inpux, isave, ibayes )
      if ( lerr ) return
      if ( lidfil(1:1) == ' ' ) then
            ! Check unit is connected
         inquire (unit=iulid,exist=lexis,opened=lopen,named=lnamd)
         if ( lopen .and. lexis ) then
            close (iulid)
            call chkpg( 3, .false., newp )
            write(imp,222) ' ', iulid
            lines = lines + 3
            write(lgu,*) ermrk1//ermrk1
            write(lgu,222) ' ', iulid
            write(lgu,*) ermrk2//ermrk2
         end if
      else
            ! Check file already exists
         inquire (file=lidfil,exist=lexis,opened=lopen,number=numl)
         if ( lopen .and. lexis ) then
            call chkpg( 3, .false., newp )
            write(imp,222) lidfil(:lenf), iulid
            lines = lines + 3
            write(lgu,*) ermrk1//ermrk1
            write(lgu,222) lidfil(:lenf), iulid
            write(lgu,*) ermrk2//ermrk2
         end if
      end if
      call filopn( 201, iulid, lidfil, 'UNKNOWN', 'FORMATTED', .false., lerr )
      if ( lerr ) return

   else

      if ( lidfil(1:1) /= ' ' ) then
            ! Use default save unit (99)
         iulid = 99
            ! Check file already exists
         inquire (file=lidfil,exist=lexis,opened=lopen,number=numl)
         if ( lopen .and. lexis ) then
            iulid = numl
            call chkpg( 3, .false., newp )
            write(imp,222) lidfil(:lenf), iulid
            lines = lines + 3
            write(lgu,*) ermrk1//ermrk1
            write(lgu,222) lidfil(:lenf), iulid
            write(lgu,*) ermrk2//ermrk2
         end if
         call filopn( 201, iulid, lidfil, 'UNKNOWN', 'FORMATTED', .false., lerr )
         if ( lerr ) return
      else
         call chkpg( 3, .false., newp )
         write(imp,111)
         lines = lines + 3
         write(lgu,*) ermrk1//ermrk1
         write(lgu,111)
         write(lgu,*) ermrk2//ermrk2
         return
      end if

   end if

   ! Create Eyelid save file
   ! -----------------------
   rewind iulid
   rewind iu2

      ! If no G or T factors, get a (temporary) symbol
   if ( ngfa0 == 0 ) then
      g1sym   = gsym(1)
      gsym(1) = 'G'
   end if
   if ( ntrt == 1 ) then
      t1sym   = tsym(1)
      tsym(1) = 'O'
   end if
      ! Create Eyelid dictionnary with 2 additional factors:
      ! factor 'S' corresponding to subjects,
      ! factor 'Z' corresponding to a constant factor,
      ! Note that no check is made for previous use of letters S, Z
      ! (and G, T possibly) for other factors
   nu = nindvu * ntrt
   write(iulid,444) titre(:72)
   write(iulid,*)   nu, 2+ngfac+ntfac, nvtot
   write(iulid,444) nomv
   write(iulid,555) 'S', nindvu,                 &
                    (gsym(i),gmod(i),i=1,ngfac), &
                    (tsym(i),tmod(i),i=1,ntfac), &
                    'Z', 1
      ! Restore factor symbols if altered
   if ( ngfa0 == 0 ) gsym(1) = g1sym
   if ( ntrt == 1 ) tsym(1) = t1sym

      ! Save data
      ! Note that constant 1 is added as last info., corresponding to
      ! constant factor 'Z' added
   iv = ntrt * (nvtot-1)
   do kase = 1, nindvu
      read(iu2) num, slab, frqcy, ko, xg, x
         ! num   is sequential case number in original data file
         ! slab  is case label
         ! frqcy is case frequency
         ! ko    is group number according to appearance order in data
         ! k     is group number according to "actual group table"
         ! xg    is group info. of case
         ! x     is case data (with repeated cova. if relevant)
      ip = nint( frqcy )
      k  = gnum(ko)
      do i = 1, ntrt
         write(iulid,*) ip, (real(x(j)),j=i,iv+i,ntrt), kase,  &
                        (gstr(k,j),j=1,ngfac), (tstr(i,j),j=1,ntfac), 1
      end do
   end do
   call chkpg( 3, .false., newp )
   write(imp,333) lidfil(:lenf), iulid, nu
   lines = lines + 3

   INCLUDE 'paclidf.'

END SUBROUTINE paclid
