SUBROUTINE pacpry( iout, lchk, name, ymat, nrow, ncol, nrop, lab, ordc )

!***********************************************************************
!     - pacpry -
!     PRints transpose of "Y"-statistics matrix for variables and groups
!     the rows of the matrix are printed in columns and conversely
!     (especially appropriate when NCW is one).
!     May be used in a more general manner to print the transpose of
!     a matrix whose rows correspond to variables
!***********************************************************************
!
!     IOUT  - INPUT . Unit number for printing                - INTEGER
!     LCHK  - INPUT . If .false. no call to CHKPG             - LOGICAL
!     NAME  - INPUT . Names of NROP first YMAT rows           - CHAR
!                     (assumed 8 characters long)
!     YMAT  - INPUT . The matrix to be printed                - REAL*8
!     NROW  - INPUT . Number of rows(variables)in YMAT matrix - INTEGER
!     NCOL  - INPUT . Number of columns(groups)in YMAT matrix - INTEGER
!     NROP  - INPUT . Number of rows to be printed: only the  - INTEGER
!                     NROP first ones are printed
!     LAB   - INPUT . Array of (group) labels of length NCOL  - CHAR
!                     (assumed 8 characters long)
!     ORDC  - INPUT . Array associating relevant label in LAB - INTEGER
!                     to printed row: LAB(ORDC(j)) is label
!                     for jth printed row (usually ORDC is
!                     sub-array SUPG from MEM2 when columns
!                     of YMAT, thus rows printed, are groups)
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG
!     External functions called:
!        NONE
!     Fortran  functions called:
!        MAX  MIN
!
!-----------------------------------------------------------------------

   USE gmpint

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: iout, nrow, ncol, nrop
   LOGICAL, INTENT(IN) :: lchk
   REAL(KIND=8), INTENT(IN) :: ymat(nrow,ncol)
   INTEGER, INTENT(IN) :: ordc(ncol)
   CHARACTER(LEN=*), INTENT(IN) :: name(nrop), lab(ncol)

   !  Local declarations
   !  ------------------

   INTEGER :: i, l, linpp, ll, maxil, nbeg, nend, nladd, nlr, nmor
   LOGICAL :: more, newp

   !--------------------------------------------------------------------

      ! Maximum number of items per line (format G14, 10 col. skipped)
   maxil = (lrecp-10-1) / 14

   if ( lchk ) then
      linpp = linep
   else
      linpp = 99999
   end if

   more = .false.
   nmor = 2
   nend = 0
   do
      nbeg = nend + 1
      nend = min( nend+maxil, nrop )
      if ( lchk ) call chkpg( nmor+1, .false., newp )
      if ( more ) write(iout,111) ' '
      write(iout,111) (name(i),i=nbeg,nend)
      write(iout,111) ('--------',i=nbeg,nend)
      lines = lines + nmor
         ! nlr  = # of lines still to be printed
         ! nladd= # of lines in current page
      nlr = ncol
      ll = 0
      do
         nladd = min( nlr, max( linpp-lines, 1 ) )
         do l = 1, nladd
            ll = ll + 1
            write(iout,222) lab(ordc(ll)), (ymat(i,ll),i=nbeg,nend)
         end do
         lines = lines + nladd
         nlr = nlr - nladd
         if ( nlr <= 0 ) exit
         if ( lchk ) call chkpg( 0, .true., newp)
      end do

      more = .true.
      nmor = 3
      if ( nend >= nrop ) exit
   end do

   111 format(10x,10(2x,a8,4x))
   222 format(1x,a8,1x,10g14.7)

END SUBROUTINE pacpry
