SUBROUTINE pacz( iuw, qcst, bcst, sumv, ius, jus, nsuj, nssav, zbar, eign, ierr )

!***********************************************************************
!     - pacz -
!     directs computation and printing for the "data reduction" step
!     ("essential" contrasts and/or discriminant variables), and saves
!     the "reduced" protocol on user-save-file if required
!     Called only if the comparison is product form and no covariates
!***********************************************************************
!
!     IUW   - INPUT . Unit number where multivariate results  - INTEGER
!                     are stored
!     QCST  - INPUT . Df of the joined comparison, i.e. sum   - REAL*8
!                     over G-comparison-support of (count-1)
!     BCST  - INPUT . A scalar such that  V'*D*V = BCST*Iv    - REAL*8
!                     where D is a diagonal matrix with
!                     1/AG(.) in diagonal, and Iv is the
!                     identity matrix of order NCV
!     SUMV  - INPUT . Whether zero or one depending on the    - REAL*8
!                     fact that the coefficients of the linear
!                     form(s) in V sum to zero or not
!     IUS   - INPUT . Unit number for data reading            - INTEGER
!     JUS   - INPUT . Unit number for temporary storage of    - INTEGER
!                     derived protocol (only used if printout
!                     but no user's save file)
!     NSUJ  - INPUT . Number of cases on input                - INTEGER
!     NSSAV - OUTPUT. Number of cases saved                   - INTEGER
!     ZBAR  - OUTPUT. Array of means of the                   - REAL*8
!                     "essential/discriminant" variables
!     EIGN  - OUTPUT. Array of names of eigenvalues/vectors   - CHAR
!     IERR  - OUTPUT. Error code: 0 if none                   - INTEGER
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  EIGVCT  MABC  MIKABC  MSBC  MTABC  MTASAZ  PACDPC
!        PACPRY  PRTSYM  PRTTAM
!     External functions called:
!        DFMTC
!     Fortran  functions called:
!        ABS  LEN_TRIM  MAX  MIN  NINT  PACK  SQRT
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE gmplgl
   USE memst2
   USE memstb
   USE pacchr
   USE pacint
   USE paclgl

   IMPLICIT NONE

   !  Arguments
   !  ---------

   REAL(KIND=8), INTENT(IN) :: qcst, bcst, sumv
   INTEGER, INTENT(IN) :: iuw, ius, jus, nsuj
   INTEGER, INTENT(OUT) :: nssav, ierr
   REAL(KIND=8), INTENT(OUT) :: zbar(ranka,ngsup)
   CHARACTER(LEN=nocw), INTENT(OUT) :: eign(ranka)

   !  Local declarations
   !  ------------------

   CHARACTER(LEN=6), EXTERNAL :: dfmtc

   REAL(KIND=8), PARAMETER :: zero=0.0_8, one=1.0_8, epsl=1.0e-12_8
      ! epsl is relative error allowed on eigenvalues
   REAL(KIND=8) :: cst, cstqb, pct, rho, scl, sumx, temp, tolz, tot, trace, tracx
   LOGICAL :: allg, canon, newp
   CHARACTER(LEN=8 ) :: crho
   CHARACTER(LEN=54) :: fmt="(1X,A8,' |',      ,2('|',F7.2),'|',A8,'|',A1,'|',25A1)"
   CHARACTER(LEN=74) :: undln="=========================================================================="
   INTEGER :: i, iout, irank, is, j, k, l, lcmpt, nbeig, nladd, nlr, ny1
   REAL(KIND=8) :: dcal(ncv*nwdc)
   REAL(KIND=8) :: pcal(nwdc*ranka), pval(ranka), pvec(nwdc*ranka), vess(ngrp*ranka)
   REAL(KIND=8) :: core(max(ncv*(ncv+1)/2,nwdct)), wrka(max(ncv*(ncv-1)/2,1))
   REAL(KIND=8) :: ysm1(max(nwdct,ntdc*ranka,ncv*ranka))
   INTEGER :: kpvt(max(ncv-1,1))
   CHARACTER(LEN=nocw) :: vtn(ntdc)

   INCLUDE 'paczp.'

   !--------------------------------------------------------------------
      ! NOTE. Currently this reduction step is not allowed when
      ! covariates are used. Therefore nwdc is here equivalent to
      ! ncw*ndep (and nvtot to ndep)

   nssav = 0
   ierr  = 0

   lcmpt = min( lrecp-1, len_trim(cmptit) )
   nbeig = ranka
   iout  = imp

   call chkpg( 1+4+2+5, .false., newp )
   if ( newp ) then
      if ( .not. linter ) then
         write(imp,555) cmptit(:lcmpt)
         write(imp,555) '  '
         lines = lines + 2
      end if
   else
      write(imp,555) '  '
      lines = lines + 1
   end if
   write(imp,222)
   lines = lines + 4

      ! Recover information from iuw:
      ! ysm1  is the square root of the inverse of ys2
      ! dcal  is the matrix of calibrated effect
      ! The matrix diagonalized was dcal'*dcal
      ! If maximum rank of the matrix is 1 then the unique element was
      ! stored, otherwise following informations are available:
      ! i     is the order of the matrix (= nwdc)
      ! irank is the maximum rank of the matrix
      ! ierr  is the return code from diagonalization
      ! trace is the trace of the matrix
      ! tracx is the trace of the matrix computed from its eigenvalues
      ! tolz  is a zero-tolerance for eigenvalues
      ! pval  is the vector of eigenvalues
      ! pvec  is the matrix of eigenvectors (in columns)

      ! pcal  will contain calibrated egeinvectors (ysm1*pvec)

   rewind iuw
   read(iuw) ysm1(1:nwdct)
   read(iuw) dcal
   if ( nwdc > 1 ) then
      read(iuw) i, irank, ierr, trace, tracx, tolz
      if ( ierr /= 0 ) then
         write(imp,333) errdia, ierr
         lines = lines + 2
         write(lgu,*) ermrk1
         write(lgu,333) errdia, ierr
         write(lgu,*) ermrk2
         return
      end if
         ! Check the computed rank is correct
      if ( ranka > irank .or. abs(trace-tracx) > tolz ) then
         write(imp,111) irank, ranka, trace, tracx
         write(imp,2222)
         lines = lines + 5
         write(lgu,*) ermrk1
         write(lgu,111) irank, ranka, trace, tracx
         write(lgu,2222)
         write(lgu,*) ermrk2
         ierr = 1
         return
      end if
      read(iuw) pval
      read(iuw) pvec
      call msbc( ysm1(1:nwdct), pvec, pcal, nwdc, nbeig )
   else
      read(iuw) trace
      pval(1) = trace
      pvec(1) = one
      pcal(1) = ysm1(1)
      tolz = trace * epsl / nwdc
   end if

      ! Create names of eigenvalues/vectors (in eign)
   do i = 1, nbeig
      write(eign(i),444) i
   end do

   ! Proceede on variables * T set
   ! -----------------------------
   ny1 = ntdc*nbeig
      ! Compute essential contrasts in ysm1 and print
   call mikabc( w, pvec, ysm1(1:ny1), ndep, ntrt, ncw, nbeig )
      ! Create names
   if ( ndep > 1 .and. ntrt > 1 ) then
      k = 1
      do i = 1, ndep
         vtn(k) = nomv(i)
         k = k + 1
         do j = 2, ntrt
            write(vtn(k),666) '...o', j, ' '
            k = k + 1
         end do
      end do
   else if ( ntrt == 1 ) then
      vtn(1:ndep) = nomv(1:ndep)
   else if ( ndep == 1 ) then
      vtn(1:ntrt) = labt
   end if
   write(imp,555) msgdis, derstd, undln(:  zdis+zder+zstd)
   lines = lines + 2
   call prttam( iout, .true., ysm1(1:ny1), ntdc, ntdc, nbeig, vtn, eign )

      ! Compute calibrated essential contrasts in ysm1 and print
   call mikabc( w, pcal, ysm1(1:ny1), ndep, ntrt, ncw, nbeig )
   call chkpg( 1+2+3, .false., newp )
   if ( newp ) then
      if ( .not. linter ) then
         write(imp,555) cmptit(:lcmpt)
         write(imp,555) '  '
         lines = lines + 2
      end if
   else
      write(imp,555) '  '
      lines = lines + 1
   end if
   write(imp,555) msgdis, dercal, undln(:zdis+zder+zcal)
   lines = lines + 2
   call prttam( iout, .true., ysm1(1:ny1), ntdc, ntdc, nbeig, vtn, eign )

      ! Save "reduced" protocol if requested (use ysm1)
   if ( lderiv .or. listdp ) then
      call pacdpc( ius, jus, gnum, gsup, ysm1(1:ny1), nbeig, nsuj, nssav )
   end if

      ! Compute and print correlations between original variables
      ! (including W-contrasts) and discriminant variables
      ! Compute covariances (ys2*pcal), store in ysm1
   ny1 = nwdc*nbeig
   call msbc( ys2, pcal, ysm1(1:ny1), nwdc, nbeig )
      ! Compute correlations
   k = 0
   do i = 1, nwdc
      k = k + i
      sumx = sqrt( ys2(k) )
      if ( sumx > zero ) then
         do j = i, ny1, nwdc
            ysm1(j) = ysm1(j) / sumx
         end do
      else
         do j = i, ny1, nwdc
            ysm1(j) = 9.0_8
         end do
      end if
   end do
      ! Print the correlation matrix
   if ( ncw > 1 ) then
         ! Create names
      k = 1
      do i = 1, ndep
         vtn(k) = nomv(i)
         k = k + 1
         do j = 2, ncw
            write(vtn(k),666) '...w', j, ' '
            k = k + 1
         end do
      end do
   else
      vtn(1:ndep) = nomv(1:ndep)
   end if
   call chkpg( 1+2+3, .false., newp )
   if ( newp ) then
      if ( .not. linter ) then
         write(imp,555) cmptit(:lcmpt)
         write(imp,555) '  '
         lines = lines + 2
      end if
   else
      write(imp,555) '  '
      lines = lines + 1
   end if
   write(imp,555) msgdis, msgcvo, undln(:zdis+zcvo)
   lines = lines + 2
   call prttam( iout, .true., ysm1(1:ny1), nwdc, nwdc, nbeig, vtn(1:nwdc), eign )

   ! Proceede on G set
   ! -----------------
   if ( ncv > 1 ) then
      ny1 = ncv*nbeig
         ! Create the core matrix (dcal*dcal'), store in core
!!!   call matas( dcal, core, ncv, nwdc )
!!!      This is the expansion of subroutine matas
      l = (nwdc-1)*ncv
      is = 1
      do i = 1, ncv
         do j = 1, i
            sumx = zero
            do k = 0, l, ncv
               sumx = sumx + dcal(i+k) * dcal(j+k)
            end do
            core(is) = sumx
            is = is + 1
         end do
      end do

         ! Get eigenvectors of core in ysm1 (eigenvalues already known)
      call eigvct( ncv, nbeig, core, pval, ysm1(1:ny1), wrka, kpvt, ierr )
      if ( ierr /= 0 ) then
         call chkpg( 2, .false., newp )
         write(imp,333) erregv, ierr
         lines = lines + 2
         write(lgu,*) ermrk1
         write(lgu,333) erregv, ierr
         write(lgu,*) ermrk2
         return
      end if
         ! Compute "essential" contrasts (v*ysm1), store in vess
      call mabc( v, ysm1(1:ny1), vess, ngrp, ncv, nbeig )

         ! Print coefficients
      call chkpg( 1+2+3, .false., newp )
      if ( newp ) then
         if ( .not. linter ) then
            write(imp,555) cmptit(:lcmpt)
            write(imp,555) '  '
            lines = lines + 2
         end if
      else
         write(imp,555) '  '
         lines = lines + 1
      end if
      write(imp,555) msgder, msggrp, undln(:zder+zgrp)
      lines = lines + 2
      call prttam( iout, .true., vess, ngrp, ngrp, nbeig, labg, eign )

   else
         ! "Essential" contrasts are merely original ones
      vess(1:ngrp) = pack( v, mask=.true. )

   end if

      ! Reduce vess to the G-support (for latter use)
   k = 1
   is = 1
   do j = 1, nbeig
      do i = 1, ngrp
         if ( gsup(i) > 0 ) then
            vess(is) = vess(k)
            is = is + 1
         end if
         k = k + 1
      end do
   end do

   ! Compute means of calibrated "discriminant/essential" variables
   ! --------------------------------------------------------------
   call mtabc( pcal, ybar, zbar, nwdc, nbeig, ngsup )

   ! Compute and print statistics of derived protocol (if requested)
   ! ---------------------------------------------------------------
   if ( levout(2) ) then

      if ( levout(3) ) then
            ! Print zbar
         call chkpg( 1+2+3, .false., newp )
         if ( newp ) then
            if ( .not. linter ) then
               write(imp,555) cmptit(:lcmpt)
               write(imp,555) '  '
               lines = lines + 2
            end if
         else
            write(imp,555) '  '
            lines = lines + 1
         end if
         write(imp,777)
         lines = lines + 2
         call pacpry( iout, .true., eign, zbar,       &
                      nbeig, ngsup, nbeig, labg, supg )
      end if

         ! Compute "essential" calibrated effect in eff
         ! (it should be a diagonal matrix, but it is computed here
         ! as a general one, thus providing a check of numerical
         ! accuracy)
      call mabc( zbar, vess, eff, nbeig, ngsup, nbeig )
         ! Print it
      call chkpg( 1+2+1, .false., newp )
      if ( newp ) then
         if ( .not. linter ) then
            write(imp,555) cmptit(:lcmpt)
            write(imp,555) '  '
            lines = lines + 2
         end if
      else
         write(imp,555) '  '
         lines = lines + 1
      end if
      write(imp,888)
      lines = lines + 2
      call prttam( iout, .true., eff, nbeig, nbeig, nbeig, eign, eign )

   end if

   if ( levout(5) .or. levout(7) ) then
         ! The within covariance matrix is pcal'*ys2*pcal (in core)
         ! It should result in the identity matrix
      call mtasaz( pcal, ys2, core, nwdc, nbeig )
         ! Print it
      call prtsym( iout, .true., core, nbeig, eign, .false., msgcov, zcov )
   end if

   ! Print eigenvalues and canonical correlations
   ! --------------------------------------------
   call chkpg( 1+3+min(nbeig,3), .false., newp )
   if ( newp ) then
      if ( .not. linter ) then
         write(imp,555) cmptit(:lcmpt)
         write(imp,555) '  '
         lines = lines + 2
      end if
   else
      write(imp,555) '  '
      lines = lines + 1
   end if
   write(imp,999)
   write(imp,1111)
   write(imp,999)
   lines = lines + 3
   cst = one / ncv
   cstqb = qcst * bcst
   trace = cst * trace
   pct = 1.0e2_8 / trace
      ! Get nice format for eigenvalues
   fmt(13:18) = dfmtc( 10, -6, trace, tolz )
      ! Get scale factor for histogram of eigenvalues (at most 25
      ! characters per eigenvalue so that the line printed will not
      ! exceed 79 columns, see variable format fmt)
   scl = 25.0_8 / (cst*pval(1))
      ! If the linear form in V is not a contrast, the canonical
      ! correlation is undefined
   canon = sumv == zero
   if ( .not. canon ) crho = undef
   allg = ncv == ngsup-1
   temp = gcount
   nlr = nbeig
   j = 0
   sumx = zero
   do
      nladd = min( nlr, max( linep-lines, 1 ) )
      do i = 1, nladd
         j = j + 1
         if ( canon ) then
            if ( allg ) then
               rho = sqrt( pval(j) / (cstqb+pval(j)) )
            else
                  ! Compute sum of squares for overall Gsup-comparison
               rho = zero
               tot = zero
               is = 0
               do l = 1, ngsup
                  k = supg(l)
                  tot = tot + ag(k)*zbar(j,l)
                  rho = rho + ag(k)*zbar(j,l)*zbar(j,l)
                  is = is + nbeig
               end do
               rho = sqrt( pval(j)/(cstqb+bcst*(rho-tot*tot/temp)) )
            end if
            write(crho,'(f8.5)') rho
         end if
         pval(j) = cst * pval(j)
         k = 1 + nint( scl*pval(j) )
         sumx = sumx + pval(j)
         write(imp,fmt) eign(j), pval(j), pval(j)*pct, sumx*pct, crho, ('*',l=1,k)
      end do
      lines = lines + nladd
      nlr = nlr - nladd
      if ( nlr <= 0 ) exit
      call chkpg( 0, .true., newp )
   end do
   if ( nbeig > 1 ) then
      call chkpg( 2, .false., newp )
      write(imp,999)
      write(imp,fmt) trahdr, trace
      lines = lines + 2
   end if

   INCLUDE 'pacmv2f1.'
   INCLUDE 'paczf.'

END SUBROUTINE pacz
