PROGRAM pac

!! USE proc_pac   ! The module that contains all procedures for PAC

!***********************************************************************
!
!            P A C - Programme d'Analyse des Comparaisons
!
!                    Methode   : Bruno   LECOUTRE
!                    Programme : Jacques POITEVINEAU
!
!
!                  LAM/LCPE - UMR 7604 CNRS UPMC Ministere de la Culture
!                  11 rue de Lourmel
!                  75015 PARIS
!                  FRANCE
!
!***********************************************************************
!
!     Main program:
!     Initialization of some module items
!     (unit numbers and terminal characteristics)
!
!***********************************************************************
!!!   INCLUDE 'gmpnotes.'
!***********************************************************************
!!!   INCLUDE 'gmpcall.'
!***********************************************************************
!!!   INCLUDE 'gmptree.'
!***********************************************************************
!
!     IMPORTANT GENERAL PARAMETERS
!
!        Alphabetic list:
!
!     HDRGMP , MAXAT  , MAXOP  , MAXVA  , INCAR  , INCDW  , NIDW   ,
!     NIRW   , NLIW   , NOCW   , KODTXT , NBREOP
!
!        General section subroutines in which they appear:
!
!     HDRGMP               : GMPGFP , GMPIN1 , GMPIPB , GMPPRO ,
!                            GMPPRP , GMPPRX , GMPSAV
!     INCAR                : GMPADS , GMPCAT , GMPCMD , GMPCOM ,
!                            GMPDIR , GMPIN1 , GMPOBJ , GMPPRO ,
!                            GMPSAV , GMPXIN , NAMAB  ,
!                            + most of ANACOM routines
!     INCDW                : GMPIN1 , GMPPRD
!     NOCW                 : GMPDIR , GMPEXT , GMPINI ,
!                            GMPPRX , TRANSA , TRANSE
!     NIDW , NIRW , NLIW   : GMPIN1 , TRANSA , TRANSE
!     MAXOP, MAXVA, MAXAT  : FORMEV , GMPIN1
!     KODTXT               : GMPCAT , GMPPRD , GMPUVN , GMPXIN
!     NBREOP               : BOUM   , FORMEV , GMPDIR
!
!-----------------------------------------------------------------------

!***********************************************************************
!
!     GENERAL MODULES
!
!        Alphabetic list:
!
!     CARL   , GMPCHR , GMPINT , GMPLGL , MEMST1 ,
!     MEMSTA , OPLHN  , OPPRM  , SPCNOM , SPCVAL ,
!     SYSTM  , TERMNL , TRANCB
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE gmplgl
   USE pgmgmp
   USE systm
   USE termnl

!***********************************************************************
!!!   INCLUDE 'gmpcbt.'
!***********************************************************************
!!!   INCLUDE 'pacnotes.'
!***********************************************************************
!!!   INCLUDE 'paccall.'
!***********************************************************************
!!!   INCLUDE 'pactree.'
!***********************************************************************
!
!     MODULES SPECIFIC TO PAC
!
!        Alphabetic list:
!
!     MEMST2 , MEMSTB , PACCHR , PACDTR , PACINT ,
!     PACLET , PACLGL , PACRNK
!
!-----------------------------------------------------------------------

   USE pacint

!***********************************************************************
!!!   INCLUDE 'paccbt.'
!-----------------------------------------------------------------------

!***********************************************************************
!
!     SCRATCH UNITS
!     -------------
!     Only IU1 is used in general section (to store relevant data),
!     and is referred to by GMPIN1 and GMPXIN
!     All are used in PAC-specific section:
!     IU1     is used by  PACSTX
!     IU2     is used by  PACDPA  PACDPC  PACIN1  PACLID  PACPLA  PACSTX
!                         PACY    PACZ
!     IU3     is used by  PACCST  PACIN1  PACY
!     IU4     is used by  PACANV  PACIN1  PACPPM  PACUVT  PACY    POLREG
!     JUNIT1  is used by  PACDPA  PACDPB  PACDPL  PACFOR  PACMVT  PACMV2
!                         PACY    PACZ
!     JUNIT2  may be  by  PACDPA  PACDPB  PACDPL  PACY
!     JUNIT3  may be  by  PACDPB  PACDPL  PACY
!     PACANV  uses        IU4
!     PACCST  uses        IU3
!     PACDPA  uses        IU2    JUNIT1/JUNIT2
!     PACDPB  uses        JUNIT1/JUNIT2/JUNIT3
!     PACDPC  uses        IU2
!     PACDPL  uses        JUNIT1/JUNIT2/JUNIT3
!     PACFOR  uses        JUNIT1
!     PACIN1  uses        IU2    IU3    IU4
!     PACLID  uses        IU2
!     PACMVT  uses        JUNIT1
!     PACMV2  uses        JUNIT1
!     PACPLA  uses        IU2
!     PACPPM  uses        IU4
!     PACSTX  uses        IU1    IU2
!     PACUVT  uses        IU4
!     PACY    uses        IU2    IU3    IU4    JUNIT1/JUNIT2/JUNIT3
!     PACZ    uses        IU2    JUNIT1
!     POLREG  uses        IU4
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Interface block
   !  ---------------

   INTERFACE
      SUBROUTINE pac999( u1, u2, u3 )
         IMPLICIT NONE
         INTEGER, INTENT(IN), OPTIONAL :: u1, u2, u3
      END SUBROUTINE pac999
   END INTERFACE

   !  Local declarations
   !  ------------------

   INTEGER :: kterm, lterm

   INTEGER :: i, j                ! Lahey
   LOGICAL :: alloc=.true.        ! Lahey
   CHARACTER(LEN=127) :: cmdlin   ! Lahey

   !--------------------------------------------------------------------

      ! Set up system dependent parameters ( systm )
      ! ----------------------------------
   lunix = .false.  ! non-UNIX
!! lunix = .true.   ! UNIX

      ! Protect against underflows (Lahey)                  ! Lahey
      ! ----------------------------------                  ! Lahey
   call undfl( alloc )                                      ! Lahey

      ! Get command line and see if batch run is specified  ! Lahey
      ! --------------------------------------------------  ! Lahey
   call getcl( cmdlin )                                     ! Lahey
   call cvuct( cmdlin )                                     ! Lahey
   i = index( cmdlin, 'BAT' )                               ! Lahey
   linter = i <= 0                                          ! Lahey
   if ( .not. linter ) then                                 ! Lahey
      if ( i > 1 ) then                                     ! Lahey
         if ( cmdlin(i-1:i-1) /= ' ' ) linter = .true.      ! Lahey
      end if                                                ! Lahey
      if ( i < 125 ) then                                   ! Lahey
         if ( cmdlin(i+3:i+3) /= ' ' ) linter = .true.      ! Lahey
      end if                                                ! Lahey
   end if                                                   ! Lahey

   lterm = 25    !!! PC !!!
   kterm = 80    !!! PC !!!

      ! lterm is the number of lines of the terminal from which the
      ! program is called in case of interactive run and
      ! kterm is its number of columns

      ! INPUT/OUTPUT CHARACTERISTICS ( TERMNL )
      ! ----------------------------
      ! ncsco & ncprt must be at least 77 because of outputs in gmpprp,
      ! but 79 is safer, as maybe required in program-specific section

      ! Screen characteristics (no. of lines & columns on input/output)
      ! (N.B. these variables may be modified in gmpin1)
   nlscr  = lterm
   ncsci  = kterm
   ncsco  = ncsci - 1
   nprmpt = 2

      ! Printer characteristics (number of lines and columns)
   nlprt  =  55  !!! PC !!!
   ncprt  =  80  !!! PC !!!


      ! INPUT/OUTPUT UNITS ( TERMNL, GMPINT, PACINT )
      ! ------------------

   lec1   =  5   ! Unit number for standard input of commands
   lec2   =  7   ! Unit number for alternate input of commands
      ! (Maximum record length allowed for these 2 units above is 80)

   imp    =  6   ! Standard output unit number
   lgu    = 97   ! Log file unit number
   inpux  = 10   ! Default unit number for input data file
   ksave  =  9   ! Default unit number for save file

   iu1    =  1   ! Unit number for scratch file
   iu2    =  2   ! Unit number for scratch file
   iu3    =  3   ! Unit number for scratch file
   iu4    =  4   ! Unit number for scratch file

   kbayes = 90   ! Default unit number for "Bayes" save-file
   kderiv = 80   ! Default unit number for "derived protocol" save-file

   junit1 = 91   ! Additional scratch unit number
   junit2 = 92   ! Additional scratch unit number
   junit3 = 93   ! Additional scratch unit number
   !>>>>>>> 99 >>! N.B. Unit number 99 is reserved for Eyelid save-file

      ! SET PROGRAM PAGE HEADER ( GMPCHR )
      ! -----------------------

   pgmhdr = hdrgmp // 'PAC '
   pgmhdr = 'PAC    '

   if ( lgu /= imp ) then
      open(lgu,file='!PAC.log',form='FORMATTED',status='UNKNOWN', &
           position='REWIND',                                     &
           carriagecontrol='FORTRAN')                              ! Lahey
   end if

      ! See if input/output files specified in command line        ! Lahey
   i = index( cmdlin, 'IN=' )                                      ! Lahey
   if ( i > 0 ) then                                               ! Lahey
      i = i + 3                                                    ! Lahey
      j = index( cmdlin(i:), ' ' ) + i - 1                         ! Lahey
      if ( j == i ) then                                           ! Lahey
         write(lgu,*) ermrk1                                       ! Lahey
         write(lgu,101) 'in='                                      ! Lahey
         write(lgu,*) ermrk2                                       ! Lahey
         stop                                                      ! Lahey
      end if                                                       ! Lahey
      if ( j < i ) j = len( cmdlin )                               ! Lahey
      linter = .false.                                             ! Lahey
      lec1   = 95                                                  ! Lahey
      open(lec1,file=cmdlin(i:j),form='FORMATTED',status='OLD',   &! Lahey
           position='REWIND',                                     &! Lahey
           action='READ')                                          ! Lahey
      !    action='READ,DENYNONE')                                 ! Lahey
      ! Lahey77: action='acces mode,sharing mode'                  ! Lahey
   end if                                                          ! Lahey
   i = index( cmdlin, 'OUT=' )                                     ! Lahey
   if ( i > 0 ) then                                               ! Lahey
      i = i + 4                                                    ! Lahey
      j = index( cmdlin(i:), ' ' ) + i - 1                         ! Lahey
      if ( j == i ) then                                           ! Lahey
         write(lgu,*) ermrk1                                       ! Lahey
         write(lgu,101) 'out='                                     ! Lahey
         write(lgu,*) ermrk2                                       ! Lahey
         stop                                                      ! Lahey
      end if                                                       ! Lahey
      if ( j < i ) j = len( cmdlin )                               ! Lahey
      imp = 96                                                     ! Lahey
      open(imp,file=cmdlin(i:j),form='FORMATTED',status='UNKNOWN',&! Lahey
           position='REWIND',                                     &! Lahey
           carriagecontrol='FORTRAN')                              ! Lahey
   end if                                                          ! Lahey

      ! START
      ! -----

   call pac001   ! pac001 may be viewed as a "pseudo main"

      ! COMPLETE: DEALLOCATE ARRAYS AND CLOSE FILES
      ! -------------------------------------------

   call gmp999
   call pac999( 99 )   ! Unit 99 is Eyelid save-file

   INCLUDE 'mainf.'

END PROGRAM pac

!!MODULE proc_pac

!!   ! Insert all procedures into one module in order to provide an
!!   ! explicit interface for calling procedures.

!!   CONTAINS

   INCLUDE 'pac_proc.lst'

!!END MODULE proc_pac

! Modules needed by PAC:
   INCLUDE 'carl.'
   INCLUDE 'flag_msg.'
   INCLUDE 'gmpchr.'
   INCLUDE 'gmpint.'
   INCLUDE 'gmplgl.'
   INCLUDE 'is_digit.'
   INCLUDE 'memst1.'
   INCLUDE 'memsta.'
   INCLUDE 'oplhn.'
   INCLUDE 'opprm.'
   INCLUDE 'pgmgmp.'
   INCLUDE 'spcnom.'
   INCLUDE 'spcval.'
   INCLUDE 'systm.'
   INCLUDE 'termnl.'
   INCLUDE 'trancb.'
   INCLUDE 'memst2.'
   INCLUDE 'memstb.'
   INCLUDE 'pacchr.'
   INCLUDE 'pacdtr.'
   INCLUDE 'pacint.'
   INCLUDE 'paclet.'
   INCLUDE 'paclgl.'
   INCLUDE 'pacrnk.'
